@@ -114,11 +114,11 @@ extern char *rfc822date __((time_t *timep));
 extern int  hdr_status __((const char *cp, const char *lbuf, int n, int octo));
 #ifdef Z_TOKEN_H
 extern u_long _hdr_compound __((const char *cp, long n, int cstart, int cend,
-				TokenType type, struct token *tp,
-				struct token **tlist, struct token **tlistp));
-extern const char *_unfold __((const char *start, const char *end, struct token *t));
-extern struct token * scan822 __((const char **cpp, size_t n, int c1, int c2,
-				  int allowcomments, struct token **tlistp));
+				TokenType type, token822 *tp,
+				token822 **tlist, token822 **tlistp));
+extern const char *_unfold __((const char *start, const char *end, token822 *t));
+extern token822 * scan822 __((const char **cpp, size_t n, int c1, int c2,
+				  int allowcomments, token822 **tlistp));
 #endif
 
 /* selfaddrs.c */
@@ -152,18 +152,18 @@ extern int strmatch __((const char *pattern, const char *term));
 
 /* token.c */
 #ifdef Z_TOKEN_H
-extern struct token *makeToken __((const char *s, u_int n));
-extern struct token *copyToken __((struct token *t));
-extern const char *formatToken __((struct token *t));
+extern token822 *makeToken __((const char *s, u_int n));
+extern token822 *copyToken __((token822 *t));
+extern const char *formatToken __((token822 *t));
 extern int  printToken  __((char *buf, char *eob,
-			    struct token *t, struct token *tend,
+			    token822 *t, token822 *tend,
 			    int quotespecials));
 extern int  printdToken __((char **bufp, int *buflenp,
-			    struct token *t, struct token *tend,
+			    token822 *t, token822 *tend,
 			    int quotespecials));
-extern int  fprintToken __((FILE *fp, struct token *t, int onlylength));
-extern int  fprintFold  __((FILE *fp, struct token *t, int col));
-extern void freeTokens __((struct token *t, int memtype));
+extern int  fprintToken __((FILE *fp, token822 *t, int onlylength));
+extern int  fprintFold  __((FILE *fp, token822 *t, int col));
+extern void freeTokens __((token822 *t, int memtype));
 #ifdef TOKENLEN /* Defined in  "mailer.h", like AddrComponent too.. */
 extern const char *formatAddr __((AddrComponent d));
 #endif
