@@ -39,8 +39,9 @@ struct address {
 struct rcpt {
 	struct rcpt	*next;
 	struct address	*addr;		/* addr.link is the sender address */
-	const char	*orcpt;		/* DSN ORCPT= string */
-	const char	*notify;	/* DSN NOTIFY= flags */
+	const char	*orcpt;		/*  DSN  ORCPT=  string */
+	const char	*inrcpt;	/* "DSN" INRCPT= string */
+	const char	*notify;	/*  DSN  NOTIFY= flags  */
 	int		notifyflgs;
 	char		***newmsgheader; /* message header line pointer ptr
 					   that points to an address of
@@ -159,8 +160,8 @@ extern int optind;
 extern int embytes;
 extern int emcalls;
 extern unsigned emsleeptime;
-/* extern univptr_t emalloc __((size_t len));
-   extern univptr_t erealloc __((univptr_t buf, size_t len)); */
+extern void * emalloc  __((size_t len));
+extern void * erealloc __((void * buf, size_t len));
 #ifdef S_IWUSR /* Defined on <sys/stat.h> */
 extern int efstat __((int fd, struct stat *stbuf));
 #endif
