@@ -32,8 +32,8 @@
 	}
 #define SIGNAL_HOLD(SIG) \
 	sigblock(sigmask(SIG))
-#define SIGNAL_RELEASE(OLDMASK) \
-	sigsetmask(OLDMASK)
+#define SIGNAL_RELEASE(SIG) \
+	sigsetmask(~sigmask(SIG) & sigblock(SIGNAL_HOLD(sig)))
 #define SIGNAL_IGNORE(SIG) \
 	SIGNAL_HANDLE(SIG,SIG_IGN)
 
@@ -41,18 +41,21 @@
 
 #if	defined(SA_NOCLDSTOP)||defined(SA_ONSTACK)||defined(SA_RESTART)
 	/* ================ POSIX.1 STUFF ================ */
+#ifndef SA_NODEFER
+# define SA_NODEFER 0
+#endif
 #ifdef SA_INTERRUPT
 #define SIGNAL_HANDLE(X,Y) \
 	{	struct sigaction act;		\
 		act.sa_handler = Y;		\
-		act.sa_flags   = SA_INTERRUPT;	\
+		act.sa_flags   = SA_INTERRUPT|SA_NODEFER; \
 		sigemptyset(&act.sa_mask);	\
 		sigaction (X, &act, NULL);	\
 	}
 #define SIGNAL_HANDLESAVE(X,Y,OLD) \
 	{	struct sigaction act, oact;	\
 		act.sa_handler = Y;		\
-		act.sa_flags   = SA_INTERRUPT;	\
+		act.sa_flags   = SA_INTERRUPT|SA_NODEFER; \
 		sigemptyset(&act.sa_mask);	\
 		sigemptyset(&oact.sa_mask);	\
 		sigaction (X, &act, &oact);	\
@@ -78,14 +81,14 @@
 #define SIGNAL_HANDLE(X,Y) \
 	{	struct sigaction act;		\
 		act.sa_handler = Y;		\
-		act.sa_flags   = 0;		\
+		act.sa_flags   = SA_NODEFER;	\
 		sigemptyset(&act.sa_mask);	\
 		sigaction (X, &act, NULL);	\
 	}
 #define SIGNAL_HANDLESAVE(X,Y,OLD) \
 	{	struct sigaction act, oact;	\
 		act.sa_handler = Y;		\
-		act.sa_flags   = 0;		\
+		act.sa_flags   = SA_NODEFER;	\
 		sigemptyset(&act.sa_mask);	\
 		sigemptyset(&oact.sa_mask);	\
 		sigaction (X, &act, &oact);	\
