@@ -4,10 +4,8 @@
 
 #include "hostenv.h"
 #include "mailer.h"
-
-extern char	*progname;
-extern int	errno;
-extern char	*strerror();
+#include "libz.h"
+#include "libc.h"
 
 /* for statistics in router/allocate.c */
 extern int embytes;
