@@ -3,8 +3,11 @@
  *	This will be free software, but only when it is finished.
  */
 
+#include "hostenv.h"
 #include "mailer.h"
-#include <signal.h> /* used for kill(2) */
+#include "zmsignal.h" /* used for kill(2) */
+#include "libz.h"
+#include "libc.h"
 
 #ifndef	_IOFBF
 #define	_IOFBF	0
@@ -15,12 +18,11 @@
 int
 killprevious(sig, pidfile)
 	int sig;
-	char *pidfile;
+	const char *pidfile;
 {
 	int c, rc;
 	FILE *fp;
 	char *path, buf[128];
-	extern char *postoffice;
 
 	path = emalloc((unsigned)(strlen(postoffice) + strlen(pidfile) + 2));
 	sprintf(path, "%s/%s", postoffice, pidfile);
