@@ -7,6 +7,7 @@
 #include "mailer.h"
 #include <fcntl.h>
 #include "zmsignal.h"
+#include "libz.h"
 
 /*
  * Reopen the log files, done at daemon startup, and when you want to roll over
@@ -44,6 +45,6 @@ loginit(sig)
 #endif	/* F_SETFL */
 	setvbuf(stdout, (char *)NULL, _IOLBF, 0);
 	setvbuf(stderr, (char *)NULL, _IOLBF, 0);
-	SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE (*)()) loginit);
+	SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE (*) __((int))) loginit);
 	return 0;
 }
