@@ -138,9 +138,9 @@ _hdr_compound(cp, n, cstart, cend, type, tp, tlist, tlistp)
 	long	n;
 	int	cstart, cend;
 	TokenType	type;
-	struct token	*tp, **tlist, **tlistp;
+	token822	*tp, **tlist, **tlistp;
 {
-	struct token *tn;
+	token822 *tn;
 
 	if (*cp != cstart)
 		abort(); /* Sanity check!  Call fault! */
@@ -195,7 +195,7 @@ nextline:
 const char *
 _unfold(start, end, t)
 	const char *start, *end;
-	struct token *t;
+	token822 *t;
 {
 	char *s, *cp;
 	/*
@@ -237,16 +237,16 @@ _unfold(start, end, t)
  * for efficiency this was changed to avoid function call overhead. The tokens
  * returned are classified by type (TokenType enum class).
  */
-struct token * scan822(cpp, nn, c1, c2, allowcomments, tlistp)
+token822 * scan822(cpp, nn, c1, c2, allowcomments, tlistp)
 	const char **cpp;		/* pointer to pointer to text */
 	size_t	nn;			/* number of characters to scan */
 	int	c1, c2;			/* temporary specials */
 	int allowcomments;		/* #prefix tokens are comments to EOT */
-	struct token **tlistp;		/* continuation line tokens if any */
+	token822 **tlistp;		/* continuation line tokens if any */
 {
 	register const char *cp;
-	static struct token  t;
-	struct token	*tlist, *tp, *tn, *ot;
+	static token822  t;
+	token822	*tlist, *tp, *tn, *ot;
 	char	msgbuf[50];
 	short	ct, sc1, sc2;
 	int n = (int) nn;
