@@ -10,8 +10,8 @@
  * configurer to do the right thing, and list them at the ZENV file
  * SELFADDRESSES= -entry  as a string of style:
  *
- *	"[1.2.3.4],[6.7.8.9],[ipv6.::1.2.3.4],my.domain.name,"
- *              ^-------^---------^----------  commas to separate them!
+ *    "[1.2.3.4],[6.7.8.9],[IPv6:::ffff:1.2.3.4],my.domain.name"
+ *              ^---------^---------------------^---- commas to separate them!
  *
  */
 
@@ -290,7 +290,7 @@ stashmyaddress(host)
 #endif
 
 #if defined(AF_INET6) && defined(INET6)
-	  if (strncasecmp(host,"[ipv6.",6)==0) {
+	  if (strncasecmp(host,"[IPv6:",6)==0) {
 	    af = AF_INET6;
 	    addrsiz = IN6ADDRSZ;
 	    rc = inet_pton(af, host+6, &au.ia6);
@@ -427,6 +427,8 @@ matchmyaddress(sa)
 {
 	int i;
 
+	if (!myaddrs)
+		stashmyaddresses(NULL);
 	if (!myaddrs) return 0; /* Don't know my addresses ! */
 	
 	/* Match loopback net.. */
@@ -503,7 +505,7 @@ char *argv[];
 #if defined(AF_INET6) && defined(INET6)
     case AF_INET6:
       inet_ntop(AF_INET6, &((struct sockaddr_in6 **) sa)[i]->sin6_addr, buf, sizeof(buf));
-      printf("IPv6: [ipv6.%s]\n");
+      printf("IPv6: [IPv6:%s]\n");
       break;
 #endif
     default:
