@@ -13,7 +13,7 @@ You should have received a copy of the license with this software. If
 you didn't get a copy, you may request one from <license@inner.net>.
 
 */
-/* getaddrinfo() v1.22 */
+/* getaddrinfo() v1.22; v1.26, v1.27(w/o debug code) */
 
 /* To do what POSIX says, even when it's broken, define: */
 /* #define BROKEN_LIKE_POSIX 1 */
@@ -47,13 +47,15 @@ you didn't get a copy, you may request one from <license@inner.net>.
 #endif
 
 #include <netdb.h>
-#ifndef EAI_AGAIN
+#if !defined(EAI_AGAIN) || !defined(AI_NONAME)
 #include "netdb6.h"
 #endif
 
 #include <arpa/nameser.h>
 #include <resolv.h>
 
+extern int h_errno;
+
 #ifndef AF_LOCAL
 #define AF_LOCAL AF_UNIX
 #endif /* AF_LOCAL */
@@ -69,6 +71,10 @@ you didn't get a copy, you may request one from <license@inner.net>.
 #include <ctype.h>
 #include "libc.h"
 
+#ifndef T_AAAA
+# define T_AAAA 28 /* IPv6 address record.  Codeless w/o INET6 */
+#endif
+
 #define GAIH_OKIFUNSPEC 0x0100
 #define GAIH_EAI        ~(GAIH_OKIFUNSPEC)
 
@@ -200,6 +206,9 @@ build:
 #ifndef HFIXEDSZ
 #define HFIXEDSZ 12
 #endif
+#ifndef RRHEADER_SZ
+#define RRHEADER_SZ 10
+#endif
 
 static int resolver_lookup_addr __((const char *name, int type,
 				    const struct addrinfo *req,
@@ -254,7 +263,8 @@ FILE *vlog;
       fprintf(vlog,"res_search() yielded answer with too small reply: %d ( < 10 )\n", answerlen);
     return -(EAI_FAIL);
   } else {
-    HEADER *h = (HEADER *)p;
+    HEADER *h = (HEADER *)p; /* This is aligned block, anything after this
+				may be nonaligned */
     qdcount = ntohs(h->qdcount);
     answers = ntohs(h->ancount);
     if (!h->qr || (h->opcode != QUERY) || (qdcount != 1) || !answers) {
@@ -265,6 +275,7 @@ FILE *vlog;
   p += HFIXEDSZ;
 
   dn[0] = 0;
+  /* Question playback analysis */
   for (; qdcount > 0; --qdcount) {
     int qt, qc;
     i = dn_expand((void*)answer, (void*)ep, (void*)p, dn, sizeof(dn));
@@ -287,11 +298,14 @@ FILE *vlog;
       return -(EAI_FAIL);
     }
 #endif
+    if (qc != C_IN)
+      return -(EAI_FAIL);
   }
 
   if (vlog)
     fprintf(vlog,"resolver(): question skipped, dn='%s', first answer at p=%d  answers=%d\n", dn, (p-answer), answers);
 
+  /* Answer analysis */
   for ( ; answers > 0; --answers) {
     i = dn_expand((void*)answer, (void*)ep, (void*)p, dn, sizeof(dn));
     if (i < 0) {
@@ -302,7 +316,7 @@ FILE *vlog;
     }
     p += i;
 
-    if (p + 10 > ep) { /* Too little data! */
+    if (p + RRHEADER_SZ > ep) { /* Too little data! */
       if (vlog) fprintf(vlog, "resolver(): Out of data in reply [1]\n");
       return -(EAI_FAIL);
     } else {
@@ -339,10 +353,8 @@ FILE *vlog;
 
 	  (*pat)->family = AF_INET;
 	  break;
+
 #if defined(INET6) && defined(AF_INET6)
-#ifndef T_AAAA
-# define T_AAAA 28 /* IPv6 address record */
-#endif
         case T_AAAA:
 	  if (i != 16) {
 	    if (vlog) fprintf(vlog, "eaifail@%s:%d; AAAA size = %d != 16\n",__FILE__,__LINE__, i);
@@ -442,7 +454,7 @@ FILE *vlog;
     }
   } else {
     if (!tmpnam(saun->sun_path))
-      return -(EAI_SYSTEM);
+      return (-(EAI_SYSTEM) | GAIH_OKIFUNSPEC);
   }
   if (req->ai_flags & AI_CANONNAME) {
     (*pai)->ai_canonname = ((char *)(*pai) + sizeof(struct addrinfo) +
@@ -654,7 +666,10 @@ build:
 	if (at2->cname != NULL)
 	  j = strlen(at2->cname) + 1;
 	else
-	  j = strlen(name) + 1;
+	  if (name)
+	    j = strlen(name) + 1;
+	  else
+	    j = 2;
       } else
 	j = 0;
 
@@ -716,7 +731,7 @@ build:
 	      prevcname = at2->cname;
 	    }
 	  } else
-	    strcpy((*pai)->ai_canonname, name);
+	    strcpy((*pai)->ai_canonname, name ? name : "*");
 	}
 
 	pai = &((*pai)->ai_next);
@@ -795,17 +810,22 @@ FILE *vlog;
   if (!req)
     req = &nullreq;
 
-  if (req->ai_flags & ~3)
+  if (req->ai_flags & ~(AI_CANONNAME | AI_PASSIVE | AI_NONAME))
     return EAI_BADFLAGS;
 
+#if BROKEN_LIKE_POSIX
   if ((req->ai_flags & AI_CANONNAME) && !name)
     return EAI_BADFLAGS;
+#endif
 
   if (service && *service) {
+    char *c;
     gaih_service.name = (void *)service;
-    gaih_service.num  = atol(gaih_service.name);
+    gaih_service.num = strtoul(service, &c, 10);
+    if (*c)
+      gaih_service.num = -1;
 #if BROKEN_LIKE_POSIX
-    if (gaih_service.num != -1)
+    else
       if (!req->ai_socktype)
 	return EAI_SERVICE;
 #endif /* BROKEN_LIKE_POSIX */
