@@ -45,7 +45,7 @@ static char rcsid[] = "$Id: inet_ntop.c,v 8.5 1996/05/22 04:56:30 vixie Exp $";
  */
 
 static const char *inet_ntop4 __((const u_char *src, char *dst, size_t size));
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 static const char *inet_ntop6 __((const u_char *src, char *dst, size_t size));
 #endif
 
@@ -57,6 +57,9 @@ static const char *inet_ntop6 __((const u_char *src, char *dst, size_t size));
  * author:
  *	Paul Vixie, 1996.
  */
+
+const char * inet_ntop __((int, const void *, char *, size_t));
+
 const char *
 inet_ntop(af, src, dst, size)
 	int af;
@@ -67,7 +70,7 @@ inet_ntop(af, src, dst, size)
 	switch (af) {
 	case AF_INET:
 		return (inet_ntop4(src, dst, size));
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	case AF_INET6:
 		return (inet_ntop6(src, dst, size));
 #endif
@@ -106,7 +109,7 @@ inet_ntop4(src, dst, size)
 	return (dst);
 }
 
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 
 #ifndef IN6ADDRSZ
 #define IN6ADDRSZ 16
