@@ -48,7 +48,7 @@ static char rcsid[] = "$Id: inet_pton.c,v 8.6 1996/06/26 23:17:26 vixie Exp $";
  */
 
 static int	inet_pton4 __((const char *src, u_char *dst));
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 static int	inet_pton6 __((const char *src, u_char *dst));
 #endif
 
@@ -63,6 +63,9 @@ static int	inet_pton6 __((const char *src, u_char *dst));
  * author:
  *	Paul Vixie, 1996.
  */
+
+int inet_pton __((int, const char *, void *));
+
 int
 inet_pton(af, src, dst)
 	int af;
@@ -72,7 +75,7 @@ inet_pton(af, src, dst)
 	switch (af) {
 	case AF_INET:
 		return (inet_pton4(src, dst));
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	case AF_INET6:
 		return (inet_pton6(src, dst));
 #endif
@@ -133,7 +136,8 @@ inet_pton4(src, dst)
 	return (1);
 }
 
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
+
 /* int
  * inet_pton6(src, dst)
  *	convert presentation level address to network order binary form.
