@@ -549,10 +549,11 @@ mail_close_alternate(fp,where,suffix)
 		eno = errno;
 		fprintf(stderr, "link(\"%s\", \"%s\"): errno %d\n",
 				message, nmessage, errno);
-		mail_free(message);
 		mail_free(nmessage);
+		unlink(message); /* Throw the file away */
+		mail_free(message);
 		errno = eno;
-		return -1;
+		return -2;
 	}
 	mail_free(nmessage);
 	unlink(message);
