@@ -43,6 +43,7 @@ static char sccsid[] = "@(#)syslog.c	5.28 (Berkeley) 6/27/90";
 
 #include <stdio.h>
 #include "hostenv.h"
+#ifndef HAVE_SYSLOG /* If not have system defined syslog, use this! */
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <fcntl.h>
@@ -62,17 +63,16 @@ static char sccsid[] = "@(#)syslog.c	5.28 (Berkeley) 6/27/90";
      * is in syslogd.c */
 #define	_PATH_LOGNAME	"/dev/log"
 #endif
-#define index		strchr
 
 #define	_PATH_CONSOLE	"/dev/console"
 
 static int	LogFile = -1;		/* fd for log */
-static int	connected;		/* have done connect */
+static int	connected = 0;		/* have done connect */
 static int	LogStat = 0;		/* status bits, set by openlog() */
 static char	*LogTag = "syslog";	/* string to tag the entry with */
 static int	LogFacility = LOG_USER;	/* default facility code */
 
-extern char *index(), *strcpy(), *strncpy(), *strcat();
+/* extern char *strchr(), *strcpy(), *strncpy(), *strcat(); */
 /* =()<extern @<SIZE_T>@	strlen();>()= */
 /* extern int	strlen(); */
 
@@ -173,7 +173,7 @@ error:error:error!  No vsprintf() NOR vsnprintf() !
 		return;
 	(void)strcat(tbuf, "\r\n");
 	cnt += 2;
-	p = index(tbuf, '>') + 1;
+	p = strchr(tbuf, '>') + 1;
 	(void)write(fd, p, cnt - (p - tbuf));
 	(void)close(fd);
 }
@@ -213,6 +213,7 @@ static struct sockaddr SyslogAddr;	/* AF_UNIX address of local logger */
 /*
  * OPENLOG -- open system log
  */
+void
 openlog(ident, logstat, logfac)
 	char *ident;
 	int logstat, logfac;
@@ -225,7 +226,7 @@ openlog(ident, logstat, logfac)
 	if (LogFile == -1) {
 		SyslogAddr.sa_family = AF_UNIX;
 		strncpy(SyslogAddr.sa_data, _PATH_LOGNAME,
-		    sizeof(SyslogAddr.sa_data));
+			sizeof(SyslogAddr.sa_data));
 		if (LogStat & LOG_NDELAY) {
 			LogFile = socket(AF_UNIX, SOCK_DGRAM, 0);
 			fcntl(LogFile, F_SETFD, 1);
@@ -239,7 +240,7 @@ openlog(ident, logstat, logfac)
 /*
  * CLOSELOG -- close the system log
  */
-closelog()
+void closelog()
 {
 	(void) close(LogFile);
 	LogFile = -1;
@@ -250,7 +251,7 @@ static int	LogMask = 0xff;		/* mask of priorities to be logged */
 /*
  * SETLOGMASK -- set the log mask level
  */
-setlogmask(pmask)
+int setlogmask(pmask)
 	int pmask;
 {
 	int omask;
@@ -260,3 +261,5 @@ setlogmask(pmask)
 		LogMask = pmask;
 	return (omask);
 }
+
+#endif
