@@ -58,9 +58,10 @@ static char sccsid[] = "@(#)herror.c	8.1 (Berkeley) 6/4/93";
 static char rcsid[] = "$Id: herror.c,v 8.2 1995/06/19 08:35:01 vixie Exp $";
 #endif /* LIBC_SCCS and not lint */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/uio.h>
-#include <netdb.h>
+#include "netdb.h"
 #if defined(BSD) && (BSD >= 199103)
 # include <unistd.h>
 # include <string.h>
@@ -78,6 +79,7 @@ const char *h_errlist[] = {
 int	h_nerr = { sizeof h_errlist / sizeof h_errlist[0] };
 
 extern int	h_errno;
+extern const char *hstrerror __P((int err));
 
 /*
  * herror --
