@@ -718,7 +718,7 @@ sh_include(argc, argv)
 	}
 	close(fd);
 	buf[(u_int)stbuf.st_size] = '\0';
-	return eval(buf, argv[1], rpath);
+	return eval(buf, argv[1], rpath, &stbuf);
 }
 
 static int
