@@ -27,6 +27,8 @@
 #include "io.h"
 #include "shconfig.h"
 
+#include "libsh.h"
+
 
 #ifdef  HAVE_WAITPID
 # include <sys/wait.h>
@@ -53,27 +55,6 @@ extern int wait();
 #define	SIGCHLD	SIGCLD
 #endif	/* SIGCHLD */
 
-extern FILE *runiofp;
-extern RETSIGTYPE (*orig_handler[])();
-extern char *path_hash();
-extern char *progname;
-extern char *strerror();
-extern char *strsignal();
-extern char *strnsave();
-extern int addbuffer __((u_char*, int, int, struct osCmd*));
-extern int errno;
-extern int interrupted;
-extern int runio __((struct IOop **));
-extern int sh_builtin();
-extern struct codedesc *interpret();
-extern struct conscell *v_find();
-extern void trapexit(), v_sync();
-extern void v_purge(), v_export(), functype(), jc_newproc();
-#ifdef	MAILER
-extern struct conscell **return_valuep;
-extern int D_functions, funclevel;
-#endif	/* MAILER */
-
 /* The parent function of whatever is about to be executed, for $@ access */
 
 struct osCmd *globalcaller;
@@ -114,6 +95,7 @@ STATIC status_t reapstatus;		/* status of last reaped process */
  * for exit status of a pipeline.
  */
 
+STATIC void reapchild __((int));
 STATIC void
 reapchild(sig)
 int sig;
@@ -137,8 +119,8 @@ int sig;
 	}
 }
 
-
-int
+STATIC int bgsetup __((struct osCmd *));
+STATIC int
 bgsetup(c)
 	struct osCmd *c;
 {
@@ -167,20 +149,22 @@ execute(c, caller, oretcode, name)
 	struct osCmd	*c,		/* function we are about to execute */
 			*caller;	/* the function we are currently in */
 	int		oretcode;	/* default value for return code */
-	char		*name;		/* Redundant function name info */
+	const char	*name;		/* Redundant function name info */
 {
 	int ac = 0;			/* argument count in av[] */
-	char **av = NULL;		/* argument vector, with ac entries */
+	const char **av = NULL;		/* argument vector, with ac entries */
 	struct sslfuncdef *sfdp;	/* defined function descriptor */
 	int pid;			/* process id of child (unix program) */
 	int retcode;			/* the new integer-valued return code */
 	int nofork;			/* flag: don't fork shell before exec */
 	status_t status;		/* return code from wait() */
 	int n;
-	register struct conscell *sl, *l;
-	struct conscell *tmp;
-	RETSIGTYPE (*oquit_handler)(), (*oint_handler)(), (*oterm_handler)();
-	RETSIGTYPE (*ochld_handler)();
+	register conscell *sl, *l;
+	conscell *tmp;
+	RETSIGTYPE (*oquit_handler) __((int)),
+	  (*oint_handler) __((int)),
+	  (*oterm_handler) __((int));
+	RETSIGTYPE (*ochld_handler) __((int));
 
 	status = 0;
 
@@ -194,8 +178,7 @@ execute(c, caller, oretcode, name)
 		if (car(c->argv) == NULL || LIST(car(c->argv)))
 			c->argv = NULL;
 		else
-			functype(car(c->argv)->string,
-				    &c->shcmdp, (struct sslfuncdef *)NULL);
+			functype(car(c->argv)->string, &c->shcmdp, NULL);
 	}
 	if (c->argv && LIST(c->argv) && STRING(car(c->argv))) {
 		/* there are string arguments */
@@ -204,8 +187,7 @@ execute(c, caller, oretcode, name)
 			fprintf(runiofp, "Run:");
 		}
 		if (c->shcmdp == NULL) {
-			functype((car(c->argv))->string,
-				 (struct shCmd *)NULL, &sfdp);
+			functype((car(c->argv))->string, NULL, &sfdp);
 			if (sfdp == NULL)
 				path_hash((car(c->argv))->string);
 		}
@@ -223,9 +205,9 @@ execute(c, caller, oretcode, name)
 					ac++;
 			/* allocate space */
 #ifdef	x_USE_ALLOCA
-			av = (char **)alloca(ac * sizeof (char *));
+			av = (const char **)alloca(ac * sizeof (char *));
 #else
-			av = (char **)tmalloc(ac * sizeof (char *));
+			av = (const char **)tmalloc(ac * sizeof (char *));
 #endif
 			/* set them up */
 			ac = 0;
@@ -281,12 +263,13 @@ execute(c, caller, oretcode, name)
 		/*
 		 * Builtin command
 		 */
-		if (c->shcmdp->lptr != NULL) {
+		if (c->shcmdp->lptr != NULL ||
+		    c->shcmdp->rptr != NULL) {
 			/*
 			 * List-valued builtin
 			 */
 			*((int *)&status) = 0;
-			if (c->prev == NULL || c->prev->shcmdp->lptr == NULL) {
+			if (c->prev == NULL || c->prev->shcmdp->sptr != NULL) {
 				/* translate stdin into a list argument */
 				if (c->shcmdp->flag & SH_STDIN)
 					sl = s_read(stdin);
@@ -310,7 +293,10 @@ execute(c, caller, oretcode, name)
 				fflush(stderr);
 			}
 #endif	/* MAILER */
-			c->rval = (c->shcmdp->lptr)(c->argv, sl, &retcode);
+			if (c->shcmdp->rptr != NULL)
+			  c->rval = (c->shcmdp->rptr)(c->argv, sl, &retcode);
+			else
+			  c->rval = (c->shcmdp->lptr)(c->argv, sl);
 			/* it's important that c->rval is NOT permanent mem. */
 			if (retcode == NO_RETCODE)
 				retcode = (c->rval == NULL);
@@ -321,7 +307,9 @@ execute(c, caller, oretcode, name)
 					if (STRING(tmp))
 						tmp->flags |= QUOTEDSTRING;
 			}
-			if (c->next != NULL && c->next->shcmdp->lptr != NULL) {
+			if (c->next != NULL &&
+			    (c->next->shcmdp->lptr != NULL ||
+			     c->next->shcmdp->rptr != NULL)  ) {
 				;/* next command takes list input, keep mum! */
 			} else if ((c->fdmask & (1<<1)) && _FILEIO(stdout)) {
 				/* stdout is a pipe to another command */
@@ -489,7 +477,7 @@ execute(c, caller, oretcode, name)
 		/*
 		 * Go fish
 		 */
-		execvp(av[0], av);
+		execvp(av[0], (char*const*)av);
 
 		if (!nofork && c->undoio)
 			RUNIO(c->undoio);
@@ -590,35 +578,19 @@ execute(c, caller, oretcode, name)
 	return retcode;
 }
 
-/*
- * Read a byte-stream from fd into a linked list of buffers in command->buffer.
- */
-
-STATIC void
-readstring(fd, command)
-	int fd;
-	struct osCmd *command;
-{
-	int n, state;
-	u_char buf[BUFSIZ];	/* read this size chunk at a time */
-
-	state = 0;
-	while ((n = read(fd, &buf[0], sizeof(buf))) > 0)
-		state = addbuffer(&buf[0], n, state, command);
-}
-
-int
+static int addbuffer __((char *, int, int, struct osCmd *));
+static int
 addbuffer(buf, len, state, command)
-	u_char *buf;
+	char *buf;
 	register int	len, state;
 	struct osCmd *command;
 {
-	register u_char *ncp, *cp;
-	struct conscell *tmp;
+	register char *ncp, *cp;
+	conscell *tmp;
 
 	ncp = cp = buf;
 	for (cp = buf; --len >= 0; ++cp) {
-		u_char c = *cp;
+		int c = (*cp) & 0xFF;
 		if (!isascii(c))
 			*ncp++ = c, state = 1;
 		else if (state) {	/* break on whitespace */
@@ -632,10 +604,11 @@ addbuffer(buf, len, state, command)
 	}
 	/* wrap up the stuff we got so far into a string buffer */
 	if (ncp > buf) {
-		if (isascii(*(ncp-1)) && isspace(*(ncp-1)))
+		int c = ncp[-1] & 0xFF;
+		if (isascii(c) && isspace(c))
 			--ncp;
 		if (ncp > buf) {
-			cp = (u_char *)strnsave((char *)buf, ncp-buf);
+			cp = strnsave(buf, ncp-buf);
 			if (isset('I') || isset('R'))
 				fprintf(runiofp,
 					"readstring: '%s'\n", cp);
@@ -646,6 +619,26 @@ addbuffer(buf, len, state, command)
 	return state;
 }
 
+
+/*
+ * Read a byte-stream from fd into a linked list of buffers in command->buffer.
+ */
+
+STATIC void readstring __((int, struct osCmd *));
+STATIC void
+readstring(fd, command)
+	int fd;
+	struct osCmd *command;
+{
+	int n, state;
+	char buf[BUFSIZ];	/* read this size chunk at a time */
+
+	state = 0;
+	while ((n = read(fd, &buf[0], sizeof(buf))) > 0)
+		state = addbuffer(&buf[0], n, state, command);
+}
+
+
 /*
  * String Buffer routines (for lack of a better name).
  * We want a stack on each fd, with the TOS marking whether I/O should
@@ -657,7 +650,8 @@ addbuffer(buf, len, state, command)
  * source), essentially.
  */
 
-void
+STATIC void sb_push __((struct siobuf **, struct siobuf *));
+STATIC void
 sb_push(siopp, sioptop)
 	struct siobuf **siopp, *sioptop;
 {
@@ -685,7 +679,8 @@ sb_push(siopp, sioptop)
 
 /* read from string buffer by replacing new read buffer with old write buffer */
 
-int
+STATIC int sb_in __((int, int));
+STATIC int
 sb_in(n, outfd)
 	int n, outfd;
 {
@@ -716,7 +711,8 @@ sb_in(n, outfd)
 
 /* allocate a string buffer to write to */
 
-void
+STATIC void sb_out __((int));
+STATIC void
 sb_out(n)
 	int n;
 {
@@ -738,7 +734,8 @@ sb_out(n)
 
 /* free a string buffer */
 
-void
+STATIC void sb_free __((int));
+STATIC void
 sb_free(n)
 	int n;
 {
@@ -879,7 +876,7 @@ runio(ioopp)
 #ifdef	S_IFIFO
 		case sIOopenPortal:	/* open named pipe */
 			if (ioop->ioflags & O_CREAT) {
-				if (stat((char *)ioop->name, &stbuf) == 0
+				if (stat(ioop->name, &stbuf) == 0
 					&& !(stbuf.st_mode & S_IFIFO)) {
 					fprintf(stderr, "%s: %d %s\n",
 						progname, ioop->name,
@@ -887,9 +884,9 @@ runio(ioopp)
 					++errflag;
 					break;
 				}
-				unlink((void*)ioop->name);
+				unlink(ioop->name);
 				/* if the pipe already exists, don't bother */
-				if (mknod((void*)ioop->name, S_IFIFO, 0) < 0) {
+				if (mknod(ioop->name, S_IFIFO, 0) < 0) {
 					fprintf(stderr, "%s: %s %s\n", progname,
 							CANNOT_MKNOD,
 							ioop->name);
@@ -900,7 +897,8 @@ runio(ioopp)
 			/* FALL THROUGH */
 #endif	/* S_IFIFO */
 		case sIOopen:		/* open file */
-			if ((fd = open((char *)ioop->name, ioop->ioflags, smask)) < 0) {
+			fd = open(ioop->name, ioop->ioflags, smask);
+			if (fd < 0) {
 				fprintf(stderr, "%s: %s %s\n", progname,
 						CANNOT_OPEN, ioop->name);
 				++errflag;
@@ -938,7 +936,7 @@ runio(ioopp)
 			if (!errflag) {
 				BEGINGRANDCHILD(pid)
 					write(fd, ioop->name,
-						 strlen((char *)ioop->name));
+					      strlen(ioop->name));
 				ENDGRANDCHILD(pid) {		/* error */
 					fprintf(stderr, "%s: %s\n",
 							progname, CANNOT_FORK);
@@ -950,7 +948,7 @@ runio(ioopp)
 				fprintf(runiofp,
 					"write(%d, '%.40s', %d)\n",
 					ioop->fd, ioop->name,
-					strlen((char *)ioop->name));
+					strlen(ioop->name));
 			break;
 		case sIOopenPipe:
 			if (pipe(p) < 0) {
@@ -978,8 +976,7 @@ runio(ioopp)
 			if (isset('R'))
 				fprintf(runiofp, "intoBuffer '%s'\n",
 					ioop->command->buffer ?
-					ioop->command->buffer->string
-					: (u_char *)"");
+					ioop->command->buffer->string : "");
 			break;
 		case sIOdup:
 /*putc('\n', runiofp);*/
