@@ -101,18 +101,47 @@ dequote (str)
 {
 	int	    len;
 	const char *sp, *ep;
+	char *s, *s0 = NULL;
 
-	len = strlen(str);
-	sp = str;
-	ep = str + len-1;
+	do {
 
-	if ((*sp == '"' && *ep == '"') || (*sp == '\'' && *ep == '\'')) {
-		sp++;
-		ep--;
-		len -= 2;
-	}
+	  len = strlen(str);
+	  if (s0 == NULL)
+	    s0 = tmalloc(len+1); /* All subsequent runs will be smaller,
+				    AND they fit running in-place! */
+	  sp = str;
+	  ep = str + len -1;
 
-	return (strnsave(sp, len));
+	  if (len > 1 && *sp == *ep && 
+	      (*sp == '"' || *sp == '\'')) {
+	    ++sp;
+	    --ep;
+	    len -= 2;
+
+	    for (s = s0; len > 0; ++sp, --len, ++s) {
+	      if (*sp == '\\' && sp[1] != 0) {
+		*s = *++sp;
+		--len;
+	      } else
+		*s = *sp;
+	    }
+	    *s = 0;
+	  } else {
+	    if (s0 != sp) /* Don't copy, if 'in-place' */
+	      memcpy(s0, sp, len);
+	    s = s0;
+	    if (len > 0)
+		s += len;
+	    *s = 0;
+	  }
+
+	  ep = s-1;
+
+	  str = s0;
+
+	} while (ep > s0 && *s0 == *ep && (*s0 == '"' || *s0 == '\''));
+
+	return (s0);
 }
 
 STATIC void free_regexp __((regexp *prog));
@@ -714,7 +743,7 @@ assign(sl_lhs, sl_rhs, command)
 {
 	register conscell *s, *l, *tmp;
 	const char *varname;
-	int freerhs;
+	int freerhs = 0, sl_rhs_set;
 	memtypes oval;
 
 	oval = stickymem;
@@ -724,17 +753,31 @@ assign(sl_lhs, sl_rhs, command)
 	 * Variable values are really (potentially) lists.
 	 */
 	varname = sl_lhs->string;
-	l = s_copy_tree(sl_rhs);
-	if (l != NULL)
-	  sl_rhs = l;
 	s = v_find(varname);
+
 	if (s == NULL) {
+
+	  /* We don't know this variable, we create it */
+
+	  /* Following copy code was common, but separate locations
+	     are a lot easier to debug with DEC ATOM-tools */
+	  l = s_copy_tree(sl_rhs);
+	  if (l != NULL) {
+	    sl_rhs = l;
+	    sl_rhs_set = 1;
+	  } else
+	    sl_rhs_set = 0;
+
 	  /* create the variable in the global but non-exported scope */
 	  for (l = car(envarlist); cddr(l) != NULL; l = cdr(l))
 	    continue;
 	  /* l points at the next-to-last sublist of envarlist */
 	  if (isset('a'))
 	    l = cdr(l);	/* or the list of exports */
+	  if (sl_rhs_set && cdr(sl_rhs) != NULL) {
+	    s_free_tree(cdr(sl_rhs));
+	    sl_rhs_set = 0;
+	  }
 	  cdr(sl_rhs) = car(l);
 	  s = newstring(strsave(varname));
 	  s_set_prev(s, sl_rhs);
@@ -743,22 +786,38 @@ assign(sl_lhs, sl_rhs, command)
 	  car(l) = s;
 	  l = NIL;
 	  freerhs = 0;
+
 	} else {
+
+	  /* The variable exists, we replace the data content */
+
+	  /* Following copy code was common, but separate locations
+	     are a lot easier to debug with DEC ATOM-tools */
+	  l = s_copy_tree(sl_rhs);
+	  if (l != NULL) {
+	    sl_rhs = l;
+	    sl_rhs_set = 1;
+	  } else
+	    sl_rhs_set = 0;
+
 #if 1
 	  l = copycell(cdr(s));
 	  cdr(l) = NULL;
-	  /*
-	    if (command == NULL) {
+
+	  if (command == NULL) {
+	    /* We have no place to save the old content, thus
+	       we throw it now away. */
 	    if (LIST(cdr(s)))
-	    s_free_tree(cadr(s));
+	      s_free_tree(cadr(s));
 	    else
-	    free(cdr(s)->string);
-	    }
-	    */
+	      free(cdr(s)->string);
+	  }
+
 	  cdr(s)->flags = sl_rhs->flags;
 	  if (LIST(sl_rhs)) {
 	    cadr(s) = car(sl_rhs);
 	    s_set_prev(cdr(s), cadr(s));
+	    s_free_tree(cdr(sl_rhs));
 	  } else
 	    cdr(s)->string = sl_rhs->string;
 	  freerhs = 1;
@@ -770,6 +829,10 @@ assign(sl_lhs, sl_rhs, command)
 	  s_set_prev(s, sl_rhs);
 	  cdr(s) = sl_rhs;
 	  sl_rhs->pflags = 1;	/* cdr(sl_rhs->prev) == sl_rhs */
+	  if (sl_rhs_set && cdr(sl_rhs) != NULL) {
+	    s_free_tree(cdr(sl_rhs));
+	    sl_rhs_set = 0;
+	  }
 	  cdr(sl_rhs) = tmp;
 	  freerhs = 0;
 #endif
@@ -787,8 +850,10 @@ assign(sl_lhs, sl_rhs, command)
 	  s = conststring(varname);
 	  cdr(s) = command->envold;
 	  command->envold = s;
-	} else
-	  s_free_tree(l);	/* was free(l) */
+	} else {
+	  free(l); /* No location to save this -- not  s_free_tree() ?  */
+	}
+
 	stickymem = oval;
 	/* fvcache.namesymbol = 0; */
 	v_sync(varname);
@@ -806,7 +871,7 @@ assign(sl_lhs, sl_rhs, command)
 	}
 #endif	/* MAILER */
 	if (freerhs)
-	  free((char *)sl_rhs);
+	  free(sl_rhs); /* Not s_free_tree() ! */
 }
 
 /*
@@ -1118,15 +1183,15 @@ struct loopinfo {
 	short	varindex;	/* index of active loop variable */
 };
 
-STATIC struct token *tscanstring __((const char *));
+STATIC token822 *tscanstring __((const char *));
 STATIC
-struct token *
+token822 *
 tscanstring(s)
 	const char *s;
 {
 	const char *cp;
 	int len;
-	struct token *t;
+	token822 *t;
 
 	t = HDR_SCANNER(s);
 	if (t != NULL && t->t_next == NULL && t->t_type == String) {
@@ -1135,8 +1200,8 @@ tscanstring(s)
 	  const char *buf;
 	  len = TOKENLEN(t);
 	  buf = bp = (char *)tmalloc(len+1);
-	  for (cp = t->t_pname; cp - t->t_pname < len ; ++cp) {
-	    if (*cp == '\\' && cp - t->t_pname < len-1)
+	  for (cp = t->t_pname; (cp - t->t_pname) < len ; ++cp) {
+	    if ((*cp == '\\') && ((cp - t->t_pname) < len-1))
 	      *bp++ = *++cp;
 	    else
 	      *bp++ = *cp;
@@ -1889,8 +1954,8 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 				cdar(l) = d;
 				stickymem = stickytmp;
 				/* grindef("ARGV = ", caller->argv);
-				grindef("VARS = ", envarlist);
-				grindef("TMPO = ", l); */
+				   grindef("VARS = ", envarlist);
+				   grindef("TMPO = ", l);  */
 			} else {
 				fprintf(stderr, "parameter without call\n");
 				abort(); /* parameter without call! */
@@ -1898,8 +1963,8 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 			break;
 		case sAssign:
 		case sAssignTemporary:
-			if (command->argv == NULL
-					  || cdar(command->argv) == NULL)
+			if (command->argv == NULL ||
+			    cdar(command->argv) == NULL)
 				break;
 			for (d = caar(command->argv); cdr(d) != NULL; )
 				d = cdr(d);
@@ -1908,12 +1973,19 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 			  command->buffer = conststring(uBLANK);
 			if (nsift >= 0)
 			  v_accessed = sift[nsift].accessed;
+#if 0
 			assign(d, command->buffer,
-				cmd==sAssign ? (struct osCmd *)NULL : command);
+			       cmd==sAssign ? (struct osCmd *)NULL : command);
+#else
+			if (cmd == sAssign)
+			  assign(d, command->buffer, NULL);
+			else
+			  assign(d, command->buffer, command);
+#endif
 			/* do NOT reset *retcodep here */
 			command->bufferp = &command->buffer;
-			command->argv = NULL;
-			command->shcmdp = NULL;
+			command->argv    = NULL;
+			command->shcmdp  = NULL;
 			if (isset('I'))
 				grindef("Argv = ", command->argv);
 			break;
@@ -2015,13 +2087,13 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 				d = s_catstring(command->buffer);
 			else
 				d = command->buffer;
-			d = v_find(d->string);
+			d = v_expand(d->string, caller, *retcodep);
 			if (d == NULL)
 				pc = code + argi1 - 1;
 			break;
 		case sJumpIfOrValueNil:
 			if (d == NULL
-			    || (d = cdr(d)) == NULL
+			    || (cdr(d) != NULL && (d = cdr(d)) == NULL)
 			    || (STRING(d)
 				&& (d->string == NULL || *d->string == '\0'))
 			    || (LIST(d) && car(d) == NULL))
