@@ -23,8 +23,8 @@
 struct	siobuf {
 	struct siobuf	*_sb_data;	/* usually self pointer unless dup'ed */
 	int		_sb_cnt;	/* no. of bytes left in buffer */
-	u_char		*_sb_ptr;	/* current position in buffer */
-	u_char		*_sb_base;	/* base of buffer */
+	char		*_sb_ptr;	/* current position in buffer */
+	char		*_sb_base;	/* base of buffer */
 	int		_sb_bufsiz;	/* size of buffer */
 	short		_sb_flag;	/* open flags, O_CREAT == malloc'ed */
 	short		_sb_refcnt;	/* reference count */
@@ -42,21 +42,9 @@ extern struct siobuf *siofds[];		/* array of linked lists of siobufs */
 
 extern struct siobuf *sIOp;
 
+#define	_FILEIO(p)  (((sIOp = siofds[fileno(p)]) == NULL) || sIOp->sb_flag < 0)
 
 #ifndef	REALSTDIO
-#if 0
-#ifdef	fileno
-#undef	fileno
-#endif
-
-#ifdef _HPUX_SOURCE
-#define fileno(p)	((unsigned)(((p)->__fileL) + ((p)->__fileH << 8)))
-#else
-#define	fileno(p)	((unsigned)(p)->__file)
-#endif
-#endif
-
-#define	_FILEIO(p)   (((sIOp = siofds[fileno(p)]) == NULL) || sIOp->sb_flag < 0)
 
 #ifdef	putc
 #undef	putc
@@ -86,7 +74,7 @@ extern struct siobuf *sIOp;
 #endif	/* ungetc */
 #define	ungetc(c,p)	(_FILEIO(p) ? std_ungetc(c,p) : \
 			 (sIOp->sb_ptr > sIOp->sb_base ? \
-				    ++sIOp->sb_cnt, (int)(*--sIOp->sb_ptr = c) : EOF))
+			     ++sIOp->sb_cnt, (int)(*--sIOp->sb_ptr = c) : EOF))
 
 #ifdef	feof
 #undef	feof
@@ -106,27 +94,37 @@ extern struct siobuf *sIOp;
 
 #define	printf		(_FILEIO(stdout) ? std_printf : sioprintf)
 #define	fprintf		siofprintf
-#define	fscanf		siofscanf
+/* #define fscanf siofscanf */
 
-extern char	*std_gets(), *std_fgets();
-extern char	*siogets(), *siofgets();
-extern int	std_puts(), std_fputs();
-extern int	sioputs(), siofputs();
-extern int	siofread(), siofwrite();
-
-extern int	std_putc(), std_getc(), std_ungetc(), std_feof();
-extern int	std_fread(), std_fwrite();
-extern long	std_ftell();
-#ifdef HAVE_STDARG_H
-extern int	(*std_printf) __((char *fmt, ...));
-extern int	sioprintf  __((char *fmt, ...));
-extern int	siofprintf __((FILE *fp, char *fmt, ...));
-#else
-extern int	(*std_printf) __((char *fmt, ...));
-extern int	sioprintf  __((char *fmt, ...));
-extern int	siofprintf __((FILE *fp, char *fmt, ...));
-#endif
-extern int	siofscanf __(());
-extern int	siomore __(());
 #endif	/* REALSTDIO */
+
+extern char	*std_gets  __((char *));
+extern char	*std_fgets __((char *, u_int, FILE *));
+extern char	*siogets  __((char *));
+extern char	*siofgets __((char *, u_int, FILE *));
+extern int	std_puts  __((const char *));
+extern int	std_fputs __((const char *, FILE *));
+extern int	sioputs   __((const char *));
+extern int	siofputs  __((const char *, FILE *));
+extern int	siofread  __((char *, u_int, u_int, FILE *));
+extern int	siofwrite __((const char *, u_int, u_int, FILE *));
+
+extern int	std_putc __((int, FILE *));
+extern int	std_getc __((FILE *));
+extern int	std_ungetc __((int, FILE *));
+extern int	std_feof   __((FILE *));
+extern int	std_fread  __((char *, u_int, u_int, FILE *));
+extern int	std_fwrite __((const char *, u_int, u_int, FILE *));
+extern long	std_ftell  __((FILE *));
+#ifdef HAVE_STDARG_H
+extern int	(*std_printf) __((const char *fmt, ...));
+extern int	sioprintf  __((const char *fmt, ...));
+extern int	siofprintf __((FILE *fp, const char *fmt, ...));
+#else
+extern int	(*std_printf) __((const char *fmt, ...));
+extern int	sioprintf  __((const char *fmt, ...));
+extern int	siofprintf __((FILE *fp, const char *fmt, ...));
+#endif
+/* extern int siofscanf __(()); */
+extern int	siomore   __((struct siobuf *));
 #endif	/* Z_IO_H */
