@@ -10,8 +10,8 @@
 #ifdef	MAILER
 #include "sift.h"
 #endif	/* MAILER */
-#include "interpret.h"
 #include "listutils.h"
+#include "interpret.h"
 
 struct smallSymbol {
 	InputTokens	name;		/* token for 1 char of this kind */
@@ -29,7 +29,7 @@ extern struct smallSymbol shsymbol[];
 #define	BITSET(B,N)	(B[(N)/NBBY] |= (1<<((N)%NBBY)))
 #define	BITCLR(B,N)	(B[(N)/NBBY] &= ~(1<<((N)%NBBY)))
 
-extern u_char shfl[];
+extern char shfl[];
 
 #ifdef isset
 /* sometimes isset() is defined in <sys/param.h> */
@@ -40,7 +40,7 @@ extern u_char shfl[];
 
 struct cmddef {
 	OutputTokens	opcode;
-	char		*name;
+	const char	*name;
 	int		nargs;		/* > 0 is char *, < 0 is int */
 };
 
@@ -50,21 +50,18 @@ extern int ncommands;
 #define	TOKEN_NARGS(T)	commands[(int)(T)].nargs
 #define	TOKEN_NAME(T)	commands[(int)(T)].name
 
-extern unsigned char *SslWalker();
-extern unsigned char *optimize();
-
 struct sslfuncdef {
-	char		*name;		/* function name */
-	unsigned char	*pos;		/* code position */
-	unsigned char	*eot;		/* end of function code */
+	const char	*name;		/* function name */
+	const void	*pos;		/* code position */
+	const void	*eot;		/* end of function code */
 	struct codedesc	*tabledesc;	/* ptr to table containing its code */
-	char		*file;		/* file name defining function */
+	const char	*file;		/* file name defining function */
 	struct sslfuncdef *next;
 };
 
 struct codedesc {
-	unsigned char	*table;
-	unsigned char	*eotable;
+	const void	*table;
+	const void	*eotable;
 	struct sslfuncdef *functions;
 #ifdef	MAILER
 	/* Stringwise ...	*/
@@ -99,19 +96,19 @@ typedef enum {
  */
 
 struct osCmd {
-	struct conscell *argv;
-	struct conscell *envold;	/* old values of cmd-line var assigns */
-	struct conscell	*rval;		/* rval from list-valued command */
-	struct conscell	*buffer;	/* string buffer to build up words */
-	struct conscell	**bufferp;	/* pointer to cdr() of last buffer */
-	IOFlag		iocmd;		/* flag, see above */
-	int		fdmask;		/* low bits indicate io taken care of */
-	short		reaperTop;	/* cur top of list of active children */
-	short		flag;		/* miscellaneous flag bits, see below */
+	conscell *argv;
+	conscell *envold;	/* old values of cmd-line var assigns */
+	conscell *rval;		/* rval from list-valued command */
+	conscell *buffer;	/* string buffer to build up words */
+	conscell **bufferp;	/* pointer to cdr() of last buffer */
+	IOFlag	  iocmd;	/* flag, see above */
+	int	  fdmask;	/* low bits indicate io taken care of */
+	short	  reaperTop;	/* cur top of list of active children */
+	short	  flag;		/* miscellaneous flag bits, see below */
 	struct IOop	*doio;		/* list of I/O ops to do before cmd */
 	struct IOop	*execio;	/* I/O ops to do between fork/exec */
 	struct IOop	*undoio;	/* I/O ops to do in parent after cmd */
-	char		*memlevel;	/* caddr_t used for MEM_SHCMD stack */
+	void		*memlevel;	/* caddr_t used for MEM_SHCMD stack */
 	struct shCmd	*shcmdp;	/* shell command pointer */
 	struct sslfuncdef *sfdp;	/* S/SL function data pointer */
 	struct osCmd	*prev;		/* previous command if known */
