@@ -27,13 +27,13 @@ struct si_retab {
 	regexp		*rep;
 	tregexp		*trep;
 	const char	**match;
-	struct token	*startp[NSUBEXP];
-	struct token	*endp[NSUBEXP];
+	token822	*startp[NSUBEXP];
+	token822	*endp[NSUBEXP];
 };
 
 struct siftinfo {
 	int		kind;		/* 0: StringSift, !0: TokenSift */
-	struct token	*tlist;		/* token list for tsift expression */
+	token822	*tlist;		/* token list for tsift expression */
 	const char	*str;		/* string for sift expression */
 	struct vaccess	*accessed;	/* variables dependencies of expr. */
 	int		label;		/* label to go to when reevaluating */
