@@ -214,6 +214,7 @@ testeval(avp)
 {
 	register const char *cp;
 	int	i, value = -1;
+	const char *av1, *av2;
 
 	cp = **avp;
 	dprintf("testeval: %s (%d)\n", cp, value);
@@ -229,42 +230,93 @@ testeval(avp)
 			}
 		}
 	}
+	av1 = **avp;
+	av2 = *(*avp+1);
 	dprintf("    : %s (%d)\n", cp, value);
-	if (value < 0 && **avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "=") == 0) {
-		value = !strcmp(cp, *(*avp+1)), *avp += 2;
-	} else if (value < 0 && **avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "!=") == 0) {
-		value = strcmp(cp, *(*avp+1)), *avp += 2;
-	} else if (**avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "-eq") == 0) {
-		value = (N1 == N2), *avp += 2;
-	} else if (**avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "-ne") == 0) {
-		value = (N1 != N2), *avp += 2;
-	} else if (**avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "-gt") == 0) {
-		value = (N1 > N2), *avp += 2;
-	} else if (**avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "-ge") == 0) {
-		value = (N1 >= N2), *avp += 2;
-	} else if (**avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "-lt") == 0) {
-		value = (N1 < N2), *avp += 2;
-	} else if (**avp != NULL && *(*avp+1) != NULL
-		   && strcmp(**avp, "-le") == 0) {
+	if (value < 0 && av1 != NULL && av2 != NULL
+		   && strcmp(av1, "=") == 0) {
+		value = !strcmp(cp, av2);
+		*avp += 2;
+	} else if (value < 0 && av1 != NULL && av2 != NULL
+		   && strcmp(av1, "!=") == 0) {
+		value = strcmp(cp, av2);
+		*avp += 2;
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-eq") == 0) {
+		value = (N1 == N2);
+		*avp += 2;
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-ne") == 0) {
+		value = (N1 != N2);
+		*avp += 2;
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-gt") == 0) {
+		value = (N1 > N2);
+		*avp += 2;
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-ge") == 0) {
+		value = (N1 >= N2);
+		*avp += 2;
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-lt") == 0) {
+		value = (N1 < N2);
+		*avp += 2;
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-le") == 0) {
 		value = (N1 <= N2), *avp += 2;
-	} else if (**avp == NULL) {
+
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-nt") == 0) {
+
+	  struct stat stb1, stb2;
+	  if (stat(cp,  &stb1) != 0 ||
+	      stat(av2, &stb2) != 0) {
+	    fprintf(stderr,"%s: either '%s' or '%s' stat failed\n",
+		    testprogname, cp, av2);
+	    return -1;
+	  }
+	  value = (stb1.st_mtime > stb2.st_mtime);
+	  *avp += 2;
+
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-ot") == 0) {
+
+	  struct stat stb1, stb2;
+	  if (stat(cp,  &stb1) != 0 ||
+	      stat(av2, &stb2) != 0) {
+	    fprintf(stderr,"%s: either '%s' or '%s' stat failed\n",
+		    testprogname, cp, av2);
+	    return -1;
+	  }
+	  value = (stb1.st_mtime < stb2.st_mtime);
+	  *avp += 2;
+
+	} else if (av1 != NULL && av2 != NULL
+		   && strcmp(av1, "-ef") == 0) {
+
+	  struct stat stb1, stb2;
+	  if (stat(cp,  &stb1) != 0 ||
+	      stat(av2, &stb2) != 0) {
+	    fprintf(stderr,"%s: either '%s' or '%s' stat failed\n",
+		    testprogname, cp, av2);
+	    return -1;
+	  }
+	  value = ((stb1.st_ino == stb2.st_ino) &&
+		   (stb1.st_dev == stb2.st_dev));
+	  *avp += 2;
+
+
+	} else if (av1 == NULL) {
 		fprintf(stderr, "%s: argument expected\n", testprogname);
 		return -1;
-	} else if (*(*avp+1) == NULL) {
+	} else if (av2 == NULL) {
 		fprintf(stderr, "%s: unknown operand '%s'\n", testprogname, cp);
 		return -1;
 	} else
 		value = (*cp != '\0');
 gotvalue:
 #if 0
-	if (**avp != NULL && ***avp == ']' && *(**avp+1) == '\0')
+	if (av1 != NULL && av1[0] == ']' && av1[1] == '\0')
 		++*avp;
 #endif
 	return value != 0;
