@@ -232,6 +232,8 @@ v_expand(s, caller, retcode)
 			*cp++ = '-';
 		*cp = '\0';
 		return newstring(strsave(np));
+	default:
+		break;
 	}
 
 	if ((d = v_find(s)) == NULL)
