@@ -130,12 +130,19 @@ struct db_kind {
 #ifdef	HAVE_DB_H
 { "btree",	{ NULL, NULL, 0, 10, 0, NULL, search_btree, close_btree,
 		  add_btree, remove_btree, print_btree, count_btree,
-		  owner_btree, NULL, Nul } },
+		  owner_btree, modp_btree, Nul } },
+{ "bhash",	{ NULL, NULL, 0, 10, 0, NULL, search_bhash, close_bhash,
+		  add_bhash, remove_bhash, print_bhash, count_bhash,
+		  owner_bhash, modp_bhash, Nul } },
 #endif	/* HAVE_DB_H */
 #ifdef	HAVE_YP
 { "yp",		{ NULL, NULL, 0, 10, 0, NULL, search_yp, NULL,
 		  NULL, NULL, print_yp, NULL, owner_yp, NULL, Nul } },
 #endif	/* HAVE_YP */
+#ifdef HAVE_LDAP
+{ "ldap",	{ NULL, NULL, 0, 10, 0, NULL, search_ldap, close_ldap,
+		  NULL, NULL, NULL, NULL, NULL, modp_ldap, Nul } },
+#endif
 { 0 }	/* proto_config is initialized from this entry */
 };
 
@@ -278,7 +285,7 @@ run_relation(argc, argv)
 		return 3;
 	}
 
-	symid = symbol(argv[optind]);
+	symid = symbol(argv[optind]); /* Database name symbol  */
 	if (sp_lookup(symid, spt_databases) != NULL) {
 		fprintf(stderr, "%s: %s is already a defined database!\n",
 				argv[0], argv[optind]);
