@@ -1278,7 +1278,7 @@ run_listexpand(avl, il)
 	struct header hs;
 	struct envelope *e;
 	struct address *ap, *aroot = NULL, **atail = &aroot;
-	struct token *t;
+	token822 *t;
 	conscell *al, *alp = NULL, *tmp;
 	char *localpart, *origaddr, *attributenam;
 	int c, n, errflag, stuff;
@@ -1733,7 +1733,7 @@ run_listaddrs(argc, argv)
 	struct header hs;
 	struct envelope *e;
 	struct address *ap, *aroot = NULL, **atail = &aroot;
-	struct token *t;
+	token822 *t;
 	struct addr *pp;
 	int c, n, errflag, stuff;
 	const char *comment, *erroraddress;
