@@ -10,11 +10,15 @@
 
 #undef	RFC974		/* MX/WKS/A processing according to RFC974 */
 
+#include <sys/socket.h>
+
 #include <netdb.h>
 #ifndef EAI_AGAIN
 # include "netdb6.h" /* IPv6 API stuff */
 #endif
 
+#define CUC const u_char
+
 /* ================ do we have a normal nameserver? ================ */
 #ifdef	TRY_AGAIN
 
@@ -43,9 +47,12 @@
 #endif
 #include <arpa/nameser.h>
 #include <resolv.h>
+#include <string.h>
 #include "search.h"
 
+#include "libz.h"
 #include "libc.h"
+#include "libsh.h"
 #include "dnsgetrr.h"
 
 
@@ -76,10 +83,7 @@
 #endif	/* !BIND_VER */
 
 #if	defined(BIND_VER) && (BIND_VER >= 473)
-char	*conffile;
-typedef u_char msgdata;
-#else	/* !defined(BIND_VER) || (BIND_VER < 473) */
-typedef char msgdata;
+const char * conffile;
 #endif	/* defined(BIND_VER) && (BIND_VER >= 473) */
 
 
@@ -136,8 +140,8 @@ extern int	h_errno;
 char h_errhost[MAXNAME];
 
 struct qtypes {
-	char	*typename;
-	u_short	value;
+	const char *typename;
+	int	value;
 } qt[] = {
 	{	"cname",	T_CNAME		},
 	{	"any",		T_ANY		},
@@ -157,7 +161,7 @@ struct qtypes {
 	{	0,		0		}
 };
 
-char *zh_errlist[] = {
+const char *zh_errlist[] = {
 	"Error 0",
 	"unknown host/not findable at all",	/* 1 - HOST_NOT_FOUND	*/
 	"host name lookup/try again",		/* 2 - TRY_AGAIN	*/
@@ -165,18 +169,15 @@ char *zh_errlist[] = {
 	"no address associated with name"	/* 4 - NO_ADDRESS	*/
 };
 
-extern struct conscell *getmxrr(), *getcrrtype(), *getrrtypec();
-
-extern char *conffile, *myhostname;
-extern int deferit, res_init(), cistrcmp();
-extern void die(), v_set();
-extern char *dottedquad();
-extern int res_mkquery(), res_send(), dn_expand(), dn_skipname();
-#ifndef strchr
-extern char *strchr();
-#endif
+static conscell * getmxrr    __((const char *, const char *, time_t *, int));
+static conscell * getcrrtype __((const char *, int, time_t *, int));
+static conscell * getrrtypec __((const char *, int, time_t *, int));
 
+extern const char * myhostname;
+extern int deferit;
+extern int cistrcmp __((const char *, const char *));
 
+#if 0
 void
 zherror(s)
 	char *s;
@@ -187,14 +188,16 @@ zherror(s)
 	else
 	  fprintf(stderr, "%s: resolver error: %d\n", s, h_errno);
 }
+#endif
 
-struct conscell *
+conscell *
 search_res(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	struct qtypes *qtp;
-	struct conscell *rval;
-	char *host, buf[BUFSIZ];
+	conscell *rval;
+	const char *host;
+	char        buf[BUFSIZ];
 
 	if (!(_res.options & RES_INIT)) {
 	  if (sip->file != NULL)
@@ -249,18 +252,20 @@ search_res(sip)
 #endif	/* T_UNAME */
 	case T_TXT:
 	case T_UINFO:
-		rval = getrrtypec(sip->key, qtp->value, &sip->ttl, 0);
+		rval = getrrtypec(sip->key, qtp->value,
+				  &sip->ttl, 0);
 		break;
 	case T_ANY:
 	case T_CNAME:
-		rval = getcrrtype(sip->key, qtp->value, &sip->ttl, 0);
+		rval = getcrrtype(sip->key, qtp->value,
+				  &sip->ttl, 0);
 		break;
 	default:
 		rval = NULL;
 	}
 	if (h_errno > 0) {
 	  if (h_errhost[0] == '\0')
-	    host = (char *)sip->key;
+	    host = sip->key;
 	  else
 	    host = h_errhost;
 
@@ -289,26 +294,26 @@ typedef union {
 } querybuf;
 
 struct mxdata {
-	msgdata *host;
-	u_short	pref;
+	char *host;
+	int	pref;
 	time_t	ttl;
 };
 
-struct conscell *
+static conscell *
 getmxrr(host, localhost, ttlp, depth)
-	char *host;
-	char *localhost;
+	const char *host;
+	const char *localhost;
 	time_t *ttlp;
 	int depth;
 {
 	HEADER *hp;
-	msgdata *eom, *cp;
+	CUC *eom, *cp;
 	querybuf buf, answer;
 	int n, qlen, nmx, i, ancount, qdcount, maxpref;
-	struct conscell *lhead, *l, *tmp;
+	conscell *lhead, *l, *tmp;
 	struct mxdata mx[20];
-	msgdata hbuf[MAXNAME], realname[MAXNAME];
-	u_short type;
+	char hbuf[MAXNAME], realname[MAXNAME];
+	int type;
 	time_t ttl, maxttl;
 
 	if (depth > 4) {
@@ -340,7 +345,7 @@ getmxrr(host, localhost, ttlp, depth)
 	    return NULL;
 	  }
 	}
-	eom = (msgdata *)&answer + n;
+	eom = (CUC *)&answer + n;
 	/*
 	 * find first satisfactory answer
 	 */
@@ -382,12 +387,12 @@ getmxrr(host, localhost, ttlp, depth)
 		}
 		return NULL;
 	}
-	cp = (msgdata *)&answer + sizeof(HEADER);
+	cp = (CUC *)&answer + sizeof(HEADER);
 	for (; qdcount > 0; --qdcount)
 #if	defined(BIND_VER) && (BIND_VER >= 473)
-		cp += dn_skipname(cp, eom) + QFIXEDSZ;
+		cp += dn_skipname((CUC*)cp, (CUC*)eom) + QFIXEDSZ;
 #else	/* !defined(BIND_VER) || (BIND_VER < 473) */
-		cp += dn_skip(cp) + QFIXEDSZ;
+		cp += dn_skip((CUC*)cp) + QFIXEDSZ;
 #endif	/* defined(BIND_VER) && (BIND_VER >= 473) */
 	nmx = 0;
 	maxttl = ttl = 0;
@@ -395,7 +400,8 @@ getmxrr(host, localhost, ttlp, depth)
 	maxpref = -1;
 	/* assert: stickymem == MEM_MALLOC;  for storing RHS of MX RR's */
 	while (--ancount >= 0 && cp < eom && nmx < (sizeof mx/sizeof mx[0])) {
-		n = dn_expand((msgdata *)&answer, eom, cp, (void*)hbuf, sizeof hbuf);
+		n = dn_expand((CUC*)&answer, (CUC*)eom, (CUC*)cp,
+			      (void*)hbuf, sizeof hbuf);
 		if (n < 0)
 			break;
 		cp += n;
@@ -407,7 +413,7 @@ getmxrr(host, localhost, ttlp, depth)
 		n = _getshort(cp);
 		cp += 2;				/* dlen -- short */
 		if (type == T_CNAME) {
-			cp += dn_expand((msgdata *)&answer, eom, cp,
+			cp += dn_expand((CUC*)&answer, (CUC*)eom, (CUC*)cp,
 					(void*)realname, sizeof realname);
 			continue;
 		} else if (type != T_MX)  {
@@ -421,11 +427,12 @@ getmxrr(host, localhost, ttlp, depth)
 		mx[nmx].ttl = ttl;
 		mx[nmx].pref = _getshort(cp);
 		cp += 2; /* "short" */		/* MX preference value */
-		n = dn_expand((msgdata *)&answer, eom, cp, (void*)hbuf, sizeof hbuf);
+		n = dn_expand((CUC*)&answer, (CUC*)eom, (CUC*)cp,
+			      (void*)hbuf, sizeof hbuf);
 		if (n < 0)
 			break;
 		cp += n;
-		mx[nmx].host = (msgdata *)strsave((char*)hbuf);
+		mx[nmx].host = (char *)strsave((char*)hbuf);
 		if (localhost != NULL && CISTREQ(hbuf, localhost))
 		    if ((maxpref < 0) || (maxpref > (int)mx[nmx].pref))
 			maxpref = mx[nmx].pref;
@@ -471,9 +478,9 @@ getmxrr(host, localhost, ttlp, depth)
 			fprintf(stderr, "search_res: %s: mx[%d] = %s\n",
 				host, n, mx[i].host);
 		if (lhead == NULL)
-			lhead = l = newstring((u_char *)mx[i].host);
+			lhead = l = newstring(mx[i].host);
 		else {
-			cdr(l) = newstring((u_char *)mx[i].host);
+			cdr(l) = newstring(mx[i].host);
 			l = cdr(l);
 		}
 	}
@@ -486,15 +493,16 @@ getmxrr(host, localhost, ttlp, depth)
 	return ncons(lhead);
 }
 
-struct conscell *
+static conscell *
 getcrrtype(host, rrtype, ttlp, depth)	/* getrrtypec() with completion */
-	char *host;
-	u_short rrtype;
+	const char *host;
+	int rrtype;
 	time_t *ttlp;
 	int depth;
 {
-	struct conscell *rval;
-	char buf[BUFSIZ], *domain;
+	conscell *rval;
+	char buf[BUFSIZ];
+	char *domain;
 	int i;
 
 	if (depth > 4) {
@@ -507,7 +515,8 @@ getcrrtype(host, rrtype, ttlp, depth)	/* getrrtypec() with completion */
 	}
 
 	if (strchr(host, '.') != NULL) {
-		if ((rval = getrrtypec(host, rrtype, ttlp, depth+1)) != NULL)
+		rval = getrrtypec(host, rrtype, ttlp, depth+1);
+		if (rval != NULL)
 			return rval;
 		if (*host != '\0' && *(host+strlen(host)-1) == '.')
 			return NULL;
@@ -515,31 +524,31 @@ getcrrtype(host, rrtype, ttlp, depth)	/* getrrtypec() with completion */
 	for (i = 0, domain = _res.dnsrch[i];
 	     h_errno == 0 && (domain = _res.dnsrch[i]) != NULL; ++i) {
 		sprintf(buf, "%s.%s", host, domain);
-		if ((rval = getrrtypec(buf, rrtype, ttlp, depth+1)) != NULL)
+		rval = getrrtypec(buf, rrtype, ttlp, depth+1);
+		if (rval != NULL)
 			return rval;
 	}
 	strcpy(h_errhost, host);
 	return NULL;
 }
 
-struct conscell *
+static conscell *
 getrrtypec(host, rrtype, ttlp, depth)
-	char *host;
-	u_short rrtype;
+	const char *host;
+	int rrtype;
 	time_t *ttlp;
 	int depth;
 {
-	struct conscell *lhead, *l, *tmp;
+	conscell *lhead, *l, *tmp;
 	char *s;
 	HEADER *hp;
-	msgdata *eom;
-	const msgdata *cp, *nextcp;
+	CUC *eom, *cp, *nextcp;
 	querybuf buf, answer;
 	int qlen, n, ancount, qdcount, ok, first;
 	struct in_addr inaddr;
 	time_t maxttl, ttl;
-	u_short type;
-	msgdata nbuf[BUFSIZ];
+	int type;
+	char nbuf[BUFSIZ];
 	char hb[MAXNAME];
 
 	if (depth > 4) {
@@ -574,7 +583,7 @@ getrrtypec(host, rrtype, ttlp, depth)
 	    return NULL;
 	  }
 	}
-	eom = (msgdata *)&answer + n;
+	eom = (CUC *)&answer + n;
 	/*
 	 * find first satisfactory answer
 	 */
@@ -590,54 +599,54 @@ getrrtypec(host, rrtype, ttlp, depth)
 				"search_res: rcode=%d, ancount=%d, rrtype=%d\n",
 				hp->rcode, ancount, rrtype);
 		if (rrtype == T_CNAME && hp->rcode == NOERROR) {
-			if (qdcount > 0 && strchr(host, '.') == NULL) {
-				cp = (msgdata *)&answer + sizeof(HEADER);
-				if (dn_expand((msgdata *)&answer,eom,cp,
-					      (void*)hb,sizeof hb)>=0) {
-					if (hb[0] == '\0') {
-						hb[0] = '.'; hb[1] = '\0';
-					}
-					return newstring((u_char*)strsave(hb));
-				}
-			}
-			return newstring((u_char*)strsave(host));
+		  if (qdcount > 0 && strchr(host, '.') == NULL) {
+		    cp = (CUC*) &answer + sizeof(HEADER);
+		    if (dn_expand((CUC*)&answer, (CUC*)eom, (CUC*)cp,
+				  (void*)hb, sizeof hb)>=0) {
+		      if (hb[0] == '\0') {
+			hb[0] = '.'; hb[1] = '\0';
+		      }
+		      return newstring(strsave(hb));
+		    }
+		  }
+		  return newstring(strsave(host));
 		}
 		if (rrtype == T_WKS) /* absence of WKS means YES ... */
-			return newstring((u_char*)strsave(host));
+			return newstring(strsave(host));
 		return NULL;
 	}
-	cp = (msgdata *)&answer + sizeof(HEADER);
+	cp = (CUC *)&answer + sizeof(HEADER);
 	for (; qdcount > 0; --qdcount)
 #if	defined(BIND_VER) && (BIND_VER >= 473)
-		cp += dn_skipname(cp, eom) + QFIXEDSZ;
+		cp += dn_skipname((CUC*)cp, (CUC*)eom) + QFIXEDSZ;
 #else	/* !defined(BIND_VER) || (BIND_VER < 473) */
-		cp += dn_skip(cp) + QFIXEDSZ;
+		cp += dn_skip((CUC*)cp) + QFIXEDSZ;
 #endif	/* defined(BIND_VER) && (BIND_VER >= 473) */
 	first = 1;
 	ok = rrtype != T_WKS;
 	maxttl = 0;
 	l = NULL;
 	for (lhead = NULL; --ancount >= 0 && cp < eom; cp = nextcp) {
-		n = dn_expand((msgdata *)&answer, eom, cp,
+		n = dn_expand((CUC*)&answer, (CUC*)eom, (CUC*)cp,
 			      (void*)nbuf, sizeof(nbuf));
 		if (n < 0)
 			break;
 		if (first) {
-			if (strlen((char *)nbuf) < sizeof hb)
-				strcpy(hb, (char *)nbuf);
+			if (strlen(nbuf) < sizeof hb)
+				strcpy(hb, nbuf);
 			else {
-				strncpy(hb, (char *)nbuf, sizeof hb);
+				strncpy(hb, nbuf, sizeof hb);
 				hb[(sizeof hb) - 1] = '\0';
 			}
 			first = 0;
 		}
 		cp += n;
-		type = _getshort(cp);
+		type = _getshort((const u_char*)cp);
  		cp += 2;			/* type  -- short */
  		cp += 2;			/* class -- short */
-		ttl = (time_t) _getlong(cp);
+		ttl = (time_t) _getlong((const u_char*)cp);
 		cp += 4;			/* ttl -- "long" */
-		n = _getshort(cp);
+		n = _getshort((const u_char*)cp);
 		cp += 2;			/* dlen -- short */
 		nextcp = cp + n;
 
@@ -663,8 +672,9 @@ getrrtypec(host, rrtype, ttlp, depth)
 				continue;	/* Not our time.. */
 
 			*ttlp = maxttl;
-			if ((n = (int)(*cp)) > 0 && n < sizeof hb)
-				return newstring((u_char*)strnsave((char*)cp+1, n));
+			n = (*cp) & 0xFF;
+			if (0 < n && n < sizeof hb)
+			  return newstring(strnsave((const char *)cp+1, n));
 			break;
 
 		case T_WKS:
@@ -681,49 +691,36 @@ getrrtypec(host, rrtype, ttlp, depth)
 			cp += 4;		/* skip IP address */
 			if (*cp++ == IPPROTO_TCP) {	/* check protocol */
 			  if (cp + (IPPORT_SMTP/8) < nextcp &&
-			      (*(cp+(IPPORT_SMTP/8)) &
-			       (0x80>>IPPORT_SMTP%8))) {
+			      *(cp+(IPPORT_SMTP/8)) & (0x80>>IPPORT_SMTP%8)) {
 			    *ttlp = maxttl;
-			    return newstring((u_char*)strsave(hb));
+			    return newstring(strsave(hb));
 			  }
 			}
 			continue;
-		case T_AAAA:
-			if (rrtype == T_ANY) {
-			  return newstring((u_char*)strsave(host));
-			}
-#if defined(AF_INET6) && defined(INET6)
-			{
-			  char tb[80];
-			  const char *ss;
-
-			  ss = inet_ntop(AF_INET6, cp, tb, sizeof(tb));
-			  s = strsave(ss);
-			  if (lhead == NULL)
-			    lhead = l = newstring((u_char *)s);
-			  else {
-			    cdr(l) = newstring((u_char *)s);
-			    l = cdr(l);
-			  }
-			  *ttlp = maxttl;
-			}
-#endif
-			continue;
 		case T_A:
 		case T_SA:
-			if (rrtype == T_ANY) {
-			    return newstring((u_char*)strsave(host));
-			}
-			memcpy((char *)&inaddr, cp, sizeof inaddr);
-			s = dottedquad(&inaddr);
-			s = strsave(s);
-			if (lhead == NULL)
-				lhead = l = newstring((u_char *)s);
-			else {
-				cdr(l) = newstring((u_char *)s);
-				l = cdr(l);
-			}
+		case T_AAAA:
 			*ttlp = maxttl;
+			if (rrtype == T_ANY) {
+			  return newstring(strsave(host));
+			} else {
+			  char tb[80];
+			  const char *ss;
+
+#if defined(AF_INET6) && defined(INET6)
+			  if (type == T_AAAA)
+			    ss = inet_ntop(AF_INET6, cp, tb, sizeof(tb));
+			  else
+#endif
+			    ss = inet_ntop(AF_INET, cp, tb, sizeof(tb));
+			  s = strsave(ss);
+			  if (lhead == NULL)
+			    lhead = l = newstring(s);
+			  else {
+			    cdr(l) = newstring(s);
+			    l = cdr(l);
+			  }
+			}
 			continue;
 
 		case T_CNAME:
@@ -746,16 +743,17 @@ getrrtypec(host, rrtype, ttlp, depth)
 				   the PTR data was taken in as CNAME.. */
 				continue;
 
-			if ((n = dn_expand((msgdata *)&answer, eom, cp,
-					   (void*)nbuf, sizeof(nbuf))) < 0)
+			n = dn_expand((CUC*)&answer, (CUC*)eom, (CUC*)cp,
+				      (void*)nbuf, sizeof(nbuf));
+			if (n < 0)
 				break;
 
 			if (type == T_CNAME && rrtype != T_ANY &&
 			    !CISTREQ(nbuf, host))
 				/* chase it down */
-				getrrtypec((char *)nbuf, rrtype, ttlp, depth+1);
+				getrrtypec(nbuf, rrtype, ttlp, depth+1);
 			*ttlp = maxttl;
-			return newstring((u_char*)strsave((char*)nbuf));
+			return newstring(strsave(nbuf));
 
 		case T_SOA:
 		case T_NS:
@@ -774,7 +772,7 @@ getrrtypec(host, rrtype, ttlp, depth)
 
 		case T_MX:
 			if (rrtype == T_ANY) {
-			    return newstring((u_char*)strsave(host));
+			    return newstring(strsave(host));
 			}
 		default:
 			fprintf(stderr,"search_res: getrrtypec: non-processed RR type for host='%s' query=%d, result=%d\n",host,rrtype,type);
@@ -785,9 +783,9 @@ getrrtypec(host, rrtype, ttlp, depth)
 	if (lhead)
 		return ncons(lhead);
 	if (ok) {
+		const char *cs = first ? host : hb;
 		*ttlp = maxttl;
-		s = first ? host : hb;
-		return newstring((u_char*)strsave(s));
+		return newstring(strsave(cs));
 	}
 	return NULL;
 }
