@@ -2,7 +2,7 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  *
- *	Copyright 1996 Matti Aarnio
+ *	Copyright 1996-1997 Matti Aarnio
  */
 
 /* LINTLIBRARY */
@@ -35,20 +35,22 @@ close_btree(sip)
 	search_info *sip;
 {
 	DB *db;
-	struct spblk *spl;
+	struct spblk *spl = NULL;
 	spkey_t symid;
 
-	if (sip->file == NULL)
+	if (sip->file == NULL || spt_files->symbols == NULL)
 		return;
-	symid = symbol(sip->file);
-	spl = sp_lookup(symid, spt_modcheck);
+	symid = symbol_lookup_db(sip->file, spt_files->symbols);
+	if ((spkey_t)0 != symid)
+	  spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL)
-		sp_delete(spl, spt_modcheck);
+	  sp_delete(spl, spt_modcheck);
 	spl = sp_lookup(symid, spt_files);
 	if (spl == NULL || (db = (DB *)spl->data) == NULL)
 		return;
 	(db->close)(db);
-	sp_install(symid, NULL, 0, spt_files);
+	symbol_free_db(sip->file, spt_files->symbols);
+	sp_delete(spl, spt_files);
 }
 
 
@@ -62,15 +64,26 @@ open_btree(sip, flag, comment)
 	DB *db;
 	struct spblk *spl;
 	spkey_t symid;
+	int i;
 
 	if (sip->file == NULL)
 		return NULL;
-	symid = symbol(sip->file);
+	if (spt_files == NULL)
+	  spt_files = sp_init();
+	if (spt_files->symbols == NULL)
+	  spt_files->symbols = sp_init();
+
+	symid = symbol_db(sip->file, spt_files->symbols);
 	spl = sp_lookup(symid, spt_files);
 	if (spl != NULL && flag == O_RDWR && spl->mark != O_RDWR)
 		close_btree(sip);
 	if (spl == NULL || (db = (DB *)spl->data) == NULL) {
-		db = dbopen(sip->file, flag, 0, DB_BTREE, &BINFO);
+		for (i = 0; i < 3; ++i) {
+		  db = dbopen(sip->file, flag, 0, DB_BTREE, &BINFO);
+		  if (db != NULL)
+		    break;
+		  sleep(1); /* Open failed, retry after a moment */
+		}
 		if (db == NULL) {
 			++deferit;
 			v_set(DEFER, DEFER_IO_ERROR);
@@ -103,15 +116,12 @@ search_btree(sip)
 	spkey_t symid;
 	char *us;
 
-	if (sip->file == NULL)
-		return NULL;
 	retry = 0;
-	symid = symbol(sip->file);
-	spl = sp_lookup(symid, spt_files);
-	if (spl == NULL || (db = (DB *)spl->data) == NULL) {
 reopen:
-		db = open_btree(sip, O_RDONLY, "search_btree");
-	}
+	db = open_btree(sip, O_RDONLY, "search_btree");
+	if (db == NULL)
+	  return NULL; /* Huh! */
+
 	key.data = (char*)sip->key;
 	key.size = strlen(sip->key) + 1;
 	rc = (db->get)(db, &key, &val, 0);
@@ -144,6 +154,7 @@ add_btree(sip, value)
 	db = open_btree(sip, O_RDWR, "add_btree");
 	if (db == NULL)
 		return EOF;
+
 	key.data = (char*)sip->key;
 	key.size = strlen(sip->key) + 1;
 	val.data = (char*)value;
@@ -174,6 +185,7 @@ remove_btree(sip)
 	db = open_btree(sip, O_RDWR, "remove_btree");
 	if (db == NULL)
 		return EOF;
+
 	key.data = (char*)sip->key;
 	key.size = strlen(sip->key) + 1;
 	rc = (db->del)(db, &key, 0);
@@ -200,11 +212,12 @@ print_btree(sip, outfp)
 	DBT key, val;
 	int rc;
 
-	if ((db = open_btree(sip, O_RDONLY, "print_btree")) == NULL)
+	db = open_btree(sip, O_RDONLY, "print_btree");
+	if (db == NULL)
 		return;
 
 	rc = (db->seq)(db, &key, &val, R_FIRST);
-	for ( ; rc != 0 ; ) {
+	for ( ; rc == 0 ; ) {
 		if (val.data == NULL)
 			continue;
 		if (*(char*)val.data == '\0')
@@ -230,7 +243,8 @@ count_btree(sip, outfp)
 	int cnt = 0;
 	int rc;
 
-	if ((db = open_btree(sip, O_RDONLY, "count_btree")) != NULL) {
+	db = open_btree(sip, O_RDONLY, "count_btree");
+	if (db != NULL) {
 	  rc = (db->seq)(db, &key, &val, R_FIRST);
 	  while (rc == 0) {
 	    if (val.data == NULL)
@@ -257,9 +271,11 @@ owner_btree(sip, outfp)
 	DB *db;
 	struct stat stbuf;
 
-	if ((db = open_btree(sip, O_RDONLY, "owner_btree")) == NULL)
+	db = open_btree(sip, O_RDONLY, "owner_btree");
+	if (db == NULL)
 		return;
-	/* There are more timing hazards, when the internal fd is not
+
+	/* There are timing hazards, when the internal fd is not
 	   available for probing.. */
 	if (fstat((db->fd)(db), &stbuf) < 0) {
 		fprintf(stderr, "owner_btree: cannot fstat(\"%s\")!\n",
@@ -280,9 +296,10 @@ modp_btree(sip)
 	spkey_t symid;
 	int rval;
 
-	if (sip->file == NULL
-	    || (db = open_btree(sip, O_RDONLY, "owner_btree")) == NULL)
+	db = open_btree(sip, O_RDONLY, "owner_btree");
+	if (db == NULL)
 		return 0;
+
 	if (fstat((db->fd)(db), &stbuf) < 0) {
 		fprintf(stderr, "modp_btree: cannot fstat(\"%s\")!\n",
 				sip->file);
@@ -291,7 +308,7 @@ modp_btree(sip)
 	if (stbuf.st_nlink == 0)
 		return 1;	/* Unlinked underneath of us! */
 	
-	symid = symbol(sip->file);
+	symid = symbol_lookup_db(sip->file, spt_files->symbols);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
 		rval = ((long)stbuf.st_mtime != (long)spl->data ||
