@@ -2,7 +2,7 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  *
- *	Port of NDBM version to GDBM by Matti Aarnio 1989-1994
+ *	Port of NDBM version to GDBM by Matti Aarnio 1989-1997
  *
  *	Currently uses GDBM version 1.7.3 !
  */
@@ -43,6 +43,54 @@ struct __gdbmfoo {
 extern int deferit;
 extern int nobody;	/* UID of NOBODY */
 
+static GDBM_FILE open_gdbm __((search_info *, int, const char *));
+
+static GDBM_FILE
+open_gdbm(sip, flag, comment)
+	search_info *sip;
+	int flag;
+	const char *comment;
+{
+	GDBM_FILE db;
+	struct spblk *spl;
+	spkey_t symid;
+	int i;
+
+	if (sip->file == NULL)
+		return NULL;
+	if (spt_files == NULL)		spt_files = sp_init();
+	if (spt_files->symbols == NULL) spt_files->symbols = sp_init();
+
+	symid = symbol_db(sip->file, spt_files->symbols);
+	spl = sp_lookup(symid, spt_files);
+
+	if (spl != NULL && flag == O_RDWR && spl->mark != O_RDWR)
+		close_gdbm(sip);
+	if (flag == O_RDWR)	flag = GDBM_WRITER;
+	else			flag = GDBM_READER;
+	if (spl == NULL || (db = (GDBM_FILE)spl->data) == NULL) {
+		for (i = 0; i < 3; ++i) {
+		  db = gdbm_open((char*)sip->file, 0, flag, 0, NULL);
+		  if (db != NULL)
+		    break;
+		  sleep(1);
+		}
+		
+		if (db == NULL) {
+			++deferit;
+			v_set(DEFER, DEFER_IO_ERROR);
+			fprintf(stderr, "%s: cannot open %s!\n",
+					comment, sip->file);
+			return NULL;
+		}
+		if (spl == NULL)
+			sp_install(symid, (void *)db, flag, spt_files);
+		else
+			spl->data = (void *)db;
+	}
+	return db;
+}
+
 
 /*
  * Search an GDBM format database for a key pair.
@@ -55,30 +103,18 @@ search_gdbm(sip)
 	GDBM_FILE db;
 	datum val, key;
 	conscell *tmp;
-	struct spblk *spl;
-	int retry;
+	struct spblk *spl = NULL;
+	int retry, i;
 	spkey_t symid;
 
-	if (sip->file == NULL)
-		return NULL;
 	retry = 0;
-	symid = symbol(sip->file);
-	spl   = sp_lookup(symid, spt_files);
-	if (spl == NULL || (db = (GDBM_FILE)spl->data) == NULL) {
+
 reopen:
-		db = gdbm_open((char*)sip->file, 0, GDBM_READER, 0, NULL);
-		if (db == NULL) {
-			++deferit;
-			v_set(DEFER, DEFER_IO_ERROR);
-			fprintf(stderr, "search_gdbm: cannot open %s!\n",
-					sip->file);
-			return NULL;
-		}
-		if (spl == NULL)
-			sp_install(symid, (u_char *)db, O_RDONLY, spt_files);
-		else
-			spl->data = (void *)db;
-	}
+	db = open_gdbm(sip, O_RDONLY, "search_gdbm");
+
+	if (db == NULL)
+	  return NULL;
+
 	key.dptr  = (char*)sip->key;
 	key.dsize = strlen(sip->key) + 1;
 	val = gdbm_fetch(db, key);
@@ -110,7 +146,9 @@ close_gdbm(sip)
 
 	if (sip->file == NULL)
 		return;
-	symid = symbol(sip->file);
+	symid = symbol_db(sip->file, spt_files->symbols);
+	if ((spkey_t)0 == symid)
+		return;
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL)
 		sp_delete(spl, spt_modcheck);
@@ -118,45 +156,9 @@ close_gdbm(sip)
 	if (spl == NULL || (db = (GDBM_FILE)spl->data) == NULL)
 		return;
 	gdbm_close(db);
-	sp_install(symid, (u_char *)NULL, 0, spt_files);
+	sp_delete(spl, spt_files);
 }
 
-static GDBM_FILE open_gdbm __((search_info *, int, const char *));
-
-static GDBM_FILE
-open_gdbm(sip, flag, comment)
-	search_info *sip;
-	int flag;
-	const char *comment;
-{
-	GDBM_FILE db;
-	struct spblk *spl;
-	spkey_t symid;
-
-	if (sip->file == NULL)
-		return NULL;
-	symid = symbol(sip->file);
-	spl = sp_lookup(symid, spt_files);
-	if (spl != NULL && flag == O_RDWR && spl->mark != O_RDWR)
-		close_gdbm(sip);
-	if (flag == O_RDWR)	flag = GDBM_WRITER;
-	else			flag = GDBM_READER;
-	if (spl == NULL || (db = (GDBM_FILE)spl->data) == NULL) {
-		db = gdbm_open((char*)sip->file, 0, flag, 0, NULL);
-		if (db == NULL) {
-			++deferit;
-			v_set(DEFER, DEFER_IO_ERROR);
-			fprintf(stderr, "%s: cannot open %s!\n",
-					comment, sip->file);
-			return NULL;
-		}
-		if (spl == NULL)
-			sp_install(symid, (void *)db, flag, spt_files);
-		else
-			spl->data = (void *)db;
-	}
-	return db;
-}
 
 /*
  * Add the indicated key/value pair to the database.
@@ -173,6 +175,7 @@ add_gdbm(sip, value)
 	db = open_gdbm(sip, O_RDWR, "add_gdbm");
 	if (db == NULL)
 		return EOF;
+
 	key.dptr  = (char*)sip->key;
 	key.dsize = strlen(sip->key) + 1;
 	val.dptr  = (char*)value;
@@ -201,6 +204,7 @@ remove_gdbm(sip)
 	db = open_gdbm(sip, O_RDWR, "remove_gdbm");
 	if (db == NULL)
 		return EOF;
+
 	key.dptr  = (char*)sip->key;
 	key.dsize = strlen(sip->key) + 1;
 	if (gdbm_delete(db, key) < 0) {
@@ -229,6 +233,7 @@ print_gdbm(sip, outfp)
 	db = open_gdbm(sip, O_RDONLY, "print_gdbm");
 	if (db == NULL)
 		return;
+
 	for (key = gdbm_firstkey(db); key.dptr != NULL; nextkey = gdbm_nextkey(db, key)) {
 		val = gdbm_fetch(db, key);
 		if (val.dptr == NULL)
@@ -294,8 +299,12 @@ owner_gdbm(sip, outfp)
 	int	fno;
 	struct stat stbuf;
 
-	if ((db = open_gdbm(sip, O_RDONLY, "owner_gdbm")) == NULL)
-		return;
+	db = open_gdbm(sip, O_RDONLY, "owner_gdbm");
+	if (db == NULL) {
+	  fprintf(outfp, "%d\n", nobody);
+	  return;
+	}
+
 	fno = gdbm_filefno(db);
 	if (fstat(fno, &stbuf) < 0) {
 		fprintf(stderr, "owner_gdbm: cannot fstat(\"%s\")!\n",
@@ -318,9 +327,10 @@ modp_gdbm(sip)
 	spkey_t symid;
 	int rval;
 
-	if (sip->file == NULL
-	    || (db = open_gdbm(sip, O_RDONLY, "modp_gdbm")) == NULL)
+	db = open_gdbm(sip, O_RDONLY, "modp_gdbm");
+	if (db == NULL)
 		return 0;
+
 	fno = gdbm_filefno(db);
 	if (fstat(fno, &stbuf) < 0) {
 		fprintf(stderr, "modp_gdbm: cannot fstat(\"%s\")!\n",
@@ -330,7 +340,7 @@ modp_gdbm(sip)
 	if (stbuf.st_nlink == 0)
 		return 1;	/* Unlinked underneath of us! */
 
-	symid = symbol(sip->file);
+	symid = symbol_db(sip->file, spt_files->symbols);
 	spl   = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
 		rval = stbuf.st_mtime != (time_t)spl->data
