@@ -30,12 +30,12 @@ struct headerinfo mandatory_hdrs[] = {
 { "cc",			AddressList,	Recipient,	normal		},
 { "from",		AMailboxList,	Sender,		normal		},
 { "message-id",		MessageID,	nilUserType,	normal		},
-{ "reply-to",		AddressList,	Sender,		normal		},
+{ "reply-to",		AMailboxList,	Sender,		normal		},
 { "resent-bcc",		Addresses,	Recipient,	Resent		},
 { "resent-cc",		AddressList,	Recipient,	Resent		},
 { "resent-from",	AMailboxList,	Sender,		Resent		},
 { "resent-message-id",	MessageID,	nilUserType,	Resent		},
-{ "resent-reply-to",	AddressList,	Sender,		Resent		},
+{ "resent-reply-to",	AMailboxList,	Sender,		Resent		},
 { "resent-sender",	Mailbox,	Sender,		Resent		},
 { "resent-to",		AddressList,	Recipient,	Resent		},
 { "sender",		Mailbox,	Sender,		normal		},
@@ -147,41 +147,42 @@ init_header()
 {
 	struct headerinfo rh, *rhp;
 	int i;
-	memtypes oval;
+	spkey_t symid;
+
+	if (spt_headers == NULL)           spt_headers           = sp_init();
+	if (spt_headers->symbols == NULL)  spt_headers->symbols  = sp_init();
+	if (spt_eheaders == NULL)          spt_eheaders          = sp_init();
+	if (spt_eheaders->symbols == NULL) spt_eheaders->symbols = sp_init();
 
-	oval = stickymem;
 	for (i = 0; i < (sizeof mandatory_hdrs / sizeof mandatory_hdrs[0]); ++i) {
 	  rh.hdr_name  = mandatory_hdrs[i].hdr_name;
 	  rh.semantics = mandatory_hdrs[i].semantics;
 	  rh.user_type = mandatory_hdrs[i].user_type;
 	  rh.class     = mandatory_hdrs[i].class;
-	  stickymem    = MEM_MALLOC;
-	  rhp  = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
-	  stickymem    = oval;
+	  rhp  = (struct headerinfo *)emalloc(sizeof (struct headerinfo));
 	  *rhp = rh;
-	  sp_install(symbol(rhp->hdr_name), (void *)rhp, 1, spt_headers);
+	  symid = symbol_db(rhp->hdr_name, spt_headers->symbols);
+	  sp_install(symid, (void *)rhp, 1, spt_headers);
 	}
 	for (i = 0; i < (sizeof optional_hdrs / sizeof optional_hdrs[0]); ++i) {
 	  rh.hdr_name  = optional_hdrs[i].hdr_name;
 	  rh.semantics = optional_hdrs[i].semantics;
 	  rh.user_type = optional_hdrs[i].user_type;
 	  rh.class     = optional_hdrs[i].class;
-	  stickymem    = MEM_MALLOC;
-	  rhp  = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
+	  rhp  = (struct headerinfo *)emalloc(sizeof (struct headerinfo));
 	  *rhp = rh;
-	  stickymem = oval;
-	  sp_install(symbol(rhp->hdr_name), (void *)rhp, 0, spt_headers);
+	  symid = symbol_db(rhp->hdr_name, spt_headers->symbols);
+	  sp_install(symid, (void *)rhp, 0, spt_headers);
 	}
 	for (i = 0; i < (sizeof envelope_hdrs / sizeof envelope_hdrs[0]); ++i) {
 	  rh.hdr_name  = envelope_hdrs[i].hdr_name;
 	  rh.semantics = envelope_hdrs[i].semantics;
 	  rh.user_type = envelope_hdrs[i].user_type;
 	  rh.class     = envelope_hdrs[i].class;
-	  stickymem = MEM_MALLOC;
-	  rhp = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
+	  rhp = (struct headerinfo *)emalloc(sizeof (struct headerinfo));
 	  *rhp = rh;
-	  stickymem = oval;
-	  sp_install(symbol(rhp->hdr_name), (void *)rhp, 0, spt_eheaders);
+	  symid = symbol_db(rhp->hdr_name, spt_eheaders->symbols);
+	  sp_install(symid, (void *)rhp, 0, spt_eheaders);
 	}
 }
 
@@ -201,8 +202,9 @@ find_header(hdb, name)
 	struct sptree *hdb;
 	const char *name;
 {
-	struct spblk *spl;
+	struct spblk *spl = NULL;
 	register char *cp;
+	spkey_t spk;
 #ifdef	USE_ALLOCA
 	int len = strlen(name);
 	char *buffer = alloca(len+1);
@@ -229,7 +231,9 @@ find_header(hdb, name)
 	    *cp = tolower(c);
 	}
 
-	spl = sp_lookup(symbol(buffer), hdb);
+	spk = symbol_lookup_db(buffer, hdb->symbols);
+	if (spk != (spkey_t)0)
+	  spl = sp_lookup(spk, hdb);
 	if (spl == NULL)
 	  return NULL;
 	return (struct headerinfo *)spl->data;
@@ -309,12 +313,12 @@ add_header(sip, cvalue)
 {
 	struct sptree *hdb;
 	struct spblk *spl;
-	memtypes oval;
 	char *cp, *value, *vbuf;
 	char *wcp;
 	char *lcbuf;
 	struct headerinfo rh, *rhp;
 	int keylen;
+	spkey_t spk;
 
 	hdb = open_header(sip);
 	if (hdb == NULL)
@@ -438,15 +442,13 @@ add_header(sip, cvalue)
 
 	rh.hdr_name = sip->key;
 
-	oval = stickymem;
-	stickymem = MEM_MALLOC;
-	rhp  = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
+	rhp  = (struct headerinfo *)emalloc(sizeof (struct headerinfo));
 	*rhp = rh;
-	stickymem = oval;
 
-	spl = sp_lookup(symbol(sip->key), hdb);
+	spk = symbol_db(sip->key, hdb->symbols);
+	spl = sp_lookup(spk, hdb);
 	if (spl == NULL)
-	  sp_install(symbol(sip->key), (void *)rhp, 0, hdb);
+	  sp_install(spk, (void *)rhp, 0, hdb);
 	else if (spl->mark == 0) {
 	  hdfreedata(spl);
 	  spl->data = (void *)rhp;
@@ -478,12 +480,15 @@ remove_header(sip)
 	search_info *sip;
 {
 	struct sptree *hdb;
-	struct spblk *spl;
+	struct spblk *spl = NULL;
+	spkey_t spk;
 
 	hdb = open_header(sip);
 	if (hdb == NULL)
 	  return EOF;
-	spl = sp_lookup(symbol(sip->key), hdb);
+	spk = symbol_lookup_db(sip->key, hdb->symbols);
+	if ((spkey_t)0 != spk)
+	  spl = sp_lookup(spk, hdb);
 	if (spl == NULL) {
 	  fprintf(stderr, "remove_header: no such key as \"%s\"!\n",
 		  sip->key);
