@@ -24,7 +24,7 @@ search_hosts(sip)
 	struct hostent *hp;
 	conscell *tmp;
 
-	sethostent(1);
+	/* sethostent(1); */
 	hp = gethostbyname(sip->key);
 	if (hp == NULL)
 		return NULL;
@@ -40,7 +40,7 @@ print_hosts(sip, outfp)
 	search_info *sip;
 	FILE *outfp;
 {
-#if	!defined(sgi) && !defined(HAVE_RESOLVER)
+#ifdef HAVE_GETHOSTENT
 	char **cpp;
 	struct hostent *hp;
 
@@ -52,6 +52,7 @@ print_hosts(sip, outfp)
 			fprintf(outfp, " %s", *cpp);
 		putc('\n', outfp);
 	}
+	endhostent();
 #else
 	fprintf(outfp,
 		"# router: the gethostent() routine is not available\n");
