@@ -29,15 +29,38 @@ static struct sptree *
 open_core(sip)
 	search_info *sip;
 {
-	if (sip->subtype == NULL)
+	struct sptree *spt;
+
+	spt = (struct sptree *)sip->subtype;
+	if (spt == NULL)
 		return NULL;
-	return (struct sptree *)sip->subtype;
+	if (spt->symbols == NULL)
+		spt->symbols = sp_init();
+	if (spt == NULL)  /* Failed, damn! */
+		return NULL;
+	return spt;
 }
 
 /*
  * Search an incore database for a key.
  */
 
+
+struct spblk *
+lookup_incoresp(name, db)
+	const char *name;
+	struct sptree *db;
+{
+	spkey_t spk;
+
+	if (db->symbols == NULL)
+		db->symbols = sp_init();
+	spk = symbol_lookup_db(name, db->symbols);
+	if (spk == (spkey_t)0)
+		return NULL;
+	return sp_lookup(spk, db);
+}
+
 conscell *
 search_core(sip)
 	search_info *sip;
@@ -45,11 +68,15 @@ search_core(sip)
 	struct sptree *db;
 	struct spblk *spl;
 	conscell *tmp;
+	spkey_t spk;
 
 	db = open_core(sip);
 	if (db == NULL)
 		return NULL;
-	spl = sp_lookup(symbol(sip->key), db);
+	spk = symbol_lookup_db(sip->key, db->symbols);
+	if (spk == (spkey_t)0)
+		return NULL;
+	spl = sp_lookup(spk, db);
 	if (spl == NULL)
 		return NULL;
 	if (spl->data == NULL)
@@ -80,6 +107,9 @@ close_core(sip)
 	db = open_core(sip);
 	if (db == NULL)
 		return;
+	if (db->symbols != NULL) {
+		symbol_null_db(db->symbols);
+	}
 	if (db != spt_loginmap)
 		sp_scan(icfreedata, (struct spblk *)NULL, db);
 	sp_null(db);
@@ -89,6 +119,40 @@ close_core(sip)
  * Add the indicated key/value pair to the database.
  */
 
+int
+add_incoresp(name, value, db)
+     const char *name, *value;
+     struct sptree *db;
+{
+	search_info si;
+	si.subtype = (void*) db;
+	si.key     = name;
+
+	return add_core( &si, value );
+}
+
+int
+addd_incoresp(name, value, db)
+	const char *name;
+	const void *value;
+	struct sptree *db;
+{
+	struct spblk *spl;
+	spkey_t spk;
+
+	if (db->symbols == NULL)
+		db->symbols = sp_init();
+	spk = symbol_db(name, db->symbols);
+	spl = sp_lookup(spk, db);
+	if (spl == NULL)
+		sp_install(spk, (const void*)value, 0, db);
+	else {
+		icfreedata(spl);
+		spl->data = (const void *) value;
+	}
+	return 0;
+}
+
 int
 add_core(sip, value)
 	search_info *sip;
@@ -96,6 +160,7 @@ add_core(sip, value)
 {
 	struct sptree *db;
 	struct spblk *spl;
+	spkey_t spk;
 	memtypes oval;
 
 	db = open_core(sip);
@@ -104,15 +169,16 @@ add_core(sip, value)
 
 	oval = stickymem;
 	stickymem = MEM_MALLOC;
-	if (*value == '\0')
+	if (value == NULL || *value == '\0')
 		value = NULL;
 	else
 		value = strsave(value);
 	stickymem = oval;
 
-	spl = sp_lookup(symbol(sip->key), db);
+	spk = symbol_db(sip->key, db->symbols);
+	spl = sp_lookup(spk, db);
 	if (spl == NULL)
-		sp_install(symbol(sip->key), (const void*)value, 0, db);
+		sp_install(spk, (const void*)value, 0, db);
 	else {
 		icfreedata(spl);
 		spl->data = (const void *) value;
@@ -131,11 +197,15 @@ remove_core(sip)
 {
 	struct sptree *db;
 	struct spblk *spl;
+	spkey_t spk;
 
 	db = open_core(sip);
 	if (db == NULL)
 		return EOF;
-	spl = sp_lookup(symbol(sip->key), db);
+	spk = symbol_lookup_db(sip->key, db->symbols);
+	if (spk == (spkey_t)0)
+		return EOF;
+	spl = sp_lookup(spk, db);
 	if (spl == NULL) {
 		fprintf(stderr, "remove_core: no such key as \"%s\"!\n",
 				sip->key);
@@ -143,6 +213,7 @@ remove_core(sip)
 	}
 	icfreedata(spl);
 	sp_delete(spl, db);
+	symbol_free_db(sip->key, db->symbols);
 	return 0;
 }
 
