@@ -27,6 +27,8 @@ extern void seq_remap __((struct file_map *, long newsize));
 extern char *mfgets __((char *, int, struct file_map *));
 #endif
 
+extern struct spblk * _open_seq __((search_info *, const char *));
+
 /*
  * Binary search of a file for keyword-value pairs.
  */
@@ -52,56 +54,21 @@ search_bin(sip)
 	  return NULL;
 
 	retry = 0;
-	symid = symbol(sip->file);
-	spl   = sp_lookup(symid, spt_files);
-	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL) {
-#ifndef HAVE_MMAP
-	reopen:
-#endif
-	  fp = fopen(sip->file, "r");
-	  if (fp == NULL) {
-	    ++deferit;
-	    v_set(DEFER, DEFER_IO_ERROR);
-	    fprintf(stderr, "search_bin: cannot open %s!\n",
-		    sip->file);
-	    return NULL;
-	  }
-	  fm = (struct file_map *)emalloc(sizeof(struct file_map));
-	  fm->fp = fp;
-#ifdef	HAVE_MMAP
-	  fstat(FILENO(fp),&fst);
-	  fm->size  = fst.st_size;
-	  fm->mtime = fst.st_mtime;
-	  fm->lines = 0;
-	  fm->offsets = NULL;
-	  if (fm->size)
-	    fm->membuf = (void*)mmap(NULL, fst.st_size,
-				     PROT_READ, MAP_SHARED,
-				     FILENO(fp), 0);
-	  else
-	    fm->membuf = NULL;
-	  if ((int)fm->membuf == -1) {
-	    fprintf(stderr,
-		    "search_bin: cannot mmap() (r/o) %ld bytes of file \"%s\" into memory, errno=%d",
-		    (long)fst.st_size, sip->file, errno);
-	    free(fm);
-	    ++deferit;
-	    v_set(DEFER, DEFER_IO_ERROR);
-	    return NULL;
-	  }
-#else
-	  fm->size   = 0;
-	  fm->mtime  = 0;
-	  fm->lines = 0;
-	  fm->offsets = NULL;
-	  fm->membuf = NULL;
-#endif
-	  if (spl == NULL)
-	    sp_install(symid, (void *)fm, DESC_FILEP, spt_files);
-	  else
-	    spl->data = (void *)fm;
-	}
+
+reopen:
+
+	spl = _open_seq(sip, "r");
+
+	if (spl == NULL)
+	  return NULL;
+
+	fm = (struct file_map *)(spl->data);
+
+	if (fm == NULL)
+	  return NULL; /* Huh !? */
+
 	fp = fm->fp;
+
 #ifdef	HAVE_MMAP
 	/* This  fstat()  for possible seq_remap() trigger causes a bit
 	   more syscalls, than is really necessary.   Therefore it is
@@ -191,9 +158,10 @@ search_bin(sip)
 	    c = getc(fp);
 	    mid++;
 	  } while (!ferror(fp) && c != EOF && c != '\n');
+
 	  if (fgets(fixbuf, sizeof fixbuf, fp) == NULL) {
 	    if (!retry && ferror(fp)) {
-	      fclose(fp);
+	      close_seq(sip);
 	      ++retry;
 	      goto reopen;
 	    }
