@@ -33,6 +33,8 @@ extern char *mfgets __((char *, int, struct file_map *));
  * Linear search of a file for keyword-value pairs.
  */
 
+extern struct spblk * _open_seq __((search_info *, const char *));
+
 conscell *
 search_seq(sip)
 	search_info *sip;
@@ -48,58 +50,25 @@ search_seq(sip)
 #ifdef	HAVE_MMAP
 	struct stat fst;
 #endif
+	int i;
 
 	if (sip->file == NULL)
 		return NULL;
 	retry = 0;
-	symid = symbol(sip->file);
-	spl = sp_lookup(symid, spt_files);
-	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL) {
+
 reopen:
-		fp = fopen(sip->file, "r");
-		if (fp == NULL) {
-			++deferit;
-			v_set(DEFER, DEFER_IO_ERROR);
-			fprintf(stderr, "search_seq: cannot open %s!\n",
-					sip->file);
-			return NULL;
-		}
-		fm = (struct file_map *)emalloc(sizeof(struct file_map));
-		fm->fp = fp;
-#ifdef	HAVE_MMAP
-		fstat(FILENO(fp),&fst);
-		fm->size = fst.st_size;
-		fm->mtime = fst.st_mtime;
-		fm->lines = 0;
-		fm->offsets = NULL;
-		if (fm->size)
-		  fm->membuf = (void*)mmap(NULL, fst.st_size,
-					   PROT_READ, MAP_SHARED, FILENO(fp),
-					   0);
-		else
-		  fm->membuf = NULL;
-		if ((long)fm->membuf == -1) {
-		  fprintf(stderr,
-			  "search_seq: cannot mmap() (r/o) %d bytes of file \"%s\" into memory, errno=%d",
-			  fst.st_size, sip->file, errno);
-		  free(fm);
-		  ++deferit;
-		  v_set(DEFER, DEFER_IO_ERROR);
-		  return NULL;
-		}
-#else
-		fm->size   = 0;
-		fm->mtime  = 0;
-		fm->lines = 0;
-		fm->offsets = NULL;
-		fm->membuf = NULL;
-#endif
-		if (spl == NULL)
-		  spl = sp_install(symid, (void *)fm, O_RDONLY, spt_files);
-		else
-		  spl->data = (void *)fm;
-	}
+	spl = _open_seq(sip, "r");
+
+	if (spl == NULL)
+	  return NULL;
+
+	fm = (struct file_map *)(spl->data);
+
+	if (fm == NULL)
+	  return NULL; /* Huh !? */
+
 	fp = fm->fp;
+
 #ifdef	HAVE_MMAP
 	/* This  fstat()  for possible seq_remap() trigger causes a bit
 	   more syscalls, than is really necessary.   Therefore it is
@@ -142,16 +111,7 @@ reopen:
 		}
 	}
 	if (!retry && ferror(fp)) {
-		fclose(fp);
-#ifdef	HAVE_MMAP
-		/* We are throwing things away... */
-		if (fm->membuf != NULL)
-		  munmap((void*)fm->membuf,fm->size);
-#endif
-		if (fm->offsets != NULL)
-		  free(fm->offsets);
-		free(fm);
-		spl->data = NULL;
+		close_seq(sip);
 		++retry;
 		goto reopen;
 	}
@@ -173,11 +133,18 @@ close_seq(sip)
 
 	if (sip->file == NULL)
 		return;
-	symid = symbol(sip->file);
+
+	if (spt_files == NULL)
+	  spt_files = sp_init();
+	if (spt_files->symbols == NULL)
+	  spt_files->symbols = sp_init();
+
+	symid = symbol_db(sip->file, spt_files->symbols);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL)
 		sp_delete(spl, spt_modcheck);
 	spl = sp_lookup(symid, spt_files);
+
 	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL)
 		return;	/* nothing to flush */
 	spl->data = NULL;	/* Delete the file-entry */
@@ -192,9 +159,9 @@ close_seq(sip)
 	free(fm);
 }
 
-static FILE * open_seq __((search_info *, const char *));
-static FILE *
-open_seq(sip, mode)
+
+struct spblk *
+_open_seq(sip, mode)
 	search_info *sip;
 	const char *mode;
 {
@@ -208,7 +175,13 @@ open_seq(sip, mode)
 
 	if (sip->file == NULL)
 		return NULL;
-	symid = symbol(sip->file);
+
+	if (spt_files == NULL)
+	  spt_files = sp_init();
+	if (spt_files->symbols == NULL)
+	  spt_files->symbols = sp_init();
+
+	symid = symbol_db(sip->file, spt_files->symbols);
 	spl = sp_lookup(symid, spt_files);
 	if (spl != NULL
 	    && (*mode == 'w' || *mode == 'a'
@@ -219,7 +192,14 @@ open_seq(sip, mode)
 	} else
 		imode = O_RDONLY;
 	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL) {
-		FILE *fp = fopen(sip->file, mode);
+		FILE *fp;
+		int i;
+		for (i = 0; i < 3; ++i) {
+		  fp = fopen(sip->file, mode);
+		  if (fp != NULL)
+		    break;
+		  sleep(1); /* Retry a few times */
+		}
 		if (fp == NULL) {
 			++deferit;
 			v_set(DEFER, DEFER_IO_ERROR);
@@ -254,9 +234,31 @@ open_seq(sip, mode)
 		else
 			spl->data = (void *)fm;
 	}
-	return fm->fp;
+	return spl;
 }
 
+static FILE *
+open_seq(sip, mode)
+	search_info *sip;
+	const char *mode;
+{
+	struct file_map *fm;
+	struct spblk *spl;
+
+	spl = _open_seq(sip, mode);
+
+	if (spl == NULL)
+	  return NULL;
+
+	fm = (struct file_map *)(spl->data);
+
+	if (fm == NULL)
+	  return NULL; /* Huh !? */
+
+	return (fm->fp);
+}
+
+
 /*
  * Add the indicated key/value pair to the list.
  */
@@ -267,6 +269,7 @@ add_seq(sip, value)
 	const char *value;
 {
 	FILE *fp;
+	int rc;
 
 	fp = open_seq(sip, "r+");
 	if (fp == NULL)
@@ -276,7 +279,9 @@ add_seq(sip, value)
 		fprintf(fp, "%s\n", sip->key);
 	else
 		fprintf(fp, "%s\t%s\n", sip->key, value);
-	return fflush(fp);
+	rc = fflush(fp);
+	close_seq(sip);
+	return rc;
 }
 
 /*
@@ -322,7 +327,7 @@ count_seq(sip, outfp)
 	if (fp != NULL) {
 #ifdef HAVE_MMAP_not
 	  struct file_map *fm;
-	  spkey_t symid     = symbol(sip->file);
+	  spkey_t symid     = symbol_db(sip->file, spt_files->symbols);
 	  struct spblk *spl = sp_lookup(symid, spt_files);
 	  int cnt;
 
@@ -341,6 +346,8 @@ count_seq(sip, outfp)
 	  }
 	  cnt = fm->lines;
 #else
+	  /* Essentially: 'wc -l' -- count newlines */
+
 	  fseek(fp, (off_t)0, 0);
 	  while ((n = fread(buf, 1, sizeof buf, fp)) > 0) {
 	    while (n >= 0) {
@@ -405,7 +412,7 @@ modp_seq(sip)
 	if (stbuf2.st_nlink == 0) /* Changed underneath of us! */
 		return 1;
 
-	symid = symbol(sip->file);
+	symid = symbol_db(sip->file, spt_files->symbols);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
 		rval = ((long)stbuf.st_mtime != (long)spl->data ||
@@ -482,16 +489,22 @@ readchunk(file, foffset)
 	spkey_t symid;
 	char buf[BUFSIZ];
 	struct file_map *fm;
+	int i;
 
 	if (file == NULL || foffset < 0)
 		return NULL;
 	
 	retry = 0;
-	symid = symbol(file);
+	symid = symbol_db(file, spt_files->symbols);
 	spl = sp_lookup(symid, spt_files);
 	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL) {
 reopen:
-		fp = fopen(file, "r");
+		for (i = 0; i < 3; ++i) {
+		  fp = fopen(file, "r");
+		  if (fp != NULL)
+		    break;
+		  sleep(1);
+		}
 		if (fp == NULL) {
 			++deferit;
 			v_set(DEFER, DEFER_IO_ERROR);
