@@ -279,7 +279,7 @@ makeLetter(e, octothorp)
 				h->h_lines->t_type = Line;
 			} else if (inheader && n > 1) {
 				/* append to the header we just saw */
-				struct token *t;
+				token822 *t;
 				if (e->e_headers == NULL) {
 				  /* Wow, continuation without previous header! */
 				  /* It must be body.. */
@@ -401,7 +401,7 @@ void
 dumpHeader(h)
 	struct header *h;
 {
-	struct token	*t;
+	token822	*t;
 	struct address	*a;
 	struct addr	*p;
 
@@ -996,7 +996,7 @@ mkTrace(e)
 	/* for */
 	th->h_contents.r->r_for = NULL;
 	/* time */
-	th->h_contents.r->r_time = e->e_nowtime;
+	th->h_contents.r->r_time = e->e_statbuf.st_mtime;
 	/* looks like:sprintf(buf, "from %s by %s via %s with %s id %s; %s"); */
 	th->h_contents.r->r_convert = NULL;
 	return th;
@@ -1115,10 +1115,11 @@ sequencer(e, file)
 	char	       *ofpname, *path, *qpath;
 	conscell       *l, *routed_addresses, *sender, *to;
 	struct rwmatrix *rwhead, *nsp, *rwp = NULL, *rcp = NULL;
-	struct token   *t = NULL;
+	token822   *t = NULL;
 	int   idnumber, nxor, i;
 	int   def_uid, header_error, perr;
 	FILE	       *ofp, *vfp;
+	int		ofperrors = 0;
 	char  vbuf[2048], verbosefile[1024];
 	const char     *envid;
 	const char     *notaryret;
@@ -1531,31 +1532,22 @@ sequencer(e, file)
 		if (ph == e->e_eHeaders)
 			return PERR_NORECIPIENTS;
 	}
-	dprintf("Nuke Bcc headers if any\n");
-	oh = NULL;
-	for (h = e->e_headers; h != NULL; oh = h, h = h->h_next) {
-	  if (h->h_descriptor->hdr_name == NULL)
-	    continue;
-	  if ((h->h_descriptor->class == normal
-	       && CISTREQ(h->h_descriptor->hdr_name,"bcc"))
-	      || (h->h_descriptor->class == Resent
-		  && CISTREQ(h->h_descriptor->hdr_name, "resent-bcc"))) {
-	    if (oh == NULL)
-	      e->e_headers = h->h_next;
-	    else
-	      oh->h_next = h->h_next;
-	  }
-	}
 
-	dprintf("Nuke Return-Path headers if any\n");
+	dprintf("Nuke Bcc/Return-Path/X-Orcpt/X-Envid headers, if any\n");
 	oh = NULL;
 	for (h = e->e_headers; h != NULL; oh = h, h = h->h_next) {
 	  if (h->h_descriptor->hdr_name == NULL)
 	    continue;
-	  if ((h->h_descriptor->class == normal
-	       && CISTREQ(h->h_descriptor->hdr_name,"return-path"))
-	      || (h->h_descriptor->class == Resent
-		  && CISTREQ(h->h_descriptor->hdr_name,"resent-return-path"))){
+	  if ((h->h_descriptor->class == normal &&
+	       (CISTREQ(h->h_descriptor->hdr_name,"bcc") ||
+		CISTREQ(h->h_descriptor->hdr_name,"return-path") ||
+		CISTREQ(h->h_descriptor->hdr_name,"x-orcpt")     ||
+		CISTREQ(h->h_descriptor->hdr_name,"x-envid")        ))
+	      || (h->h_descriptor->class == Resent &&
+		  (CISTREQ(h->h_descriptor->hdr_name,"resent-BCC")         ||
+		   CISTREQ(h->h_descriptor->hdr_name,"resent-return-path") ||
+		   CISTREQ(h->h_descriptor->hdr_name,"resent-x-orcpt")     ||
+		   CISTREQ(h->h_descriptor->hdr_name,"resent-x-envid")    ))) {
 	    if (oh == NULL)
 	      e->e_headers = h->h_next;
 	    else
@@ -1697,7 +1689,7 @@ sequencer(e, file)
 			return PERR_DEFERRED;
 	}
 
-	dprintf("Crossbar applied to all (sender,routed-address) pairs\n");
+	dprintf("Crossbar to be applied to all (sender,routed-address) pairs\n");
 
 	{
 	  conscell *tmp;
@@ -1874,7 +1866,7 @@ sequencer(e, file)
 			h->h_contents.a->a_tokens->p_tokens->t_type = Atom;
 		printToken(verbosefile, verbosefile + sizeof verbosefile,
 			   h->h_contents.a->a_tokens->p_tokens,
-			   (struct token *)NULL, 0);
+			   (token822 *)NULL, 0);
 		/*
 		 * We have to be careful how we open this file, since one might
 		 * imagine someone trying to append to /etc/passwd using this
@@ -1978,6 +1970,8 @@ sequencer(e, file)
 	 */
 
 	for (h = e->e_headers; h != NULL; h = h->h_next) {
+		ofperrors |= ferror(ofp);
+		if (ofperrors) break; /* Sigh.. */
 		if (h->h_descriptor->hdr_name == NULL
 		    || !CISTREQ(h->h_descriptor->hdr_name, "obsoletes"))
 			continue;
@@ -1995,6 +1989,8 @@ sequencer(e, file)
 	 */
 
 	for (h = e->e_headers; h != NULL; h = h->h_next) {
+		ofperrors |= ferror(ofp);
+		if (ofperrors) break; /* Sigh.. */
 		if (h->h_descriptor->hdr_name == NULL
 		    || !CISTREQ(h->h_descriptor->hdr_name, "turnme"))
 			continue;
@@ -2008,8 +2004,14 @@ sequencer(e, file)
 
 	for (rwp = rwhead; rwp != NULL; rwp = rwp->next) {
 
+		ofperrors |= ferror(ofp);
+		if (ofperrors) break; /* Sigh.. */
+
 		for (nsp = rwp->down; nsp != NULL; nsp = nsp->next) {
 
+			ofperrors |= ferror(ofp);
+			if (ofperrors) break; /* Sigh.. */
+
 			if (!iserrmessage()  &&  nsp->errto != NULL) {
 				/* print envelope sender address */
 				putc(_CF_ERRORADDR, ofp);
@@ -2034,6 +2036,10 @@ sequencer(e, file)
 			}
 			/* print recipient addresses */
 			for (rcp = nsp->down; rcp != NULL; rcp = rcp->next) {
+
+				ofperrors |= ferror(ofp);
+				if (ofperrors) break; /* Sigh.. */
+
 				if (rcp->urw.number > 0)
 					putc(_CF_XORECIPIENT, ofp);
 				else
@@ -2079,6 +2085,10 @@ sequencer(e, file)
 		   the corresponding fields as stored with the rewrite set. */
 		nh = rwp->urw.h;
 		for (h = e->e_headers; h != NULL; h = h->h_next) {
+
+			ofperrors |= ferror(ofp);
+			if (ofperrors) break; /* Sigh.. */
+
 			if (nh != NULL
 			    && (h->h_descriptor->user_type == Sender
 			     || h->h_descriptor->user_type == Recipient)) {
@@ -2122,7 +2132,9 @@ sequencer(e, file)
 #ifdef HAVE_FSYNC
 	fsync(FILENO(ofp));
 #endif
-	if ((fclose(ofp) != 0) || (erename(file, qpath) != 0)) {
+	ofperrors |= ferror(ofp);
+
+	if ((fclose(ofp) != 0) || ofperrors || (erename(file, qpath) != 0)) {
 	  unlink(qpath);
 	  unlink(ofpname);
 #ifndef	USE_ALLOCA
