@@ -38,12 +38,12 @@ hdr_scanparse(e, h, commentflag, no_line_crossing)
 	struct header *h;
 	int commentflag, no_line_crossing;
 {
-	register struct token *t;
+	register token822 *t;
 	long		 len;
 	const char	*cp, *ocp;
 	char		 c1, c2;
 	union misc	 retval;
-	struct token	*tlist, **prev_tp, *scan_t, *nt;
+	token822	*tlist, **prev_tp, *scan_t, *nt;
 	struct headerinfo *hd;
 
 	/* If we don't recognize the header, we can't parse it */
@@ -411,7 +411,7 @@ hdr_print(h, fp)
 	FILE *fp;
 {
 	int col, addrlen, newline;
-	struct token *t;
+	token822 *t;
 	struct address *ap;
 	struct addr *pp;
 
@@ -613,7 +613,7 @@ pureAddress(fp, pp)
 	FILE *fp;
 	register struct addr *pp;
 {
-	register struct token *t;
+	register token822 *t;
 
 	for (; pp != NULL ; pp = pp->p_next) {
 		if (pp->p_type != anAddress)
@@ -654,7 +654,7 @@ printAddress(fp, pp, onlylength)
 	int onlylength;
 {
 	int inAddress;
-	struct token *t;
+	token822 *t;
 	struct addr *lastp, *tpp;
 
 	inAddress = 0;
@@ -728,7 +728,7 @@ printLAddress(fp, pp, col)
 	int col;
 {
 	int inAddress;
-	struct token *t;
+	token822 *t;
 	struct addr *lastp, *tpp;
 
 	inAddress = 0;
@@ -813,7 +813,7 @@ saveAddress(pp)
 {
 	int len, totallen, inAddress;
 	char *cp, *buf;
-	struct token *t;
+	token822 *t;
 	struct addr *lastp, *tpp;
 
 	totallen = printAddress((FILE *)NULL, pp, 1);
@@ -895,9 +895,9 @@ errprint(fp, pp, hdrlen)
 	int hdrlen;
 {
 	int inAddress, n, i, j, len;
-	struct token *t;
+	token822 *t;
 	struct addr *lastp, *tpp;
-	struct { int pos; struct token *tokens; } errmsg[200];
+	struct { int pos; token822 *tokens; } errmsg[200];
 
 	inAddress = 0;
 	for (lastp = NULL, tpp = pp; tpp != NULL; tpp = tpp->p_next)
