@@ -341,7 +341,7 @@ initialize(configfile, argc, argv)
 	struct group *grp;
 	struct sptree_init *sptip;
 	int ac;
-	char **cpp;
+	const char **cpp;
 	const char **av;
 
 	av = (const char **)emalloc((5+argc)*(sizeof (char *)));
@@ -386,7 +386,7 @@ initialize(configfile, argc, argv)
 
 	/* trusted users */
 	for (cpp = default_trusted; cpp != NULL && *cpp != NULL; ++cpp)
-		sp_install(symbol(*cpp), NULL, 0, spt_goodguys);
+		add_incoresp(*cpp, "", spt_goodguys);
 
 	if (files_group != NULL) {
 		if ((grp = getgrnam(files_group)) == NULL)
@@ -421,7 +421,7 @@ login_to_uid(name)
 	char *cp, *fn;
 	struct spblk *spl;
 
-	spl = sp_lookup(symbol(name), spt_loginmap);
+	spl = lookup_incoresp(name, spt_loginmap);
 	if (spl == NULL) {
 		memtypes oval = stickymem;
 
@@ -434,10 +434,9 @@ login_to_uid(name)
 			cp = strsave(pw->pw_name);
 			sp_install(uid, cp, 0, spt_uidmap);
 			fullname(pw->pw_gecos,buf,sizeof buf,pw->pw_name);
-			fn = strsave(buf);
-			sp_install(symbol(cp), fn, 0, spt_fullnamemap);
+			add_incoresp(cp, buf, spt_fullnamemap);
 		}
-		sp_install(symbol(name), (void *)((long)uid), 0, spt_loginmap);
+		addd_incoresp(name, (void*)((long)uid), spt_loginmap);
 		stickymem = oval;
 	} else
 		uid = (int)(spl->data);
@@ -450,7 +449,6 @@ uidpwnam(uid)
 {
 	struct passwd *pw;
 	register const char *cp;
-	char *fn;
 	struct spblk *spl;
 	char buf[BUFSIZ];
 
@@ -467,11 +465,9 @@ uidpwnam(uid)
 			memtypes oval = stickymem;
 			stickymem = MEM_MALLOC;
 			cp = strsave(pw->pw_name);
-			sp_install(symbol(cp), (void*)((long)pw->pw_uid), 0,
-				   spt_loginmap);
-			fullname(pw->pw_gecos,buf,sizeof buf,pw->pw_name);
-			fn = strsave(buf);
-			sp_install(symbol(cp), fn, 0, spt_fullnamemap);
+			addd_incoresp(pw->pw_name, (void*)((long)(pw->pw_uid)), spt_loginmap);
+			fullname(pw->pw_gecos,buf, sizeof buf, pw->pw_name);
+			add_incoresp(pw->pw_name, buf, spt_fullnamemap);
 			sp_install((u_int)uid, cp, 0, spt_uidmap);
 			stickymem = oval;
 		}
@@ -487,13 +483,19 @@ int
 isgoodguy(uid)
 	int	uid;
 {
+	const char *name;
+	spkey_t spk;
 	struct spblk *spl;
 
 	/*
 	 * If you're wondering about this comparison... I had to store
 	 * *something* in the splay tree; I'm really using it as a boolean.
 	 */
-	spl = sp_lookup(symbol(uidpwnam(uid)), spt_goodguys);
+	name = uidpwnam(uid);
+	spk = symbol_lookup_db(name, spt_goodguys->symbols);
+	spl = NULL;
+	if (spk != (spkey_t)0)
+	  spl = sp_lookup(spk, spt_goodguys);
 	return spl != NULL;
 }
 
