@@ -48,8 +48,8 @@ extern char *qlogdir;
 extern char *qcf_suffix;
 extern char *qdefaultdir;
 extern char *qoutputfile;
-extern char *replhost;
-extern char *replchannel;
+extern const char *replhost;
+extern const char *replchannel;
 extern int  nobody;
 extern int  sweepinterval;
 
