@@ -1,22 +1,22 @@
 /*
- *	Copyright 1994 Matti Aarnio, all rights reserved.
+ *	Copyright 1994-1996 Matti Aarnio, all rights reserved.
  *	This module is a part of Zmailer, and subject to its
  *	distribution rules.
  */
 
 
+#include "hostenv.h"
 #include <stdio.h>
 #include <sys/types.h>
-#include "hostenv.h"
 
-#if	defined(__linux__) || defined(__svr4__) || defined(_SYSTYPE_BSD)
+#ifdef HAVE_SETRLIMIT
 /* ================================================================ */
 
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 
-/* #include "prototypes.h" /* Scheduler's prototypes */
+/* #include "prototypes.h" */ /* Scheduler's prototypes */
 
 int resources_query_nofiles()
 {
@@ -30,7 +30,7 @@ int resources_query_nofiles()
 	else
 	  return getdtablesize();
 #else
-	(int) sysconf(_SC_OPEN_MAX);
+	return sysconf(_SC_OPEN_MAX);
 #endif
 }
 
@@ -63,74 +63,15 @@ int nfiles;
 #endif
 }
 
-int
-resources_query_pipesize(fildes)
-int fildes;
-{
-	return (int) fpathconf(fildes, _PC_PIPE_BUF);
-}
-
-#else /* Not POSIX, not __linux__, nor __svr4__ */
-#if	defined(BSD) || defined(sun)
-/* ================================================================ */
-
-#include <unistd.h>
-#include <sys/time.h>
-#include <sys/resource.h>
-
-int
-resources_query_nofiles()
-{
-	struct rlimit rl;
-	int rc;
-
-	rc = getrlimit(RLIMIT_NOFILE,&rl);
-	if (rc != 0) return getdtablesize();
-
-	return rl.rlim_cur;
-}
-
-void
-resources_maximize_nofiles()
-{
-	struct rlimit rl;
-	int rc;
-
-	rc = getrlimit(RLIMIT_NOFILE,&rl);
-	if (rc != 0) return;
-	rl.rlim_cur = rl.rlim_max;
-	setrlimit(RLIMIT_NOFILE,&rl);
-}
-
-void
-resources_limit_nofiles(nfiles)
-int nfiles;
-{
-	struct rlimit rl;
-	int rc;
-
-	rc = getrlimit(RLIMIT_NOFILE,&rl);
-	if (rc != 0) return;
-	rl.rlim_cur = nfiles;
-	setrlimit(RLIMIT_NOFILE,&rl);
-}
-
-int
-resources_query_pipesize(fildes)
-int fildes;
-{
-	return 4096;	/* pipe buffer size (max contents before blocking) */
-}
-
-#else	/* No POSIX stuff, no BSD, ??? */
-#if	!defined(__linux__) && (defined(_POSIX_SOURCE))
+#else
+#ifdef HAVE_SYSCONF
 /* ================================================================ */
 
 #include <unistd.h>
 int
 resources_query_nofiles()
 {
-	(int) sysconf(_SC_OPEN_MAX);
+	return sysconf(_SC_OPEN_MAX);
 }
 
 void
@@ -144,13 +85,6 @@ int nfiles;
 {
 }
 
-int
-resources_query_pipesize(fildes)
-int fildes;
-{
-	return (int) fpathconf(fildes, _PC_PIPE_BUF);
-}
-
 #else
 /* ================================================================ */
 
@@ -171,6 +105,17 @@ int nfiles;
 {
 }
 
+#endif
+#endif
+
+#ifdef HAVE_FPATHCONF
+int
+resources_query_pipesize(fildes)
+int fildes;
+{
+	return (int) fpathconf(fildes, _PC_PIPE_BUF);
+}
+#else
 int
 resources_query_pipesize(fildes)
 int fildes;
@@ -178,5 +123,3 @@ int fildes;
 	return 4096;	/* pipe buffer size (max contents before blocking) */
 }
 #endif
-#endif
-#endif
