@@ -324,6 +324,16 @@ main(argc, argv)
 	long offout, offerr;
 	struct config_entry *tp;
 
+	char *t, *syslogflg = getzenv("SYSLOGFLG");
+	if (syslogflg == NULL)
+	  syslogflg = "";
+	t = syslogflg;
+	for ( ; *t ; ++t ) {
+	  if (*t == 'c' || *t == 'C')
+	    break;
+	}
+	do_syslog = (*t != '\0');
+
 #ifdef HAVE_SETGROUPS
 	/* We null supplementary groups list entirely */
 	setgroups(0, NULL);
@@ -549,8 +559,10 @@ main(argc, argv)
 	  /* NOTREACHED */
 	}
 
-	for (i = 0; i < SIZE_L; ++i)
+	for (i = 0; i < SIZE_L; ++i) {
 	  spt_mesh[i] = sp_init();
+	  spt_mesh[i]->symbols = sp_init();
+	}
 	zopenlog("scheduler", LOG_PID, LOG_MAIL);
 	if (optind < argc) {
 	  /* process the specified control files only */
