@@ -179,6 +179,15 @@ struct config_entry *cep;
 	thr->wchan   = vtx->orig[L_CHANNEL];
 	thr->whost   = vtx->orig[L_HOST];
 
+	if (thgrp->cep->flags & CFG_QUEUEONLY) {
+		/* Start with the first retry */
+		thr->attempts = 1;
+		if(thgrp->cep->nretries) {
+			mytime(&now);
+			thr->wakeup = now + thgrp->cep->retries[0];
+		}
+	}
+
 	if (thgrp->thread == NULL) {
 	  thgrp->thread = thr;
 	} else {
@@ -439,7 +448,9 @@ if (verbose) printf("thread_linkin() to thg=0x%p[%s/%d/%s]; added into existing
 	    vp->thgrp = thg;
 	    thr->jobs += 1;
 	    /* Hookay..  Try to start it, in case it isn't yet running */
-	    thread_start(thr);
+	    if(!(thg->cep->flags & CFG_QUEUEONLY)) {
+		thread_start(thr);
+	    }
 
 	    return;
 	  }
@@ -455,7 +466,9 @@ if (verbose) printf("thread_linkin() to thg=0x%p[%s/%d/%s]; created a new thread
 
 
 	  /* Try to start it too */
-	  thread_start(thr);
+	  if(!(thg->cep->flags & CFG_QUEUEONLY)) {
+		thread_start(thr);
+	  }
 	  return;
 	}
 
@@ -468,7 +481,9 @@ if (verbose) printf("thread_linkin() to thg=0x%p[%s/%d/%s]; created a new thread
 		    thg,cep->channel,thg->withhost,cep->host,
 		    wc->name,wh->name);
 	/* Try to start it too */
-	thread_start(thr);
+	if(!(thg->cep->flags & CFG_QUEUEONLY)) {
+		thread_start(thr);
+	}
 }
 
 struct web *
@@ -478,15 +493,17 @@ const char *s;
 {
 	struct spblk *spl;
 	struct web *wp;
+	spkey_t spk;
 
 	/* caller has done 'strlower()' to our input.. */
 
-	spl = sp_lookup(symbol(s), spt_mesh[flag]);
+	spk = symbol_db(s, spt_mesh[flag]->symbols);
+	spl = sp_lookup(spk, spt_mesh[flag]);
 	if (spl == NULL || (wp = (struct web *)spl->data) == NULL) {
 	  /* Not found, create it */
 	  wp = (struct web *)emalloc(sizeof (struct web));
 	  memset((void*)wp, 0, sizeof (struct web));
-	  sp_install(symbol(s), (void *)wp, 0, spt_mesh[flag]);
+	  sp_install(spk, (void *)wp, 0, spt_mesh[flag]);
 	  wp->name     = strsave(s);
 	  wp->kids     = 0;
 	  wp->link     = NULL;
@@ -508,16 +525,21 @@ unweb(flag, wp)
 	int flag;
 	struct web *wp;
 {
-	struct spblk *spl;
+	struct spblk *spl = NULL;
+	spkey_t spk;
 
 	wp->link = wp->lastlink = NULL;
 	if (wp->kids != 0)
 	  return;		/* too early to actually remove it */
 	if (wp->linkcnt > 0)	/* Yet objects holding it */
 	  return;
-	spl = sp_lookup(symbol((u_char *)wp->name), spt_mesh[flag]);
-	if (spl != NULL)
+	spk = symbol_lookup_db((u_char *)wp->name, spt_mesh[flag]->symbols);
+	if ((spkey_t)0 == spk)	/* Not in the symbol table */
+	  return;
+	spl = sp_lookup(spk, spt_mesh[flag]);
+	if (spl != NULL)	/* Should always have this ... */
 	  sp_delete(spl, spt_mesh[flag]);
+	symbol_free_db((u_char *)wp->name, spt_mesh[flag]->symbols);
 	free(wp->name);
 	free((char *)wp);
 }
@@ -573,11 +595,12 @@ web_disentangle(vp, ok)
 	  vp->proc = NULL;
 	}
 
-	unthread(vp);
+	if (vp->thread != NULL)
+	  unthread(vp);
 
 	/* The thread can now be EMPTY! */
 
-	if (vp->thread->vertices == NULL)
+	if (vp->thread != NULL && vp->thread->vertices == NULL)
 	  delete_thread(vp->thread, ok);
 }
 
@@ -1033,6 +1056,7 @@ time_t retrytime;
 
 	/* if we are already scheduled for the future, don't reschedule */
 	if (vtx->wakeup > now) {
+	  thr->wakeup = vtx->wakeup;
 	  if (verbose)
 	    printf("...prescheduled\n");
 	  return;
@@ -1046,7 +1070,10 @@ time_t retrytime;
 	}
 
 	if (thr->retryindex >= vtx->thgrp->ce.nretries)
-	  thr->retryindex = ranny(vtx->thgrp->ce.nretries-1);
+	  if (vtx->thgrp->ce.nretries > 1)
+	    thr->retryindex = ranny(vtx->thgrp->ce.nretries-1);
+	  else
+	    thr->retryindex = 0;
 	
 	/*
 	 * clamp retry time to a predictable interval so we
@@ -1184,7 +1211,10 @@ reschedule(vp, factor, index)
 #endif
 	if (factor == -1 && vp->attempts) {
 	  if (thr->retryindex >= ce->nretries)
-	    thr->retryindex = ranny(ce->nretries-1);
+	    if (ce->nretries > 1)
+	      thr->retryindex = ranny(ce->nretries-1);
+	    else
+	      thr->retryindex = 0;
 
 	  /*
 	   * clamp retry time to a predictable interval so we
