@@ -24,13 +24,15 @@
 #include <unistd.h>
 
 #include "prototypes.h"
+#include "zsyslog.h"
+#include <sysexits.h>
 
 extern int forkrate_limit;
 extern int freeze;
 
 static int  scheduler_nofiles = -1; /* Will be filled below */
-static int  runcommand __((char **, struct vertex *, struct web *, struct web*));
-static void stashprocess __((int, int, int, struct web*, struct web*, struct vertex *));
+static int  runcommand __((const char **, struct vertex *, struct web *, struct web*));
+static void stashprocess __((int, int, int, struct web*, struct web*, struct vertex *, const char *argv[]));
 static void reclaim __((int, int));
 static void waitandclose __((int));
 static void readfrom __((int));
@@ -311,10 +313,6 @@ start_child(vhead, chwp, howp)
 		    strcpy(s,howp->name);
 		  } else if (strcmp(ocp,"channel")==0) {
 		    strcpy(s,chwp->name);
-		  } else if (strcmp(ocp,"LOGDIR")==0) {
-		    char *e = getzenv("LOGDIR");
-		    if (e == NULL) e = "/var/log/mail";
-		    strcpy(s, e);
 		  } else
 		    strcpy(s, getzenv(ocp));
 		  s += strlen(s);
@@ -339,11 +337,11 @@ start_child(vhead, chwp, howp)
 	    printf(" %s", av[i]);
 	  printf("\n");
 	}
-	return runcommand(av, vhead, chwp, howp);
+	return runcommand((const char **)av, vhead, chwp, howp);
 }
 
 static int runcommand(argv, vhead, chwp, howp)
-	char	*argv[];
+	const char *argv[];
 	struct vertex *vhead;
 	struct web *chwp, *howp;
 {
@@ -381,7 +379,7 @@ static int runcommand(argv, vhead, chwp, howp)
 	  resources_limit_nofiles(transportmaxnofiles);
 	  setgid(gid);	/* Do GID setup while still UID 0..   */
 	  setuid(uid);	/* Now discard all excessive powers.. */
-	  execv(cmd, argv);
+	  execv(cmd, (char**)argv);
 	  fprintf(stderr, "Exec of %s failed!\n", cmd);
 	  _exit(1);
 	} else if (pid < 0) {	/* fork failed - yell and forget it */
@@ -396,24 +394,21 @@ static int runcommand(argv, vhead, chwp, howp)
 	pipes_close_parent(to,from);
 
 	/* save from[0] away as a descriptor to watch */
-	stashprocess(pid, from[0], to[1], chwp, howp, vhead);
+	stashprocess(pid, from[0], to[1], chwp, howp, vhead, argv);
 	/* We wait for the child to report "#hungry", then we feed it.. */
 	return 1;
 }
 
 
-static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
+static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead, argv)
 	int pid, fromfd, tofd;
 	struct web *chwp, *howp;
 	struct vertex *vhead;
+	const char *argv[];
 {
-	int i;
+	int i, l, j;
 	struct procinfo *proc;
 
-	if (verbose)
-	  fprintf(stderr, "stashprocess(%d, %d, %d, %s, %s)\n",
-		  pid, fromfd, tofd, chwp ? chwp->name : "nil",
-		  howp ? howp->name : "nil");
 	if (cpids == NULL) {
 	  if (scheduler_nofiles < 1)
 	    scheduler_nofiles = resources_query_nofiles();
@@ -452,6 +447,33 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	fd_nonblockingmode(fromfd);
 	if (fromfd != tofd)
 	  fd_nonblockingmode(tofd);
+
+	/* Costruct a faximille of the argv[] in a single string.
+	   This is entirely for debug porposes in some rare cases
+	   where transport subprocess returns EX_SOFTWARE, and we
+	   send out LOG_EMERG alerts thru syslog.  */
+	proc->cmdline[0] = 0;
+	l = 0;
+	for (i = 0; argv[i] != NULL; ++i) {
+	  if (i > 0)
+	    proc->cmdline[l++] = ' ';
+	  j = strlen(argv[i]);
+	  if (l + j < (sizeof(proc->cmdline) - 2)) {
+	    memcpy(proc->cmdline+l, argv[i], j);
+	    l += j;
+	  } else {
+	    /* All will not fit in, we cut.. */
+	    j = sizeof(proc->cmdline) - 2 - l;
+	    memcpy(proc->cmdline + l, argv[i], j);
+	    break;
+	  }
+	}
+	proc->cmdline[l] = '\0';
+
+	if (verbose)
+	  fprintf(stderr, "stashprocess(%d, %d, %d, %s, %s, '%s')\n",
+		  pid, fromfd, tofd, chwp ? chwp->name : "nil",
+		  howp ? howp->name : "nil", proc->cmdline);
 }
 
 /*
@@ -1110,7 +1132,7 @@ int signum;
 	  if (verbose)
 	    fprintf(stderr,"sig_chld() pid=%d, ok=%d, stat=0x%x\n",
 		    pid,ok,statloc);
-	  
+
 	  if (ok && cpids != NULL) {
 	    /* Only EXIT and SIGxx DEATHS accepted */
 
@@ -1119,6 +1141,11 @@ int signum;
 		cpids[i].pid = -pid; /* Mark it as reaped.. */
 		cpids[i].reaped = 1;
 		ok = 0;
+		if (WSIGNALSTATUS(statloc) == 0 &&
+		    WEXITSTATUS(statloc) == EX_SOFTWARE) {
+		  zsyslog((LOG_EMERG, "Transporter process %d exited with EX_SOFTWARE!", pid));
+		  fprintf(stderr, "Transporter process %d exited with EX_SOFTWARE; cmdline='%s'\n", pid, cpids[i].cmdline);
+	  }
 		break;
 	      }
 	      if (cpids[i].pid == -pid) {
