@@ -110,124 +110,127 @@ update(fd, diagnostic)
 	  flush_child(proc);
 	  return;
 	}
-	if (*diagnostic == '#' /* DEBUG comment.. */) {
-	  if (strncmp(diagnostic,"#hungry",7)==0) {
-	    /* This is an "actor model" behaviour,
-	       where actor tells, when it needs a new
-	       job to be fed to it. */
-	    if (proc->tofd   >= 0) {
-	      proc->hungry = 1;
-	      mytime(&proc->hungertime);
-	      if (proc->overfed > 0)
-		/* It was overfed, decrement that counter first.. */
-		proc->overfed -= 1;
-	      if (!proc->overfed) {
-		++hungry_childs;
-		/* Unless it is still overfed,
-		   Pick next, and feed it! */
+	if (*diagnostic != '#') { /* Not debug diagnostic message */
+	  inum = atol(diagnostic);
+	  if ((cp = strchr(diagnostic, '/')) == NULL) {
+	    fprintf(stderr, "%s Misformed diagnostic: %s\n",
+		    timestring(), diagnostic);
+	    return;
+	  }
+	  offset = atol(++cp);
+	  if ((cp = strchr(cp, '\t')) == NULL) {
+	    fprintf(stderr, "%s Misformed diagnostic: %s\n",
+		    timestring(), diagnostic);
+	    return;
+	  }
+	  notary = ++cp;
+	  if ((cp = strchr(cp, '\t')) == NULL) {
+	    fprintf(stderr, "%s Misformed diagnostic: %s\n",
+		    timestring(), diagnostic);
+	    return;
+	  }
+	  *cp = 0; /* Trailing TAB after notary string, NULL it */
+	  type = ++cp;
+	  while (*cp != '\0' && isascii(*cp) && !isspace(*cp))
+	    ++cp;
+	  if (*cp == '\0') {
+	    message = NULL;
+	  } else {
+	    *cp++ = '\0';
+	    message = cp;
+	  }
+	  if (verbose)
+	    printf("diagnostic: %ld/%ld\t%s\t%s\n",inum,offset,notary,type);
+
+	  if ((vp = findvertex(inum, offset, &index)) == NULL)
+	    return;
+
+	  if (vp->notary != NULL)
+	    free(vp->notary);
+	  vp->notary = NULL;
+	  if (*notary)
+	    vp->notary = strsave(notary);
+
+	  /* Select function by the type name: ok/error/retryat/deferred */
+
+	  for (dcp = &diags[0]; dcp->name != NULL; ++dcp) {
+	    /* XX: replace strcmp() with cistrcmp() ??  Should not need,
+	       unless something is wrong with the transporters. */
+	    if (strcmp(dcp->name, type) == 0) {
+	      (dcp->fcn)(vp, index, inum, offset, notary, message);
+	      break;
+	    }
+	  }
+	  if (dcp->name == NULL)
+	    fprintf(stderr, "%s Unknown diagnostic type ignored: %s\n",
+		    timestring(), type);
+	  return;
+	}
+
+	/* Now (*diagnostic == '#') */
+	if (strncmp(diagnostic,"#hungry",7)==0) {
+	  /* This is an "actor model" behaviour,
+	     where actor tells, when it needs a new
+	     job to be fed to it. */
+	  if (proc->tofd   >= 0) {
+	    proc->hungry = 1;
+	    mytime(&proc->hungertime);
+	    if (proc->overfed > 0)
+	      /* It was overfed, decrement that counter first.. */
+	      proc->overfed -= 1;
+	    if (!proc->overfed) {
+	      ++hungry_childs;
+	      /* Unless it is still overfed,
+		 Pick next, and feed it! */
+	      pick_next_vertex(proc, 1, 0);
+	      /* While we have a thread, and things to feed.. */
+	      while (!proc->fed && proc->thread) {
+		if (proc->hungry)
+		  feed_child(proc);
+		if (proc->fed)
+		  proc->overfed += 1;
+		else
+		  break; /* Huh! Feed/flush failure! */
+		/* See if we should, and can feed more! */
+		if (proc->thg == NULL ||
+		    proc->pid == 0    ||
+		    proc->thread == NULL)
+		  break;	/* No new active threads/vertices/proc.. */
+		if (proc->overfed >= proc->thg->ce.overfeed)
+		  break;	/* if the limit is zero, don't overfeed ever.*/
+		/* Ok, increment the counter, and loop back.. */
+		proc->hungry = 1; /* Simulate hunger.. */
+		proc->fed = 1; /* tell them, it has been fed.. */
 		pick_next_vertex(proc, 1, 0);
-		/* While we have a thread, and things to feed.. */
-		while (!proc->fed && proc->thread) {
-		  if (proc->hungry)
-		    feed_child(proc);
-		  if (proc->fed)
-		    proc->overfed += 1;
-		  else
-		    break; /* Huh! Feed/flush failure! */
-		  /* See if we should, and can feed more! */
-		  if (proc->thg == NULL ||
-		      proc->pid == 0    ||
-		      proc->thread == NULL)
-		    break;	/* No new active threads/vertices/proc.. */
-		  if (proc->overfed >= proc->thg->ce.overfeed)
-		    break;	/* if the limit is zero, don't overfeed ever.*/
-		  /* Ok, increment the counter, and loop back.. */
-		  proc->hungry = 1; /* Simulate hunger.. */
-		  proc->fed = 1; /* tell them, it has been fed.. */
-		  pick_next_vertex(proc, 1, 0);
-		  /* If it got next,  ``proc->fed'' is now zero.. */
-		}
-		flush_child(proc);
-		proc->hungry = 0; /* ... satiated.. */
-	      } else {
-		if (verbose)
-		  printf("... child pid %d overfed=%d\n",
-			 proc->pid,proc->overfed);
+		/* If it got next,  ``proc->fed'' is now zero.. */
 	      }
-	    } else
+	      flush_child(proc);
+	      proc->hungry = 0; /* ... satiated.. */
+	    } else {
 	      if (verbose)
-		printf("'#hungry' from child without forward-channel\n");
-	    return;
-	  } /* end of '#hungry' processing */
-	  if (strncmp(diagnostic,"#resync",7)==0) {
-	    /* The transporter has noticed that the scheduler
-	       gave it a job spec, which does not have anything
-	       left for processing, it is time for the scheduler
-	       to recheck the job file. */
-	    char *p, *s = diagnostic + 7;
-	    while (*s == ' ' || *s == '\t') ++s;
-	    p = strchr(s,'\n');
-	    if (p) *p = 0; /* newline AFTER filename */
-	    if (*s != 0)
-	      resync_file(s);
-	    return;
-	  }
-	  fprintf(stderr, "%s DBGdiag: %s\n", timestring(), diagnostic);
+		printf("... child pid %d overfed=%d\n",
+		       proc->pid,proc->overfed);
+	    }
+	  } else
+	    if (verbose)
+	      printf("'#hungry' from child without forward-channel\n");
 	  return;
-	} /* end of debug-diagnostics processing */
-	inum = atol(diagnostic);
-	if ((cp = strchr(diagnostic, '/')) == NULL) {
-	  fprintf(stderr, "%s Misformed diagnostic: %s\n",
-		  timestring(), diagnostic);
+	} /* end of '#hungry' processing */
+	if (strncmp(diagnostic,"#resync",7)==0) {
+	  /* The transporter has noticed that the scheduler
+	     gave it a job spec, which does not have anything
+	     left for processing, it is time for the scheduler
+	     to recheck the job file. */
+	  char *p, *s = diagnostic + 7;
+	  while (*s == ' ' || *s == '\t') ++s;
+	  p = strchr(s,'\n');
+	  if (p) *p = 0; /* newline AFTER filename */
+	  if (*s != 0)
+	    resync_file(s);
 	  return;
 	}
-	offset = atol(++cp);
-	if ((cp = strchr(cp, '\t')) == NULL) {
-	  fprintf(stderr, "%s Misformed diagnostic: %s\n",
-		  timestring(), diagnostic);
-	  return;
-	}
-	notary = ++cp;
-	if ((cp = strchr(cp, '\t')) == NULL) {
-	  fprintf(stderr, "%s Misformed diagnostic: %s\n",
-		  timestring(), diagnostic);
-	  return;
-	}
-	*cp = 0; /* Trailing TAB after notary string, NULL it */
-	type = ++cp;
-	while (*cp != '\0' && isascii(*cp) && !isspace(*cp))
-	  ++cp;
-	if (*cp == '\0') {
-	  message = NULL;
-	} else {
-	  *cp++ = '\0';
-	  message = cp;
-	}
-	if (verbose)
-	  printf("diagnostic: %ld/%ld\t%s\t%s\n",inum,offset,notary,type);
-
-	if ((vp = findvertex(inum, offset, &index)) == NULL)
-	  return;
-
-	if (vp->notary != NULL)
-	  free(vp->notary);
-	vp->notary = NULL;
-	if (*notary)
-	  vp->notary = strsave(notary);
-
-	/* Select function by the type name: ok/error/retryat/deferred */
-
-	for (dcp = &diags[0]; dcp->name != NULL; ++dcp) {
-	  /* XX: replace strcmp() with cistrcmp() ??  Should not need,
-	     unless something is wrong with the transporters. */
-	  if (strcmp(dcp->name, type) == 0) {
-	    (dcp->fcn)(vp, index, inum, offset, notary, message);
-	    break;
-	  }
-	}
-	if (dcp->name == NULL)
-	  fprintf(stderr, "%s Unknown diagnostic type ignored: %s\n",
-		  timestring(), type);
+	fprintf(stderr, "%s DBGdiag: %s\n", timestring(), diagnostic);
+	return;
 }
 
 /*
@@ -250,8 +253,13 @@ unctlfile(cfp, no_unlink)
 	if (!no_unlink && !procselect) {
 	  reporterrs(cfp);
 
-	  if (do_syslog)
-	    zsyslog((LOG_INFO, "%s: done", cfp->mid));
+	  if (do_syslog) {
+	    char taspid[30],fnam[30];
+	    sprintf(fnam,"%lu",(long)cfp->id);
+	    taspoolid(taspid, sizeof(taspid), cfp->ctime, fnam);
+	    zsyslog((LOG_INFO, "%s: complete (total %d recepients, %d failed)",
+		     taspid, cfp->rcpnts_total, cfp->rcpnts_failed));
+	  }
 
 	  eunlink(path);
 	  if (verbose)
@@ -302,7 +310,8 @@ void unvertex(vp, justfree, ok)
 		  "unvertex(vtx=0x%p (thr=0x%p proc=0x%p, ng=%d) ,%d,%d)\n",
 		  vp,vp->thread,vp->proc,vp->ngroup,justfree,ok);
 
-	if (vp->thread->vertices == vp && vp->thread->proc &&
+	if (vp->thread != NULL &&
+	    vp->thread->vertices == vp && vp->thread->proc &&
 	    vp->thread->proc->vertex == vp) {
 	    /* Whoops, we are the active first on thread.. */
 	  if (vp->proc == NULL)
@@ -325,7 +334,8 @@ void unvertex(vp, justfree, ok)
 	    /* abort(); */
 	  }
 	}
-	if (vp->thread->proc && vp->thread->proc->vertex == vp) {
+	if (vp->thread != NULL &&
+	    vp->thread->proc && vp->thread->proc->vertex == vp) {
 	  /* XX: This is actually vestigal from somewhere, and
 	         should not occur at all.. */
 	  vp->thread->proc->vertex = NULL;
