@@ -19,6 +19,7 @@
 int is_in_toplevel(str)
      const char *str;
 {
+	return 1;
 }
 
 void add_to_toplevels(str)
@@ -88,6 +89,10 @@ char *str;
 	sscanf(param1, "%d", &MaxSameIpSource);
 	return;
     }
+    if (cistrcmp(name, "ListenQueueSize") == 0) {
+	sscanf(param1, "%d", &ListenQueueSize);
+	return;
+    }
     if (cistrcmp(name, "accept-percent-kludge") == 0) {
 	percent_accept = 1;
 	return;
@@ -96,6 +101,10 @@ char *str;
 	percent_accept = -1;
 	return;
     }
+    if (cistrcmp(name, "allowsourceroute") == 0) {
+      allow_source_route = 1;
+      return;
+    }
     /* Following have two parameters:  DBTYPE and DBPATH */
     if (cistrcmp(name, "policydb") == 0) {
 	policydefine(&policydb, param1, param2);
@@ -103,9 +112,23 @@ char *str;
     }
     if (cistrcmp(name, "tcprcvbuffersize") == 0) {
 	sscanf(param1, "%d", &TcpRcvBufferSize);
+	return;
     }
     if (cistrcmp(name, "tcpxmitbuffersize") == 0) {
 	sscanf(param1, "%d", &TcpXmitBufferSize);
+	return;
+    }
+    if (cistrcmp(name, "debugcmd") == 0) {
+      debugcmdok = 1;
+      return;
+    }
+    if (cistrcmp(name, "expncmd") == 0) {
+      expncmdok = 1;
+      return;
+    }
+    if (cistrcmp(name, "vrfycmd") == 0) {
+      vrfycmdok = 1;
+      return;
     }
 }
 
