@@ -77,7 +77,7 @@ typedef u_char msgdata;
 typedef char msgdata;
 #endif	/* defined(BIND_VER) && (BIND_VER >= 473) */
 
-static int dnsmxlookup __((const char*, int, int));
+static int dnsmxlookup __((const char*, int, int, int));
 
 typedef union {
 	HEADER qb1;
@@ -88,7 +88,7 @@ typedef union {
 extern int debug;
 
 static int
-dnsmxlookup(host, depth, mxmode)
+dnsmxlookup(host, depth, mxmode, qtype)
 	const char *host;
 	int depth;
 	int mxmode;
@@ -111,13 +111,15 @@ dnsmxlookup(host, depth, mxmode)
 	}
 
 	if (debug) {
-	  if (mxmode)
+	  if (qtype == T_TXT)
+	    printf("TXT-lookup for domain: '%s'\n", host);
+	  else if (mxmode)
 	    printf("MX-Verify: Look MX for host '%s'\n", host);
 	  else
 	    printf("DNS-Verify: Look MX, or Addr for host '%s'\n", host);
 	}
 
-	qlen = res_mkquery(QUERY, host, C_IN, T_MX, NULL, 0, NULL,
+	qlen = res_mkquery(QUERY, host, C_IN, qtype, NULL, 0, NULL,
 			   (void*)&qbuf, sizeof qbuf);
 	if (qlen < 0) {
 	  fprintf(stdout, "res_mkquery failed\n");
@@ -198,92 +200,103 @@ dnsmxlookup(host, depth, mxmode)
 			    (void*)realname, sizeof realname);
 	    saw_cname = 1;
 	    continue;
-	  } else if (type != T_MX)  {
+	  } else if (type != qtype)  {
 	    cp += n;
 	    continue;
 	  }
-	  cp += 2; /* MX preference value */
-	  n = dn_expand((msgdata *)&answer, eom, cp, (void*)buf, sizeof buf);
-	  if (n < 0)
-	    break;
-	  cp += n;
 
-	  memset(&req, 0, sizeof(req));
-	  req.ai_socktype = SOCK_STREAM;
-	  req.ai_protocol = IPPROTO_TCP;
-	  req.ai_flags    = AI_CANONNAME;
-	  req.ai_family   = 0; /* Both OK (IPv4/IPv6) */
-	  ai = NULL;
+	  if (type == T_MX) {
+	    cp += 2; /* MX preference value */
+	    n = dn_expand((msgdata *)&answer, eom, cp, (void*)buf, sizeof buf);
+	    if (n < 0)
+	      break;
+	    cp += n;
 
-	  /* This resolves CNAME, it should not happen in case
-	     of MX server, though..    */
+	    memset(&req, 0, sizeof(req));
+	    req.ai_socktype = SOCK_STREAM;
+	    req.ai_protocol = IPPROTO_TCP;
+	    req.ai_flags    = AI_CANONNAME;
+	    req.ai_family   = 0; /* Both OK (IPv4/IPv6) */
+	    ai = NULL;
+
+	    /* This resolves CNAME, it should not happen in case
+	       of MX server, though..    */
 #define GETADDRINFODEBUG 0
 #if !GETADDRINFODEBUG
-	  i = getaddrinfo((const char*)buf, "0", &req, &ai);
+	    i = getaddrinfo((const char*)buf, "0", &req, &ai);
 #else
-	  i = _getaddrinfo_((const char*)buf, "0", &req, &ai, debug ? stdout : NULL);
-if (debug)
-  printf("  getaddrinfo('%s','0') -> r=%d, ai=%p\n",buf,i,ai);
-#endif
-
-	  if (debug)
-	    printf("  getaddrinfo('%s') yields %d\n", buf, i);
-
-	  if (i != 0)
-	    continue;		/* Well well.. spurious! */
-
-	  if (!mxmode) /* Accept if found ANYTHING! */ {
-	    if (debug) printf("  ... accepted!\n");
-	    freeaddrinfo(ai);
-	    return 1;
-	  }
-	  
-	  {
-	    struct addrinfo *ai2 = ai;
-	    int i = 1, rc;
-	    
-	    for ( ; ai2 != NULL; ai2 = ai2->ai_next, ++i ) {
-#if 0
-	      if (debug) {
-		struct sockaddr * sa = ai2->ai_addr;
-		char buf[60];
-		if (sa->sa_family == AF_INET) {
-		  inet_ntop(AF_INET, & ((struct sockaddr_in *)sa)->sin_addr, buf, sizeof(buf));
-		  printf("  matching address IPv4:[%s]\n", buf);
-		}
-#if defined(AF_INET6) && defined(INET6)
-		else if (sa->sa_family == AF_INET6) {
-		  inet_ntop(AF_INET6, ((struct sockaddr_in6 *)sa)->sin6_addr, buf, sizeof(buf));
-		  printf("  matching address IPv6:[%s]\n", buf);
-		}
-#endif
-		else
-		  printf(" matching unknown address family address; AF=%d\n",
-			 sa->sa_family);
-	      }
-#endif
-	      rc = matchmyaddress(ai2->ai_addr);
-	      if (rc == 1) {
-		if (debug)
-		  printf("  ADDRESS MATCH!\n");
-		freeaddrinfo(ai);
-		return 1; /* Found a match! */
-	      } else
-		if (debug)
-		  printf("  matchmyaddress() yields: %d\n", rc);
-	    }
+	    i = _getaddrinfo_((const char*)buf, "0", &req, &ai,
+			      debug ? stdout : NULL);
 	    if (debug)
-	      printf("  No address match among %d address!\n",i);
+	      printf("  getaddrinfo('%s','0') -> r=%d, ai=%p\n",buf,i,ai);
+#endif
+
+	    if (debug)
+	      printf("  getaddrinfo('%s') yields %d\n", buf, i);
+	    
+	    if (i != 0)
+	      continue;		/* Well well.. spurious! */
+
+	    if (!mxmode) /* Accept if found ANYTHING! */ {
+	      if (debug) printf("  ... accepted!\n");
+	      freeaddrinfo(ai);
+	      return 1;
+	    }
+	  
+	    {
+	      struct addrinfo *ai2 = ai;
+	      int i = 1, rc;
+	    
+	      for ( ; ai2 != NULL; ai2 = ai2->ai_next, ++i ) {
+#if 0
+		if (debug) {
+		  struct sockaddr * sa = ai2->ai_addr;
+		  char buf[60];
+		  if (sa->sa_family == AF_INET) {
+		    inet_ntop(AF_INET, & ((struct sockaddr_in *)sa)->sin_addr, buf, sizeof(buf));
+		    printf("  matching address IPv4:[%s]\n", buf);
+		  }
+#if defined(AF_INET6) && defined(INET6)
+		  else if (sa->sa_family == AF_INET6) {
+		    inet_ntop(AF_INET6, ((struct sockaddr_in6 *)sa)->sin6_addr, buf, sizeof(buf));
+		    printf("  matching address IPv6:[%s]\n", buf);
+		  }
+#endif
+		  else
+		    printf(" matching unknown address family address; AF=%d\n",
+			   sa->sa_family);
+		}
+#endif
+		rc = matchmyaddress(ai2->ai_addr);
+		if (rc == 1) {
+		  if (debug)
+		    printf("  ADDRESS MATCH!\n");
+		  freeaddrinfo(ai);
+		  return 1; /* Found a match! */
+		} else
+		  if (debug)
+		    printf("  matchmyaddress() yields: %d\n", rc);
+	      }
+	      if (debug)
+		printf("  No address match among %d address!\n",i);
+	    }
+	    freeaddrinfo(ai);
+#if 0
+	    return 2; /* We have SOME date, but no match on ourselves! */
+#endif
+	  } /* ===== END OF MX DATA PROCESING ========= */
+
+	  if (type == T_TXT) {
 	  }
-	  freeaddrinfo(ai);
+
+	} /* ===== END OF DNS ANSWER PROCESSING ======= */
 
-	}
 	/* Didn't find any, but saw CNAME ? Recurse with the real name */
 	if (saw_cname)
-	  return dnsmxlookup(realname, depth+1, mxmode);
+	  return dnsmxlookup((void *)realname, depth+1, mxmode, qtype);
 
 perhaps_address_record:
-	if (!mxmode) {
+	if (qtype == T_MX && !mxmode) {
 	  /* No MX, perhaps A ? */
 	  memset(&req, 0, sizeof(req));
 	  req.ai_socktype = SOCK_STREAM;
@@ -320,7 +333,7 @@ perhaps_address_record:
 }
 
 
-/* For SOFT errors, return -100, for hard errors, -1.
+/* For SOFT errors, return -102, for hard errors, -2.
    For 'we are MX', return 0.
    For (retmode == '+'), and without MX, return 1.
  */
@@ -338,18 +351,22 @@ int mx_client_verify(retmode, domain, alen)
 	strncpy(hbuf, domain, alen);
 	hbuf[alen] = 0; /* Chop off the trailers from the name */
 
-	rc = dnsmxlookup(hbuf, 1, 1);
+	rc = dnsmxlookup(hbuf, 1, 1, T_MX);
 
 	if (rc == 1) return 0; /* Found! */
 
 	if (retmode == '+') {
 	  if (rc == -EX_NOHOST ||
 	      rc == -EX_UNAVAILABLE)
-	    return -1; /* Definitely hard errors */
-	  return -100; /* Soft error */
+	    return -2; /* Definitely hard errors */
+	  if (rc == 2)
+	    return -103;
+	  return -102; /* Soft error */
 	}
 
-	return -1;     /* Reject */
+	if (rc == 2)
+	  return -3;
+	return -2;     /* Reject */
 }
 
 int sender_dns_verify(retmode, domain, alen)
@@ -365,18 +382,22 @@ int sender_dns_verify(retmode, domain, alen)
 	strncpy(hbuf, domain, alen);
 	hbuf[alen] = 0; /* Chop off the trailers from the name */
 
-	rc = dnsmxlookup(hbuf, 1, 0);
+	rc = dnsmxlookup(hbuf, 1, 0, T_MX);
 
 	if (rc == 1) return 0; /* Found! */
 
 	if (retmode == '+') {
 	  if (rc == -EX_NOHOST ||
 	      rc == -EX_UNAVAILABLE)
-	    return -1; /* Definitely hard errors */
-	  return -100; /* Soft error */
+	    return -2; /* Definitely hard errors */
+	  if (rc == 2)
+	    return -103;
+	  return -102; /* Soft error */
 	}
 
-	return -1;     /* Reject */
+	if (rc == 2)
+	  return -3;
+	return -2;     /* Reject */
 }
 
 int client_dns_verify(retmode, domain, alen)
@@ -385,3 +406,40 @@ int client_dns_verify(retmode, domain, alen)
 {
 	return sender_dns_verify(retmode, domain, alen);
 }
+
+int rbl_dns_test(ipv4addr, msgp)
+     u_char *ipv4addr;
+     char **msgp;
+{
+	char hbuf[2000], *s;
+	int rc;
+
+	sprintf (hbuf, "%d.%d.%d.%d.rbl.maps.vix.com.",
+		 ipv4addr[3], ipv4addr[2], ipv4addr[1], ipv4addr[0]);
+
+
+	if (debug)
+	  printf("looking up DNS object: %s\n", hbuf);
+
+	/* XX: Should verify that the named object has A record: 127.0.0.2 */
+
+	if (gethostbyname(hbuf) != NULL) {
+
+	/* XX: Could - perhaps - use:  dnsmxlookup(hbuf, 0, 0, T_TXT) ?
+	   QQ: How to return the TXT entry ? */
+
+	  sprintf (hbuf, "Blackholed - see <URL:http://maps.vix.com/cgi-bin/lookup?%d.%d.%d.%d>",
+		   ipv4addr[3], ipv4addr[2], ipv4addr[1], ipv4addr[0]);
+
+	  if (*msgp != NULL)
+	    free(*msgp);
+
+	  *msgp = strdup(hbuf);
+
+	  if (debug)
+	    printf("looking up DNS object: %s\n", hbuf);
+
+	  return -1;
+	}
+	return 0;
+}
