@@ -1,13 +1,12 @@
 /*
  *  relaytest.c -- module for ZMailer's smtpserver
- *  By Matti Aarnio <mea@nic.funet.fi>
+ *  By Matti Aarnio <mea@nic.funet.fi> 1997
  *
  */
 
 /*
  * TODO:
  *  - Attribute 'request' initializations for resolving
- *  - 'freeze' attribute processing
  *  - Addresses in form  <@foo:uu@dd>, <host!user>, <host!user@domain>
  */
 
@@ -55,7 +54,6 @@
 
 #define _POLICYTEST_INTERNAL_
 #include "policytest.h"
-#include "policy.h"
 
 extern int debug;
 
@@ -68,6 +66,77 @@ static int  checkaddr  __((struct policytest *, struct policystate *, const char
 extern const struct in6_addr zv4mapprefix;
 #endif
 
+/* KK() and KA() macroes are at "policy.h" */
+
+static char *showkey __((const char *key));
+static char *showkey(key)
+const char *key;
+{
+    static char buf[256];
+
+    if (key[1] != P_K_IPv4 && key[1] != P_K_IPv6) {
+	if (strlen(key+2) > (sizeof(buf) - 200))
+	    sprintf(buf,"%d/%s/'%s'", key[0], KK(key[1]), "<too long name>");
+	else
+	    sprintf(buf,"%d/%s/'%s'", key[0], KK(key[1]), key+2);
+    } else
+      if (key[1] == P_K_IPv4)
+	sprintf(buf,"%d/%s/%u.%u.%u.%u",
+		key[0], KK(key[1]),
+		key[2] & 0xff, key[3] & 0xff, key[4] & 0xff, key[5] & 0xff);
+      else
+	sprintf(buf,"%d/%s/%02x%02x:%02x%02x:...",
+		key[0], KK(key[1]),
+		key[2] & 0xff, key[3] & 0xff, key[4] & 0xff, key[5] & 0xff);
+    return buf;
+}
+
+static char *showattr __((const char *key));
+static char *showattr(key)
+const char *key;
+{
+    static char buf[256];
+    sprintf(buf,"%d/%s/'%s'", key[0], KA(key[1]), key+2);
+    return buf;
+}
+
+static char *showresults __((const char values[]));
+static char *showresults(values)
+const char *values;
+{
+    static char buf[256];
+    int i;
+
+    buf[0] = '\0';
+    for (i = P_A_FirstAttr; i <= P_A_LastAttr; ++i) {
+	sprintf(buf+strlen(buf),"%s ",KA(i));
+	sprintf(buf+strlen(buf),"%c ",values[i] ? values[i] : '.');
+    }
+    return buf;
+}
+
+static void printstate __((const struct policystate *state));
+static void printstate (state)
+const struct policystate *state;
+{
+	int i;
+
+	printf("always_reject=%d\n",state->always_reject);
+	printf("always_freeze=%d\n",state->always_freeze);
+	printf("always_accept=%d\n",state->always_accept);
+	printf("sender_reject=%d\n",state->sender_reject);
+	printf("sender_freeze=%d\n",state->sender_freeze);
+	printf("sender_norelay=%d\n",state->sender_norelay);
+	printf("relaycustnet=%d\n", state->relaycustnet);
+	printf("rcpt_nocheck=%d\n", state->rcpt_nocheck);
+
+	for ( i = P_A_FirstAttr; i <= P_A_LastAttr ; ++i) {
+		printf("%s: %s, value=%c\n",
+		       KA(i),(state->request&(1<<i))?"requested":"not req",
+		       state->values[i]?state->values[i]:'.');
+	}
+}
+
 static void mask_ip_bits __((unsigned char *, int, int));
 static void mask_ip_bits(ipnum, width, maxwidth)
 unsigned char *ipnum;
@@ -100,80 +169,11 @@ struct policytest **relp;
 const char *dbtype, *dbpath;
 {
     struct policytest *rel = (void *) emalloc(sizeof(*rel));
-    int openok;
-    char *dbname;
-
     *relp = rel;
     memset(rel, 0, sizeof(*rel));
     rel->dbtype = strdup(dbtype);
     rel->dbpath = strdup(dbpath);
     rel->dbt = _dbt_none;
-#ifdef HAVE_NDBM_H
-    if (cistrcmp(dbtype, "ndbm") == 0)
-	rel->dbt = _dbt_ndbm;
-#endif
-#ifdef HAVE_GDBM_H
-    if (cistrcmp(dbtype, "gdbm") == 0)
-	rel->dbt = _dbt_ndbm;
-#endif
-#ifdef HAVE_DB_H
-    if (cistrcmp(dbtype, "btree") == 0)
-	rel->dbt = _dbt_btree;
-#endif
-    if (rel->dbt == _dbt_none) {
-	/* XX: ERROR! Unknown/unsupported dbtype! */
-	return;
-    }
-    openok = 0;
-#ifdef HAVE_ALLOCA
-    dbname = alloca(strlen(dbpath) + 8);
-#else
-    dbname = emalloc(strlen(dbpath) + 8);
-#endif
-    switch (rel->dbt) {
-#ifdef HAVE_NDBM_H
-    case _dbt_ndbm:
-	/*
-	   rel->ndbm = dbm_open((char*)dbpath, O_RDWR|O_CREAT|O_TRUNC, 0644);
-	 */
-	rel->ndbm = dbm_open((char *) dbpath, O_RDWR, 0644);
-	openok = (rel->ndbm != NULL);
-	break;
-#endif
-#ifdef HAVE_GDBM_H
-    case _dbt_gdbm:
-	/* Append '.gdbm' to the name */
-	sprintf(dbname, "%s.gdbm", dbpath);
-	rel->gdbm = gdbm_open(dbname, 0, GDBM_READER, 0644, NULL);
-	openok = (rel->gdbm != NULL);
-	break;
-#endif
-#ifdef HAVE_DB_H
-    case _dbt_btree:
-	/* Append '.db' to the name */
-	sprintf(dbname, "%s.db", dbpath);
-	rel->btree = dbopen(dbname, O_RDONLY, 0644, DB_BTREE, NULL);
-	openok = (rel->btree != NULL);
-	break;
-#endif
-    default:
-	break;
-    }
-#ifndef HAVE_ALLOCA
-    free(dbname);
-#endif
-    if (!openok) {
-	/* ERROR!  Could not open the database! */
-      if (debug)
-	printf("ERROR!  Could not open the database!\n");
-      return;
-    }
-#ifdef HAVE_NDBM_H
-#endif
-#ifdef HAVE_GDBM_H
-#endif
-#ifdef HAVE_DB_H
-#endif
 }
 
 /* Do the actual query - return pointer to the result record */
@@ -250,6 +250,23 @@ int *rlenp;			/* result length ptr ! */
 	*rlenp = Bresult.size;
 	return buffer;
 
+	break;
+
+    case _dbt_bhash:
+
+	Bkey.data = (void *) qptr;
+	Bkey.size = qlen;
+
+	rc = (rel->bhash->get) (rel->bhash, &Bkey, &Bresult, 0);
+	if (rc != 0)
+	    return NULL;
+
+	buffer = (char *) emalloc(Bresult.size);
+	memcpy(buffer, Bresult.data, Bresult.size);
+
+	*rlenp = Bresult.size;
+	return buffer;
+
 	break;
 #endif
     default:
@@ -282,7 +299,7 @@ struct policystate *state;
 const char *key;
 int init;
 {
-  char *str, *str_base, pbuf[256];
+    char *str, *str_base, pbuf[256];
     int rlen, result, interest, len;
 
     if (init) {
@@ -292,6 +309,9 @@ int init;
     }
     --recursions;
 
+    if (debug)
+       printf("Key: %s\n", showkey(key));
+/*
     if (key[1] != P_K_IPv4 && key[1] != P_K_IPv6) {
 	if (debug)
 	  printf("Key: %d/%d/%s\n", key[0],key[1],key+2);
@@ -299,6 +319,7 @@ int init;
       if (debug)
 	printf("Key: %u.%u.%u.%u\n", key[2] & 0xff, key[3] & 0xff, 
 	       key[4] & 0xff, key[5] & 0xff);
+*/
 
     str_base = str = (char *) dbquery(rel, &key[0], key[0], &rlen);
 
@@ -308,7 +329,7 @@ int init;
 
     if (str == NULL) {
       if (debug)
-	printf("Query failed: key='%d/%d/%s'\n", key[0], key[1], key+2);
+	printf("  query failed\n");
       return -1;
     }
     /* Scan trough attribute list. Call resolveattributes recursively
@@ -332,13 +353,22 @@ int init;
 	      result = resolveattributes(rel, recursions, state, pbuf, 0);
 	    }
 	    rlen -= str[0];
-	    str += str[0];
+	    str  += str[0];
 	    continue;
 	}
 
 	/* Attribute */
 	if (debug)
-	  printf("Attribute: %d/%d/%s\n", (int) str[0] & 0xFF, (int) str[1] & 0xff, str + 2);
+	  printf("  Attribute: %s\n", showattr(str));
+
+	if (str[1] == P_A_MESSAGE) {
+	  if (state->msgstr != NULL)
+	    free(state->msgstr);
+	  state->msgstr = strdup(str+2);
+	  rlen -= str[0];
+	  str  += str[0];
+	  continue;
+	}
 
 	interest = 1 << str[1];	/* Convert attrib. num. value into flag bit */
 	if ((interest & state->request) == 0) {
@@ -346,7 +376,7 @@ int init;
 	    rlen -= str[0];
 	    str  += str[0];
 	    if (debug)
-	      printf("Not interested. Skipping over.\n");
+	      printf("    not interested, skipped...\n");
 	    continue;
 	} else {
 	    /* Mask it off. */
@@ -356,6 +386,8 @@ int init;
 	if (str[2] != '+' && str[2] != '-') {
 	  if (debug)
 	    printf("Unknown flag: %s\n", &str[2]);
+	  rlen -= str[0];
+	  str  += str[0];
 	  continue;
 	}
 	/* Store valid attribute.
@@ -364,13 +396,15 @@ int init;
 	if (P_A_FirstAttr <= str[1] && str[1] <= P_A_LastAttr) {
 	    if (state->values[str[1] & 0xFF] == 0)
 		state->values[str[1] & 0xFF] = str[2];
+	  if (debug)
+	    printf("    accepted!\n");
 	} else {
 	  if (debug)
-	    printf("Unknown attribute, number: %d\n", str[1] & 0xFF);
+	    printf("  Unknown attribute, number: %d\n", str[1] & 0xFF);
 	}
 
 	rlen -= str[0];
-	str += str[0];
+	str  += str[0];
 
 	/* If all requests are done, exit. */
 	if (!state->request) {
@@ -406,10 +440,13 @@ const char *pbuf;
 	result = resolveattributes(rel, maxrecursions, state, pbuf, 1);
 	if (debug) {
 	  int i;
+	  printf("Results: %s\n", showresults(state->values));
+/*
 	  printf("Results: ");
 	  for (i = 0; i < sizeof(state->values); i++)
 	    printf(":%c", state->values[i] ? state->values[i] : '0');
 	  printf("\n");
+*/
 	}
 	return (result);
     }
@@ -419,10 +456,13 @@ const char *pbuf;
 	result = resolveattributes(rel, maxrecursions, state, pbuf, 1);
 	if (debug) {
 	  int i;
+	  printf("Results: %s\n", showresults(state->values));
+/*
 	  printf("Results: ");
 	  for (i = 0; i < sizeof(state->values); i++)
 	    printf(":%c", state->values[i] ? state->values[i] : '0');
 	  printf("\n");
+*/
 	}
 	return (result);
     } else if (pbuf[1] == P_K_IPv4)
@@ -431,10 +471,11 @@ const char *pbuf;
 	countmax = 128;
 
     result = 1;
-    count = 1;
+    count = 0;
 
     while (result != 0 && count <= countmax) {
 
+	count++;
 	/* Search database */
 	result = resolveattributes(rel, maxrecursions, state, pbuf, 1);
 	if (result == 0)	/* Found. */
@@ -448,7 +489,6 @@ const char *pbuf;
 	else			/* AF_INET6 */
 	  ((char*)pbuf)[18] = 128 - count;
 #endif
-	count++;
     }
 
     
@@ -471,86 +511,115 @@ const char *pbuf;
       return -1;
     } else
       if (debug) {
-	int i;
-	for (i = 0; i < 32; i++)
-	  printf(":%c", state->values[i] ? state->values[i] : '0');
-	printf(":  Count: %d\n", count - 1);
+	printf("  %s\n", showresults(state->values));
       }
     return 0;
 }
 
 
-void policyinit(rel, state)
-struct policytest *rel;
+int policyinit(relp, state)
+struct policytest **relp;
 struct policystate *state;
 {
-  memset(state, 0, sizeof(*state));
+    int openok;
+    char *dbname;
+    struct policytest *rel = *relp;
+
+    if (rel == NULL)
+      return 0;  /* Not defined! */
+
+#ifdef HAVE_NDBM_H
+    if (cistrcmp(rel->dbtype, "ndbm") == 0)
+	rel->dbt = _dbt_ndbm;
+#endif
+#ifdef HAVE_GDBM_H
+    if (cistrcmp(rel->dbtype, "gdbm") == 0)
+	rel->dbt = _dbt_gdbm;
+#endif
+#ifdef HAVE_DB_H
+    if (cistrcmp(rel->dbtype, "btree") == 0)
+	rel->dbt = _dbt_btree;
+    if (cistrcmp(rel->dbtype, "bhash") == 0)
+	rel->dbt = _dbt_bhash;
+#endif
+    if (rel->dbt == _dbt_none) {
+	/* XX: ERROR! Unknown/unsupported dbtype! */
+      *relp = NULL;
+	return 1;
+    }
+    openok = 0;
+#ifdef HAVE_ALLOCA
+    dbname = alloca(strlen(rel->dbpath) + 8);
+#else
+    dbname = emalloc(strlen(rel->dbpath) + 8);
+#endif
+    switch (rel->dbt) {
+#ifdef HAVE_NDBM_H
+    case _dbt_ndbm:
+	/*
+	   rel->ndbm = dbm_open((char*)rel->dbpath, O_RDWR|O_CREAT|O_TRUNC, 0644);
+	 */
+	rel->ndbm = dbm_open((char *) rel->dbpath, O_RDONLY, 0644);
+	openok = (rel->ndbm != NULL);
+	break;
+#endif
+#ifdef HAVE_GDBM_H
+    case _dbt_gdbm:
+	/* Append '.gdbm' to the name */
+	sprintf(dbname, "%s.gdbm", rel->dbpath);
+	rel->gdbm = gdbm_open(dbname, 0, GDBM_READER, 0644, NULL);
+	openok = (rel->gdbm != NULL);
+	break;
+#endif
+#ifdef HAVE_DB_H
+    case _dbt_btree:
+	/* Append '.db' to the name */
+	sprintf(dbname, "%s.db", rel->dbpath);
+	rel->btree = dbopen(dbname, O_RDONLY, 0644, DB_BTREE, NULL);
+	openok = (rel->btree != NULL);
+	break;
+
+    case _dbt_bhash:
+	rel->bhash = dbopen(rel->dbpath, O_RDONLY, 0644, DB_HASH, NULL);
+	openok = (rel->bhash != NULL);
+	break;
+#endif
+    default:
+	break;
+    }
+#ifndef HAVE_ALLOCA
+    free(dbname);
+#endif
+    if (!openok) {
+	/* ERROR!  Could not open the database! */
+      if (debug)
+	printf("ERROR!  Could not open the database!\n");
+      *relp = NULL;
+      return 2;
+    }
+
+    memset(state, 0, sizeof(*state));
+    return 0;
 }
 
 
-
-int policytestaddr(rel, state, what, raddr)
+static int _addrtest_(rel, state, pbuf)
 struct policytest *rel;
 struct policystate *state;
-PolicyTest what;
-struct sockaddr *raddr;
+const char *pbuf;
 {
-    char pbuf[64]; /* Not THAT much space needed.. */
-
-    struct sockaddr_in *si4;
-#if defined(AF_INET6) && defined(INET6)
-    struct sockaddr_in6 *si6;
-#endif
-
-
-    if (what != POLICY_SOURCEADDR)
-	abort();		/* Urgle..! Code mismatch! */
-
-    if (rel == NULL)
-      return 0;
-
-    /* Find address match -- IPv4 mapped into IPv6 space too! */
-
-    if (raddr->sa_family == 0)
-      return 0; /* Interactive testing... */
-
-    if (raddr->sa_family == AF_INET) {
-	si4 = (struct sockaddr_in *) raddr;
-	pbuf[0] = 7;
-	pbuf[1] = P_K_IPv4;
-	memcpy(&pbuf[2], (char *) &si4->sin_addr.s_addr, 4);
-	pbuf[6] = 32;		/* 32 bits */
-    } else
-#if defined(AF_INET6) && defined(INET6)
-    if (raddr->sa_family == AF_INET6) {
-	si6 = (struct sockaddr_in6 *) raddr;
-	if (memcmp((void *)&si6->sin6_addr, &zv4mapprefix, 12) == 0) {
-	  /* This is IPv4 address mapped into IPv6 */
-	  pbuf[0] = 7;
-	  pbuf[1] = P_K_IPv4;
-	  memcpy(pbuf+2, ((char *) &si6->sin6_addr) + 12, 4);
-	  pbuf[6] = 32;			/*  32 bits */
-	} else {
-	  pbuf[0] = 19;
-	  pbuf[1] = P_K_IPv6;
-	  memcpy(pbuf+2, ((char *) &si6->sin6_addr), 16);
-	  pbuf[18] = 128;		/* 128 bits */
-	}
-    } else
-#endif
-    {
-      printf("Unknown address format; sa_family = %d\n", raddr->sa_family);
-      return -2;
-    }
+    u_char ipv4addr[4];
 
+    if (pbuf[1] == P_K_IPv4)
+      memcpy(ipv4addr, pbuf+2, 4);
 
     /* state->request initialization !! */
 
     state->request = ( 1 << P_A_REJECTNET    |
 		       1 << P_A_FREEZENET    |
 		       1 << P_A_RELAYCUSTNET |
-		       1 << P_A_REJECTSOURCE |
-		       1 << P_A_FREEZESOURCE   );
+		       1 << P_A_TestDnsRBL   );
+
 
     if (checkaddr(rel, state, pbuf) != 0)
       return 0; /* Nothing found */
@@ -576,40 +645,107 @@ struct sockaddr *raddr;
     [state->sender_reject = 1; return -1;]
  */
 #endif
+
+    if (state->message != NULL)
+      free(state->message);
+    state->message = state->msgstr;
+    state->msgstr = NULL;
+
     if (state->values[P_A_REJECTNET] == '+') {
       if (debug)
 	printf("policytestaddr: 'rejectnet +' found\n");
-	state->always_reject = 1;
-	return -1;
+      if (state->message == NULL)
+	state->message = strdup("Your network address is blackholed in our static tables");
+      state->always_reject = 1;
+      return -1;
     }
     if (state->values[P_A_FREEZENET] == '+') {
       if (debug)
 	printf("policytestaddr: 'freezenet +' found\n");
-	state->always_freeze = 1;
-	return  1;
+      if (state->message == NULL)
+	state->message = strdup("Your network address is blackholed in our static tables");
+      state->always_freeze = 1;
+      return  1;
     }
     if (state->values[P_A_RELAYCUSTNET] == '+') {
       if (debug)
 	printf("policytestaddr: 'relaycustnet +' found\n");
-	state->always_accept = 1;
-	return  0;
+      state->always_accept = 1;
+      return  0;
     }
-    if (state->values[P_A_REJECTSOURCE] == '+') {
+    if (state->values[P_A_TestDnsRBL] == '+' &&
+	pbuf[1] == P_K_IPv4) {
+      int rc;
       if (debug)
-	printf("policytestaddr: 'rejectsource +' found\n");
-	state->sender_reject = 1;
-	return -1;
-    }
-    if (state->values[P_A_FREEZESOURCE] == '+') {
+	printf("policytestaddr: 'test-dns-rbl +' (IPv4) found;");
+      rc = rbl_dns_test(ipv4addr, &state->message);
       if (debug)
-	printf("policytestaddr: 'freezesource +' found\n");
-	state->sender_freeze = 1;
-	return -1;
+	printf(" rc=%d\n", rc);
+      return rc;
     }
 
     return 0;
 }
 
+int policytestaddr(rel, state, what, raddr)
+struct policytest *rel;
+struct policystate *state;
+PolicyTest what;
+Usockaddr *raddr;
+{
+    char pbuf[64]; /* Not THAT much space needed.. */
+
+    struct sockaddr_in *si4;
+#if defined(AF_INET6) && defined(INET6)
+    struct sockaddr_in6 *si6;
+#endif
+
+
+    if (what != POLICY_SOURCEADDR)
+	abort();		/* Urgle..! Code mismatch! */
+
+    if (rel == NULL)
+      return 0;
+
+    /* Find address match -- IPv4 mapped into IPv6 space too! */
+
+    state->message = NULL; /* This is early initial clearing */
+
+    if (raddr->v4.sin_family == 0)
+      return 0; /* Interactive testing... */
+
+    if (raddr->v4.sin_family == AF_INET) {
+	si4 = & (raddr->v4);
+	pbuf[0] = 7;
+	pbuf[1] = P_K_IPv4;
+	memcpy(&pbuf[2], (char *) &si4->sin_addr.s_addr, 4);
+	pbuf[6] = 32;		/* 32 bits */
+    } else
+#if defined(AF_INET6) && defined(INET6)
+    if (raddr->v6.sin6_family == AF_INET6) {
+	si6 = & (raddr->v6);
+	if (memcmp((void *)&si6->sin6_addr, &zv4mapprefix, 12) == 0) {
+	  /* This is IPv4 address mapped into IPv6 */
+	  pbuf[0] = 7;
+	  pbuf[1] = P_K_IPv4;
+	  memcpy(pbuf+2, ((char *) &si6->sin6_addr) + 12, 4);
+	  pbuf[6] = 32;			/*  32 bits */
+	} else {
+	  pbuf[0] = 19;
+	  pbuf[1] = P_K_IPv6;
+	  memcpy(pbuf+2, ((char *) &si6->sin6_addr), 16);
+	  pbuf[18] = 128;		/* 128 bits */
+	}
+    } else
+#endif
+    {
+      printf("Unknown address format; sa_family = %d\n", raddr->v4.sin_family);
+      return -2;
+    }
+
+    return _addrtest_(rel,state,pbuf);
+}
+
 
 static void check_domain(rel, state, input, inlen)
 struct policytest *rel;
@@ -636,8 +772,8 @@ int inlen;
 #endif
 
     /* Convert to lower case. */
-    if (addr_len > 254)
-	addr_len = 254;
+    if (addr_len > sizeof(pbuf)-3)
+	addr_len = sizeof(pbuf)-3;
     strncpy(pbuf+2, ptr, addr_len);
     pbuf[2+addr_len] = 0;
     strlower(pbuf+2);
@@ -653,7 +789,7 @@ int inlen;
 
     while (result != 0) {
 	if (debug)
-	  printf("DEBUG: %d/%d/%s\n", pbuf[0], pbuf[1], pbuf+2);
+	  printf("DEBUG: %s\n", showkey(pbuf));
 	result = checkaddr(rel, state, pbuf);
 
 	if (result == 0) /* Found! */
@@ -690,6 +826,26 @@ int inlen;
     }
 }
 
+static const char * find_at __((const char *, int));
+static const char *
+find_at(input, inlen)
+const char *input;
+int inlen;
+{
+  int quote = 0;
+  /* Find first unquoted '@' character, and return a pointer to it */
+  for (; inlen > 0; --inlen,++input) {
+    if (*input == '"')
+      quote = !quote;
+    if (*input == '\\') {
+      --inlen; ++input;
+      continue;
+    }
+    if (*input == '@' && !quote)
+      return input;
+  }
+  return NULL;
+}
 
 /* Return 0, when found something */
 static int check_user(rel, state, input, inlen)
@@ -698,7 +854,8 @@ struct policystate *state;
 const char *input;
 int inlen;
 {
-    char pbuf[512], *at;
+    char pbuf[512];
+    const char *at;
     int i, result;
 
     if (inlen > (sizeof(pbuf) - 3))
@@ -710,7 +867,7 @@ int inlen;
     pbuf[2+inlen] = 0;
     strlower(pbuf + 2);
 
-    at = memchr(pbuf + 2, '@', inlen);
+    at = find_at(pbuf + 2, inlen);
     if (!at) return 0;
 
     pbuf[inlen + 2] = '\0';
@@ -732,14 +889,15 @@ int inlen;
 }
 
 
-/*
-   static int pt_heloname __((struct policytest *, struct policystate *, const char *, const int));
+static int pt_heloname __((struct policytest *, struct policystate *, const char *, const int));
+
+static int pt_mailfrom __((struct policytest *, struct policystate *, const char *, const int));
 
-   static int pt_mailfrom __((struct policytest *, struct policystate *, const char *, const int));
- */
 static int pt_rcptto __((struct policytest *, struct policystate *, const char *, const int));
 
-int pt_heloname(rel, state, str, len)
+static int pt_rcptpostmaster __((struct policytest *, struct policystate *, const char *, const int));
+
+static int pt_heloname(rel, state, str, len)
 struct policytest *rel;
 struct policystate *state;
 const char *str;
@@ -753,7 +911,7 @@ const int len;
 	return 0;
 
     /* state->request initialization !! */
-    state->request = ( 1 << P_A_REJECTNET |
+    state->request = ( 1 << P_A_REJECTNET    |
 		       1 << P_A_FREEZENET  );
 
     check_domain(rel, state, str, len);
@@ -774,16 +932,53 @@ const int len;
     return 0;
 }
 
-int pt_mailfrom(rel, state, str, len)
+static int pt_sourcedomain(rel, state, str, len)
 struct policytest *rel;
 struct policystate *state;
 const char *str;
 const int len;
 {
-	char *at;
+    if (state->always_reject)
+	return -1;
+    if (state->always_freeze)
+	return 1;
+    if (state->always_accept)
+	return 0;
+
+    /* state->request initialization !! */
+    state->request = ( 1 << P_A_REJECTNET    |
+		       1 << P_A_FREEZENET  );
+
+    check_domain(rel, state, str, len);
+
+/*
+   # if (name of SMTP client has 'rejectnet +' attribute) then
+   #    any further conversation refused
+   #      [state->always_reject = 1; return -1;]
+ */
+    if (state->values[P_A_REJECTNET] == '+') {
+	state->always_reject = 1;
+	return -1;
+    }
+    if (state->values[P_A_FREEZENET] == '+') {
+	state->always_freeze = 1;
+	return  1;
+    }
+    return 0;
+}
+
+static int pt_mailfrom(rel, state, str, len)
+struct policytest *rel;
+struct policystate *state;
+const char *str;
+const int len;
+{
+    const char *at;
+
     state->rcpt_nocheck  = 0;
     state->sender_reject = 0;
     state->sender_freeze = 0;
+    state->sender_norelay = 0;
 
     if (state->always_reject)
 	return -1;
@@ -800,6 +995,8 @@ const int len;
 		       1 << P_A_FREEZESOURCE   );
 
     /* XX: How about  <@foo:user@domain> ??? */
+    /* XX: With IGNORING RFC-821-source-route "@foo:" we
+           don't have problems here */
 
     /* Check source user */
     if (check_user(rel, state, str, len) == 0) {
@@ -819,10 +1016,11 @@ const int len;
 
     state->request = ( 1 << P_A_REJECTSOURCE  |
 		       1 << P_A_FREEZESOURCE  |
-		       1 << P_A_RELAYCUSTNET  |
-		       1 << P_A_RELAYCUSTOMER   );
+		       1 << P_A_RELAYCUSTOMER |
+		       1 << P_A_SENDERNoRelay |
+		       1 << P_A_SENDERokWithDNS );
 
-    at = memchr(str, '@', len);
+    at = find_at(str, len);
     if (at != NULL)
       check_domain(rel, state, at+1, len - (1 + at - str));
     else {
@@ -830,26 +1028,6 @@ const int len;
       return -1;
     }
 
-/*
-   # else if (sender's domain has 'rejectsource +' attribute) then
-   #    sender rejected, any further conversation refused
-   #        [state->sender_reject = 1; return -1;]
-   # else if (sender's domain has 'freezesource +' attribute) then
-   #    sender marks freeze, and we present happy accepting face..
-   #        [state->sender_freeze = 1; return -1;]
-   # else if (IP address of SMTP client has 'relaycustnet +' attribute) then
-   #           [state->relaycustnet]
-   #    sender accepted, recipients not checked
-   #        [state->rcpt_nocheck = 1; return 0;]
-   # else if (HELO-name of SMTP client has 'relaycustnet +' attribute) then
-   #    sender accepted, recipients not checked
-   #        [state->rcpt_nocheck = 1; return 0;]
-   --- danger! danger!
-   # else if (sender's domain has 'relaycustomer +' attribute) then
-   #    sender accepted, recipients not checked
-   #        [state->rcpt_nocheck = 1; return 0;]
- */
-
     if (state->values[P_A_REJECTSOURCE] == '+') {
 	if (debug)
 	  printf("mailfrom: 'rejectsource +'\n");
@@ -862,14 +1040,6 @@ const int len;
 	state->sender_freeze = 1;
 	return -1;
     }
-    /* IP address of SMTP client */
-    if (state->values[P_A_RELAYCUSTNET] == '+') {
-	if (debug)
-	  printf("mailfrom: 'relaycustnet +'\n");
-	state->relaycustnet = 1;
-	state->rcpt_nocheck = 1;
-	return  0;
-    }
     if (state->values[P_A_RELAYCUSTOMER] == '+') {
 	if (debug)
 	  printf("mailfrom: 'relaycustomer +'\n");
@@ -884,6 +1054,18 @@ const int len;
 	return -1;
     }
 #endif
+    if (state->values[P_A_SENDERokWithDNS] != 0) {
+      int rc = sender_dns_verify(state->values[P_A_SENDERokWithDNS],
+				 at+1, len - (1 + at - str));
+      if (debug)
+	printf("... returns: %d\n", rc);
+      return rc;
+    }
+    if (state->values[P_A_SENDERNoRelay] == '+') {
+      if (debug)
+	printf("mailfrom: 'sendernorelay +'\n");
+      state->sender_norelay = 1;
+    }
     return 0;
 }
 
@@ -893,31 +1075,47 @@ struct policystate *state;
 const char *str;
 const int len;
 {
-	char *at;
+    const char *at;
 
-    if (state->always_reject)
-	return -1;
-    if (state->sender_reject)
-	return -2;
-    if (state->always_freeze)
-	return 1;
-    if (state->sender_freeze)
-	return 1;
-    if (state->always_accept)
-	return 0;
+    if (state->always_reject) return -1;
+    if (state->sender_reject) return -2;
+    if (state->always_freeze) return  1;
+    if (state->sender_freeze) return  1;
+    if (state->always_accept) return  0;
 
-    if (state->rcpt_nocheck)
-	return 0;
     /* rcptfreeze even for 'rcpt-nocheck' ? */
 
     /* state->request initialization !! */
     state->request = ( 1 << P_A_RELAYTARGET     |
 		       1 << P_A_ACCEPTbutFREEZE );
 
-    at = memchr(str, '@', len);
+    /* Test first the full address */
+    if (check_user(rel, state, str, len) == 0) {
+      if (state->values[P_A_RELAYTARGET] == '+') {
+	return  0;
+      }
+      if (state->values[P_A_RELAYTARGET] == '-') {
+	return -1;
+      }
+      if (state->values[P_A_ACCEPTbutFREEZE] == '+') {
+	state->sender_freeze = 1;
+	return  1;
+      }
+    }
+
+    /* state->request initialization !! */
+    state->request = ( 1 << P_A_RELAYTARGET     |
+		       1 << P_A_ACCEPTbutFREEZE |
+		       1 << P_A_ACCEPTifMX      |
+		       1 << P_A_ACCEPTifDNS     );
+
+    at = find_at(str, len);
     if (at != NULL)
       check_domain(rel, state, at+1, len - (1 + at - str));
     else {
+      if (state->rcpt_nocheck)
+	return 0;
+
       /* Doh ??  Not  <user@domain> ??? */
       return -1;
     }
@@ -944,10 +1142,53 @@ const int len;
 	state->sender_freeze = 1;
 	return  1;
     }
-    /*      return 0; */
+
+    if (state->rcpt_nocheck) {
+      if (debug)
+	printf("... rcpt_nocheck is on!\n");
+	return 0;
+    }
+
+    if (state->values[P_A_ACCEPTifMX] != 0 || state->sender_norelay != 0) {
+      int rc = mx_client_verify(state->values[P_A_ACCEPTifMX],
+				at+1, len - (1 + at - str)); 
+      /* XX: state->message setup! */
+      if (debug)
+	printf("...(mx_client_verify('%.*s')) returns: %d\n",
+	       (len - (1 + at - str)), at+1, rc);
+      return rc;
+    }
+    if (state->values[P_A_ACCEPTifDNS] != 0) {
+      int rc = client_dns_verify(state->values[P_A_ACCEPTifDNS],
+				 at+1, len - (1 + at - str));
+      /* XX: state->message setup! */
+      if (debug)
+	printf("... returns: %d\n", rc);
+      return rc;
+    }
     return 0;
 }
 
+static int pt_rcptpostmaster(rel, state, str, len)
+struct policytest *rel;
+struct policystate *state;
+const char *str;
+const int len;
+{
+	char *at;
+
+    /* state->request initialization !! */
+    state->request = ( 1 << P_A_RELAYTARGET );
+
+    if (check_user(rel, state, str, len) == 0) {
+      if (state->values[P_A_RELAYTARGET] == '+') {
+	return  0;
+      }
+    }
+    return -1;
+}
+
+
 int policytest(rel, state, what, str, len)
 struct policytest *rel;
 struct policystate *state;
@@ -958,12 +1199,33 @@ const int len;
     if (rel == NULL)
       return 0;
 
+    if (debug) {
+	printf("policytest what=%d\n", what);
+	printstate(state);
+    }
+
+    if (state->message != NULL)
+      free(state->message);
+    state->message = NULL;
+
+    if (what == POLICY_SOURCEDOMAIN)
+	return pt_sourcedomain(rel, state, str, len);
     if (what == POLICY_HELONAME)
 	return pt_heloname(rel, state, str, len);
     if (what == POLICY_MAILFROM)
 	return pt_mailfrom(rel, state, str, len);
     if (what == POLICY_RCPTTO)
 	return pt_rcptto(rel, state, str, len);
+    if (what == POLICY_RCPTPOSTMASTER)
+	return pt_rcptpostmaster(rel, state, str, len);
 
     abort();			/* Code error! Bad policy ! */
 }
+
+char *
+policymsg(rel, state)
+struct policytest *rel;
+struct policystate *state;
+{
+    return state->message;
+}
