@@ -70,7 +70,9 @@ char *str, *wstr;
 	if (!s)
 	    return -1;
 	*s = 0;
-	if (cistrncmp(str, "ipv6.", 5) == 0)
+	if (cistrncmp(str, "IPv6.", 5) == 0)
+	    str += 5;
+	if (cistrncmp(str, "IPv6:", 5) == 0)
 	    str += 5;
 	return parse_ip_policy(pbuf, str, wstr);
     }
@@ -140,32 +142,46 @@ char *str;
     return parse_gen_policy(pbuf, str, P_K_DOMAIN);
 }
 
+struct _tokenpair {
+  char *name;
+  int  key;
+} attributes [] = {
+  { "=",		P_A_ALIAS		},
+  { "rejectnet",	P_A_REJECTNET		},
+  { "freezenet",	P_A_FREEZENET		},
+  { "rejectsource",	P_A_REJECTSOURCE	},
+  { "freezesource",	P_A_FREEZESOURCE	},
+  { "relaycustomer",	P_A_RELAYCUSTOMER	},
+  { "relaycustnet",	P_A_RELAYCUSTNET	},
+  { "relaytarget",	P_A_RELAYTARGET		},
+  { "acceptifmx",	P_A_ACCEPTifMX		},
+  { "acceptifdns",	P_A_ACCEPTifDNS		},
+  { "senderokwithdns",	P_A_SENDERokWithDNS	},
+  { "freeze",		P_A_ACCEPTbutFREEZE	},
+  { "sendernorelay",	P_A_SENDERNoRelay	},
+  { "message",		P_A_MESSAGE		},
+  { "test-dns-rbl",	P_A_TestDnsRBL		},
+  { "localdomain",	P_A_LocalDomain		},
+  { NULL, 0 },
+};
+
+
 int parseattributepair(abuf, str1, str2)
 struct attribute *abuf;
 char *str1, *str2;
 {
+    struct _tokenpair *ap = attributes;
+
     abuf->len = 2 + strlen(str2) + 1;
     strcpy(abuf->data, str2);
 
-    if (strcmp(str1, "=") == 0)
-	abuf->attrib = P_A_ALIAS;
-    else if (strcmp(str1, "rejectnet") == 0)
-	abuf->attrib = P_A_REJECTNET;
-    else if (strcmp(str1, "freezenet") == 0)
-	abuf->attrib = P_A_FREEZENET;
-    else if (strcmp(str1, "rejectsource") == 0)
-	abuf->attrib = P_A_REJECTSOURCE;
-    else if (strcmp(str1, "freezesource") == 0)
-	abuf->attrib = P_A_FREEZESOURCE;
-    else if (strcmp(str1, "relaycustomer") == 0)
-	abuf->attrib = P_A_RELAYCUSTOMER;
-    else if (strcmp(str1, "relaycustnet") == 0)
-	abuf->attrib = P_A_RELAYCUSTNET;
-    else if (strcmp(str1, "relaytarget") == 0)
-	abuf->attrib = P_A_RELAYTARGET;
-    else if (strcmp(str1, "freeze") == 0)
-	abuf->attrib = P_A_ACCEPTbutFREEZE;
-    else
-	return -1;
-    return 0;
+    while (ap->name != NULL) {
+      if (strcmp(ap->name, str1) == 0) {
+	abuf->attrib = ap->key;
+	return 0;
+      }
+      ++ap;
+    }
+
+    return -1;
 }
