@@ -1,7 +1,7 @@
 /* Small pieces for scanning forward on a buffer of RFC-821/822 compliant
    addresses */
 
-/* (c) Matti Aarnio 1993-1996 <mea@nic.funet.fi> */
+/* (c) Matti Aarnio 1993-1997 <mea@nic.funet.fi> */
 
 /* All these routines scan over the lexical elements they are after, and
    if successfull, return pointer just AFTER such element.
@@ -48,17 +48,17 @@ static const char *no_input = "No input";
    anything smaller than int (32 bit) */
 static int char_array[] =
 {
-		/*        /--------------------- CHAR_ALPHA
-		   | /------------------- CHAR_ALNUM
-		   | | /----------------- CHAR_SPECL
-		   | | | /--------------- CHAR_DIGIT
-		   | | | | /------------- CHAR_C
-		   | | | | | /----------- CHAR_X
-		   | | | | | | /--------- CHAR_Q
-		   | | | | | | | /------- CHAR_XDIGIT
-		   | | | | | | | | /----- CHAR_822ATM
-		   | | | | | | | | | /--- CHAR_XCHAR
-		   | | | | | | | | | |                  */
+		       /* /------------------------------ CHAR_ALPHA
+			  |  /--------------------------- CHAR_ALNUM
+			  |  |  /------------------------ CHAR_SPECL
+			  |  |  |  /--------------------- CHAR_DIGIT
+			  |  |  |  |  /------------------ CHAR_C
+			  |  |  |  |  |  /--------------- CHAR_X
+			  |  |  |  |  |  |  /------------ CHAR_Q
+			  |  |  |  |  |  |  |  /--------- CHAR_XDIGIT
+			  |  |  |  |  |  |  |  |  /------ CHAR_822ATM
+			  |  |  |  |  |  |  |  |  |  /--- CHAR_XCHAR
+			  |  |  |  |  |  |  |  |  |  |                  */
 /*   0: `^@'   */ CHR_ENT(0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
 /*   1: `^A'   */ CHR_ENT(0, 0, 1, 0, 0, 1, 1, 0, 0, 0),
 /*   2: `^B'   */ CHR_ENT(0, 0, 1, 0, 0, 1, 1, 0, 0, 0),
@@ -92,11 +92,11 @@ static int char_array[] =
 /*  30: `^^'   */ CHR_ENT(0, 0, 1, 0, 0, 1, 1, 0, 0, 0),
 /*  31: `^_'   */ CHR_ENT(0, 0, 1, 0, 0, 1, 1, 0, 0, 0),
 /*  32: ` '    */ CHR_ENT(0, 0, 0, 0, 0, 1, 1, 0, 0, 0),
-/*  33: `!'    */ CHR_ENT(0, 0, 0, 0, 1, 1, 1, 0, 1, 1),
+/*  33: `!'    */ CHR_ENT(0, 0, 0, 0, 0, 1, 1, 0, 1, 1),
 /*  34: `"'    */ CHR_ENT(0, 0, 1, 0, 0, 1, 0, 0, 1, 1),
 /*  35: `#'    */ CHR_ENT(0, 0, 0, 0, 1, 1, 1, 0, 1, 1),
 /*  36: `$'    */ CHR_ENT(0, 0, 0, 0, 1, 1, 1, 0, 1, 1),
-/*  37: `%'    */ CHR_ENT(0, 0, 0, 0, 1, 1, 1, 0, 1, 1),
+/*  37: `%'    */ CHR_ENT(0, 0, 0, 0, 0, 1, 1, 0, 1, 1),
 /*  38: `&'    */ CHR_ENT(0, 0, 0, 0, 1, 1, 1, 0, 1, 1),
 /*  39: `''    */ CHR_ENT(0, 0, 0, 0, 1, 1, 1, 0, 1, 1),
 /*  40: `('    */ CHR_ENT(0, 0, 1, 0, 0, 1, 1, 0, 0, 1),
@@ -490,7 +490,7 @@ char *s;
 int strict;
 {
     /* rfc821_v6dotnum() -- rather heavily mutated from BIND 4.9.4 inet_pton6()
-       *                       routine by Matti Aarnio <mea@nic.funet.fi>, 1997   */
+     *                      routine by Matti Aarnio <mea@nic.funet.fi>, 1997 */
     /* int
      * inet_pton6(src, dst)
      *    convert presentation level address to network order binary form.
@@ -530,7 +530,7 @@ int strict;
     saw_xdigit = 0;
     val = 0;
 
-    while ((ch = *src++) != '\0') {
+    for ( ;((ch = *src) != '\0' && ch != ']'); ++src) {
 
 	if (is_821_xdigit(ch)) {
 	    val <<= 4;
@@ -578,6 +578,8 @@ int strict;
 	    saw_xdigit = 0;
 	    break;		/* '\0' was seen by inet_pton4(). */
 	}
+	rfc821_error_ptr = src - 1;
+	rfc821_error = "<v6dotnum> error ?";
 	return (s);
     }
     if (saw_xdigit) {
@@ -609,7 +611,8 @@ int strict;
 
     if (*s == 'i' || *s == 'I') {
 	/* Wow! Possibly IPv6 prefix! */
-	if (cistrncmp(s, "ipv6.", 5) == 0) {
+	if (cistrncmp(s, "IPv6:", 5) == 0 /* ||
+	    cistrncmp(s, "IPv6.", 5) == 0  */  ) {
 	    p = rfc821_v6dotnum(s + 5, strict);
 	    if (p == s + 5)
 		return s;
@@ -697,7 +700,9 @@ int strict;
     }
     if (*p == '[') {
 	q = rfc821_dotnum(p + 1, strict);
-/* printf("    dotnum: p='%s', q='%s'\n",p,q); */
+#if 0
+ printf("    dotnum: p='%s', q='%s'\n",p,q); 
+#endif
 	if (q == p + 1)
 	    return s;
 	if (*q != ']') {
@@ -708,7 +713,7 @@ int strict;
 	return q + 1;
     }
     if (*p == '#') {
-	/* Depreciated  "#1234234324" -format */
+	/* Deprecated  "#1234234324" -format */
 	++p;
 	while (is_821_digit(*p))
 	    ++p;
@@ -835,6 +840,13 @@ int strict;
 	return s;		/* Missing string */
     while (*q == '.') {		/* Well, intermediate dot.. */
 	p = q + 1;
+#if 0
+	if (!is_821_alnum(*p)) {
+	  rfc821_error_ptr = q;
+	  rfc821_error = "After a dot, something which is not alphanumeric";
+	  return s;
+	}
+#endif
 	q = rfc821_string(p, strict);
 	if (q == p)
 	    return s;		/* Missing string */
@@ -923,6 +935,15 @@ int strict;
 		return s;	/* Uh... */
 	    if (*q == '%' || *q == '!') {
 		p = q + 1;
+		if (!is_821_alnum(*p) && *p != '"') {
+		  rfc821_error_ptr = q;
+		  if (*q == '%') {
+		    rfc821_error = "After a '%', a non alphanumeric element";
+		  } else {
+		    rfc821_error = "After a '!', a non alphanumeric element";
+		  }
+		  return s;
+		}
 		continue;
 	    }
 	    return q;
@@ -932,6 +953,15 @@ int strict;
 	    return s;		/* Uh... */
 	if (*q == '%' || *q == '!') {
 	    p = q + 1;
+	    if (!is_821_alnum(*p) && *p != '"') {
+	      rfc821_error_ptr = q;
+	      if (*q == '%') {
+		rfc821_error = "After a '%', a non alphanumeric element";
+	      } else {
+		rfc821_error = "After a '!', a non alphanumeric element";
+	      }
+	      return s;
+	    }
 	    continue;
 	}
 	return q;
