@@ -50,12 +50,14 @@ rfc822commentprint(mfp, str)
 FILE *mfp;
 const char *str;
 {
+  putc('"', mfp);
   for ( ; *str ; ++str ) {
     int c = (*str) & 0xFF;
-    if (c == '(' || c == ')' || c == '\\')
+    if (c == '"' || c == '\\')
       putc('\\', mfp);
     putc(c, mfp);
   }
+  putc('"', mfp);
 }
 
 
@@ -86,53 +88,41 @@ const char *s;
    Therefore it is highly unlikely to need even more than two chars
    longer form for the RFC-822 format string. */
 
-#ifdef USE_ALLOCA
-#define RFC821_822QUOTE(cp,newcp,alen) \
-	if (cp && memchr(cp,'\\',alen) != NULL && *cp != '"') { \
-	  int i;						\
-	  const char *s1 = cp;					\
-	  char *s2 = emalloc(alen+8);				\
-	  newcp = s2;						\
-	  *s2++ = '"';						\
-	  for (i = 0; i < alen; ++i) {				\
-	    if (*s1 == '@')					\
-	      break; /* Unquoted AT --> move to plain copying! */ \
-	    if (*s1 == '\\' && s1[1] != 0)			\
-	      *s2++ = *s1++;					\
-	    /* Normal copy */					\
-	    *s2++ = *s1++;					\
-	  }							\
-	  *s2++ = '"';						\
-	  /* The possible final copying of '@hostpart' */	\
-	  for ( ; i < alen; ++i)				\
-	    *s2++ = *s1++;					\
-	  alen = s2 - cp;					\
-	  cp = newcp;						\
+static void rfc821to822quote(cpp,newcpp,alenp)
+     char **cpp, **newcpp;
+     int *alenp;
+{
+	char *cp    = *cpp;
+	char *newcp = *newcpp;
+	int alen    = *alenp;
+	if (cp && memchr(cp,'\\',alen) != NULL && *cp != '"') {
+	  int i;
+	  const char *s1 = cp;
+	  char *s2 = emalloc(alen+8);
+	  newcp = s2;
+	  *s2++ = '"';
+	  for (i = 0; i < alen; ++i) {
+	    if (*s1 == '@')
+	      break; /* Unquoted AT --> move to plain copying! */
+	    if (*s1 == '\\' && s1[1] != 0)
+	      { *s2++ = *s1++; ++i; }
+	    /* Normal copy */
+	    *s2++ = *s1++;
+	  }
+	  *s2++ = '"';
+	  /* The possible final copying of '@hostpart' */
+	  for ( ; i < alen; ++i)
+	    *s2++ = *s1++;
+	  alen = s2 - newcp;
+	  cp = newcp;
 	}
-#else				/* No alloca .. must do without */
+	*cpp    = cp;
+	*newcpp = newcp;
+	*alenp  = alen;
+}
 #define RFC821_822QUOTE(cp,newcp,alen) \
-	if (cp && memchr(cp,'\\',alen) != NULL && *cp != '"') { \
-	  int i;						\
-	  const char *s1 = cp;					\
-	  char *s2 = emalloc(alen+8);				\
-	  newcp = s2;						\
-	  *s2++ = '"';						\
-	  for (i = 0; i < alen; ++i) {				\
-	    if (*s1 == '@')					\
-	      break; /* Unquoted AT --> move to plain copying! */ \
-	    if (*s1 == '\\' && s1[1] != 0)			\
-	      *s2++ = *s1++;					\
-	    /* Normal copy */					\
-	    *s2++ = *s1++;					\
-	  }							\
-	  *s2++ = '"';						\
-	  /* The possible final copying of '@hostpart' */	\
-	  for ( ; i < alen; ++i)				\
-	    *s2++ = *s1++;					\
-	  alen = s2 - cp;					\
-	  cp = newcp;						\
-	}
-#endif
+	rfc821to822quote(&cp,&newcp,&alen)
+
 
 /* SMTP-server verb subroutines */
 
@@ -140,22 +130,27 @@ void smtp_helo(SS, buf, cp)
 SmtpState *SS;
 const char *buf, *cp;
 {
-
+    const char *msg = NULL;
     if (SS->state != Hello && SS->state != MailOrHello) {
 	switch (SS->state) {
 	case Mail:
-	    cp = "Waiting for MAIL command";
+	    msg = "Waiting for MAIL command";
 	    break;
 	case Recipient:
-	    cp = "Waiting for RCPT command";
+	    msg = "Waiting for RCPT command";
 	    break;
 	default:
-	    cp = NULL;
 	    break;
 	}
-	type(SS, 503, NULL, cp);
-	return;
     }
+
+    /* HELO/EHLO is also implicite RSET ! */
+    if (SS->mfp != NULL) {
+      clearerr(SS->mfp);
+      mail_abort(SS->mfp);
+      SS->mfp = NULL;
+    }
+
     strncpy(SS->helobuf, buf, sizeof(SS->helobuf));
     SS->helobuf[sizeof(SS->helobuf)-1] = 0;
 
@@ -164,6 +159,13 @@ const char *buf, *cp;
 
     SS->policyresult = policytest(policydb, &SS->policystate,
 				  POLICY_HELONAME, cp, strlen(cp));
+    if (logfp) {
+      char *s = policymsg(policydb, &SS->policystate);
+      fprintf(logfp, "%d\t-- policy result=%d, msg: %s\n", pid,
+	      SS->policyresult, (s ? s : "<NONE!>"));
+      fflush(logfp);
+    }
+
     /* actually we accept here about anything, and mark
        our decissions up into the 'policystate' variables */
 
@@ -183,6 +185,8 @@ const char *buf, *cp;
 	type821err(SS, -501, "", buf,
 		   "Invalid `%.200s' parameter!",
 		   buf);
+	if (msg != NULL)
+	  type(SS, -501, "", "%s", msg);
 	type(SS, 501, "", "Err: %s", rfc821_error);
 	strcpy(SS->helobuf, "Bad.Helo.Input");
 	return;
@@ -193,6 +197,8 @@ const char *buf, *cp;
 	if (SS->carp->cmd != Hello) {
 	    type821err(SS, -250, "", buf, "Invalid `%.200s' parameter!", buf);
 	    type(SS, -250, NULL, "Err: %s", rfc821_error);
+	    if (msg != NULL)
+	      type(SS, -250, "", "%s", msg);
 	}
     }
     SS->cfinfo = findcf(cp);
@@ -213,12 +219,17 @@ const char *buf, *cp;
 	    type(SS, -250, NULL, "%s your address to name mapping.",
 		 SS->myhostname);
 	}
+	if (msg != NULL)
+	  type(SS, -250, "", "%s", msg);
 	type(SS, SS->carp->cmd == Hello2 ? -250 : 250, NULL,
 	     "%s expected \"%s %s\"", SS->myhostname,
 	     SS->carp->verb, SS->rhostname);
-    } else
+    } else {
+	if (msg != NULL)
+	  type(SS, -250, "", "%s", msg);
 	type(SS, SS->carp->cmd == Hello2 ? -250 : 250, NULL,
 	     "%s Hello %s", SS->myhostname, cp);
+    }
     if (SS->carp->cmd == Hello2) {
 
 	/* ESMTP -- RFC 1651,1652,1653,1428 thingies */
@@ -230,12 +241,17 @@ const char *buf, *cp;
 	type(SS, -250, NULL, sizebuf);
 	type(SS, -250, NULL, "8BITMIME");
 	type(SS, -250, NULL, "PIPELINING");
-	type(SS, -250, NULL, "CHUNKING");	/* RFC 1830 */
+	type(SS, -250, NULL, "CHUNKING");	/* RFC 1830: BDAT */
 	type(SS, -250, NULL, "ENHANCEDSTATUSCODES");
-	type(SS, -250, NULL, "EXPN");
-	type(SS, -250, NULL, "VRFY");
+	if (expncmdok && STYLE(SS->cfinfo, 'e'))
+	  type(SS, -250, NULL, "EXPN");
+	if (vrfycmdok && STYLE(SS->cfinfo, 'v'))
+	  type(SS, -250, NULL, "VRFY");
 	type(SS, -250, NULL, "DSN");
+#if 1 /* This causes problems for the router, will fix router
+	 performance first, then enable this again.. */
 	type(SS, -250, NULL, "X-RCPTLIMIT 10000");	/* VERY HIGH figure, normal is 100 */
+#endif
 	type(SS, -250, NULL, "ETRN");
 	type(SS, 250, NULL, "HELP");
 	SS->with_protocol = WITH_ESMTP;
@@ -254,6 +270,7 @@ int insecure;
     const char *drpt_ret;
     const char *bodytype = NULL;
     const char *newcp = NULL;
+    const char *srcrtestatus = "";
     int addrlen, drptret_len, drptenvid_len;
 
     addrlen = 0;
@@ -329,6 +346,15 @@ int insecure;
 	type(SS, 501, m552, "there are too many >'s in that!");
 	return;
     }
+    /* Ok, now it is a moment to see, if we have source routes: @a,@b:c@d */
+    if (cp[1] == '@') {
+      /* Yup, Starting with an "@" ..  scan until ":", which must be
+	 in there as this is valid RFC-821 object. */
+      if (!allow_source_route) {
+	while (*cp != ':') ++cp; 
+	srcrtestatus = ", source route ignored";
+      }
+    }
     ++cp;			/* Skip the initial '<' */
     addrlen = s - 1 - cp;	/* Length until final  '>' */
 
@@ -441,33 +467,62 @@ int insecure;
 
     SS->policyresult = policytest(policydb, &SS->policystate,
 				  POLICY_MAILFROM, cp, addrlen);
+    if (logfp) {
+      char *s = policymsg(policydb, &SS->policystate);
+      fprintf(logfp, "%d\t-- policy result=%d, msg: %s\n", pid,
+	      SS->policyresult, (s ? s : "<NONE!>"));
+      fflush(logfp);
+    }
+
     if (SS->policyresult < 0) {
-	type(SS, 553, "5.7.1", "The source address is in a reject list");
-#ifndef USE_ALLOCA
-	if (newcp)
-	    free(newcp);
-#endif
-	return;
+      char *s = policymsg(policydb, &SS->policystate);
+      if (s != NULL) {
+	type(SS,-453, "4.7.1", "Policy analysis reported:");
+	type(SS, 453, "4.7.1", "%s", s);
+      } else if (SS->policyresult < -99) {
+	if (SS->policyresult < -100) {
+	  type(SS, -453, "4.7.1", "Policy analysis reports DNS error with your source domain.");
+	  type(SS,  453, "4.7.1", "Please correct your source address and/or the info at the DNS.");
+	} else {
+	  type(SS, -453, "4.7.1", "Access denied by the policy analysis functions.");
+	  type(SS, -453, "4.7.1", "This may be due to your source IP address, the IP reversal domain,");
+	  type(SS, -453, "4.7.1", "the data you gave for the HELO/EHLO parameter, or address/domain");
+	  type(SS, 453, "4.7.1", "you gave at the MAIL FROM:<...> address.");
+	}
+      } else {
+	char *s = policymsg(policydb, &SS->policystate);
+	if (s != NULL) {
+	  type(SS,-553, "5.7.1", "Policy analysis reported:");
+	  type(SS, 553, "5.7.1", "%s", s);
+	} else if (SS->policyresult < -1) {
+	  type(SS, -553, "5.7.1", "Policy analysis reports DNS error with your source domain.");
+	  type(SS,  553, "5.7.1", "Please correct your source address and/or the info at the DNS.");
+	} else {
+	  type(SS, -553, "5.7.1", "Access denied by the policy analysis functions.");
+	  type(SS, -553, "5.7.1", "This may be due to your source IP address, the IP reversal domain,");
+	  type(SS, -553, "5.7.1", "the data you gave for the HELO/EHLO parameter, or address/domain");
+	  type(SS, 553, "5.7.1", "you gave at the MAIL FROM:<...> address.");
+	}
+      }
+      if (newcp)
+	free((void*)newcp);
+      return;
     }
     s = NULL;
-    if (*cp != '\0' && STYLE(SS->cfinfo, 'f')) {
+    if (/* addrlen > 0 && */ STYLE(SS->cfinfo, 'f')) {
 	s = router(SS, RKEY_FROM, 1, cp, addrlen);
 	if (s == NULL) {
 	    /* the error was printed in router() */
-#ifndef USE_ALLOCA
 	    if (newcp)
-		free(newcp);
-#endif
+		free((void*)newcp);
 	    return;
 	}
 	if (atoi(s) / 100 != 2) {
 	    /* verification failed */
 	    type(SS, atoi(s), s + 4, "Failed", "Failed");
 	    free((void *) s);
-#ifndef USE_ALLOCA
 	    if (newcp)
 		free((void *) newcp);
-#endif
 	    return;
 	}
 	/* The 's' goes to use below */
@@ -477,21 +532,18 @@ int insecure;
 	if (s)
 	    free((void *) s);
 	type(SS, 452, m430, (char *) NULL);
-#ifndef USE_ALLOCA
 	if (newcp)
 	    free((void *) newcp);
-#endif
 	return;
     }
     rewind(SS->mfp);
     if (insecure)
 	fprintf(SS->mfp, "external\n");
-    fprintf(SS->mfp, "rcvdfrom %s ", SS->rhostname);
+    fprintf(SS->mfp, "rcvdfrom %s (", SS->rhostname);
     if (SS->ihostaddr[0] != 0)
-      fprintf(SS->mfp, "(%s) ", SS->ihostaddr);
-    fprintf(SS->mfp, "(");
+      fprintf(SS->mfp, "%s:%d ", SS->ihostaddr, SS->rport);
     rfc822commentprint(SS->mfp, SS->helobuf);
-    fprintf(SS->mfp, ") (ident: ");
+    fprintf(SS->mfp, " ident: ");
     rfc822commentprint(SS->mfp, SS->ident_username);
     fprintf(SS->mfp, ")\n");
 
@@ -511,10 +563,8 @@ int insecure;
     if (SS->policyresult > 0)
       fprintf(SS->mfp,"comment policytest() reported freeze state\n");
 
-#ifndef USE_ALLOCA
     if (newcp)
-	free(newcp);
-#endif
+	free((void *)newcp);
 
     if (drpt_envid != NULL) {
 	fprintf(SS->mfp, "envid ");
@@ -536,7 +586,7 @@ int insecure;
 	if (s)
 	    type(SS, atoi(s), s + 4, "Ok");
 	else
-	    type(SS, 250, "2.1.0", "Sender syntax Ok");
+	    type(SS, 250, "2.1.0", "Sender syntax Ok%s", srcrtestatus);
 	SS->sender_ok = 1;
     }
     if (s)
@@ -553,9 +603,21 @@ const char *buf, *cp;
 {
     const char *s;
     const char *drpt_notify, *drpt_orcpt;
-    char *newcp = NULL;
+    const char *newcp = NULL;
+    const char *srcrtestatus = "";
     int addrlen = 0, notifylen = 0, orcptlen = 0;
 
+    /* some smtp clients don't get the 503 right and try again, so
+       tell the spammers exactly what's happening. */
+    if ( (SS->state == MailOrHello || SS->state == Mail) &&
+	 policydb != NULL && SS->policyresult < 0 ) {
+      type(SS, -553, "5.1.8", "Access denied by the policy analysis functions.");
+      type(SS, -553, "5.1.8", "This may be due to your source IP address, the IP reversal domain,");
+      type(SS, -553, "5.1.8", "the data you gave for the HELO/EHLO parameter, or address/domain");
+      type(SS, 553, "5.1.8", "you gave at the MAIL FROM:<...> address.");
+      return;
+    }
+
     if (SS->state != Recipient && SS->state != RecipientOrData) {
 	switch (SS->state) {
 	case Hello:
@@ -577,7 +639,7 @@ const char *buf, *cp;
 	return;
     }
     for (cp = cp + 3; *cp != '\0' && *cp != '<'; ++cp)
-	if (!isascii(*cp) || !isspace(*cp)) {
+	if (!isspace(*cp)) {
 	    type(SS, 501, m552, "where is <...> in that?");
 	    return;
 	}
@@ -587,6 +649,9 @@ const char *buf, *cp;
     } else if (*cp != '<') {
 	type(SS, 501, m552, "strangeness between : and <");
 	return;
+    } else if (*(cp+1) == '>') {
+	type(SS, 501, m552, "Null address valid only as source");
+	return;
     }
     if (*(cp + 1) == '<') {
 	type(SS, 501, m552, "there are too many <'s in that!");
@@ -595,21 +660,39 @@ const char *buf, *cp;
     /* "<" [ <a-t-l> ":" ] <localpart> "@" <domain> ">" */
     s = rfc821_path(cp, STYLE(SS->cfinfo, 'R'));
     if (s == cp) {
-	/* Failure.. */
-	type821err(SS, 501, m552, buf, "Path data: %s", rfc821_error);
-	return;
+	/* Failure ?  Perhaps we are RESTRICTIVE, and the address
+	   is '<postmaster>' without domain ? */
+	if (CISTREQN(cp, "<POSTMASTER>", 12)) {
+	  s += 12;
+	} else {
+	  /* Genuine failure.. */
+	  type821err(SS, 501, m552, buf, "Path data: %s", rfc821_error);
+	  return;
+	}
     }
     if (*s == '>') {
 	type(SS, 501, m552, "there are too many >'s in that!");
 	return;
     }
+    /* Ok, now it is a moment to see, if we have source routes: @a,@b:c@d */
+    if (cp[1] == '@') {
+      /* Yup, Starting with an "@" ..  scan until ":", which must be
+	 in there as this is valid RFC-821 object. */
+      if (!allow_source_route) {
+	while (*cp != ':') ++cp; 
+	srcrtestatus = ", source route ignored";
+      }
+    }
     ++cp;			/* Skip the initial '<' */
     addrlen = s - 1 - cp;	/* Length until final  '>' */
 
-    /*printf("  <path>: len=%d \"%s\"\n",cp-s,cp); */
+#if 0
+    if (debug)
+      type(SS, 000, "", "<path>: len=%d \"%.*s\"\r\n", addrlen, addrlen, cp);
+#endif
 
-    if (*cp == '\0') {
-	type(SS, 501, "5.1.3", "what is a null recipient?");
+    if (addrlen < 1) {
+	type(SS, 501, "5.1.3", "What is an empty recipient?");
 	return;
     }
     drpt_notify = NULL;
@@ -670,18 +753,78 @@ const char *buf, *cp;
 
     SS->policyresult = policytest(policydb, &SS->policystate,
 				  POLICY_RCPTTO, cp, addrlen);
-    if (SS->policyresult < 0) {
-      fprintf(SS->mfp,"comment policytest() rejected rcptaddr: <");
-      fwrite(cp, 1, addrlen, SS->mfp);
-      fprintf(SS->mfp,">\n");
+    if (logfp) {
+      char *s = policymsg(policydb, &SS->policystate);
+      fprintf(logfp, "%d\t-- policy result=%d, msg: %s\n", pid,
+	      SS->policyresult, (s ? s : "<NONE!>"));
+      fflush(logfp);
+    }
 
-	type(SS, 553, "5.7.1", "This is not acceptable target address");
-#ifndef USE_ALLOCA
+    if (SS->postmasteronly ||
+	SS->policyresult < 0) {
+      if (CISTREQN(cp, "postmaster", 10)) {
+	/* Rejected, but it seems to be a postmaster ??? */
+	if (addrlen == 10)
+	  SS->policyresult = 0; /* Plain <postmaster> */
+	else
+	  if (policydb != NULL) {
+	    SS->policyresult = policytest(policydb, &SS->policystate,
+					  POLICY_RCPTPOSTMASTER, cp, addrlen);
+	    if (logfp) {
+	      char *s = policymsg(policydb, &SS->policystate);
+	      fprintf(logfp, "%d\t-- policy result=%d, msg: %s\n", pid,
+		      SS->policyresult, (s ? s : "<NONE!>"));
+	      fflush(logfp);
+	    }
+	  }
+      }
+    }
+    if (SS->policyresult < 0) {
+	char *s = policymsg(policydb, &SS->policystate);
+
+	fprintf(SS->mfp, "comment policytest() rejected rcptaddr: <");
+	fwrite(cp, 1, addrlen, SS->mfp);
+	fprintf(SS->mfp,">\n");
+
+	if (SS->policyresult < -99) { /* "soft error, 4XX code */
+	  if (s != NULL) {
+	    type(SS,-453, "4.7.1", "Policy analysis reported:");
+	    type(SS, 453, "4.7.1", "%s", s);
+	  } else if (SS->policyresult < -102) {
+	    /* Code: -103 */
+	    type(SS,-453, "4.7.1", "This target address is not our MX service client, nor you are");
+	    type(SS,-453, "4.7.1", "connecting from address that is allowed to openly use us to relay");
+	    type(SS, 453, "4.7.1", "to any arbitary address thru us.  We don't accept this recipient.");
+	  } else if (SS->policyresult < -100) {
+	    /* Code: -102 */
+	    type(SS,-453, "4.7.1", "Policy analysis found DNS error on the target address.");
+	    type(SS, 453, "4.7.1", "This address is not currently acceptable.");
+	  } else {
+	    type(SS, 453, "4.7.1", "Policy rejection on the target address");
+	  }
+	} else {
+	  char *s = policymsg(policydb, &SS->policystate);
+	  if (s != NULL) {
+	    type(SS,-553, "5.7.1", "Policy analysis reported:");
+	    type(SS, 553, "5.7.1", "%s", s);
+	  } else if (SS->policyresult < -2) {
+	    /* Code: -3 */
+	    type(SS,-553, "5.7.1", "This target address is not our MX service client, nor you are");
+	    type(SS,-553, "5.7.1", "connecting from address that is allowed to openly use us to relay");
+	    type(SS, 553, "5.7.1", "to any arbitary address thru us.  We don't accept this recipient.");
+	  } else if (SS->policyresult < -1) {
+	    /* Code: -2 */
+	    type(SS,-553, "5.7.1", "Policy analysis found DNS error on the target address.");
+	    type(SS, 553, "5.7.1", "This address is not currently acceptable.");
+	  } else {
+	    type(SS, 553, "5.7.1", "Policy rejection on the target address");
+	  }
+	}
 	if (newcp)
 	    free((void *) newcp);
-#endif
 	return;
     }
+
     s = NULL;
     if (STYLE(SS->cfinfo, 't')) {
 	s = router(SS, RKEY_TO, 1, cp, addrlen);
@@ -692,10 +835,8 @@ const char *buf, *cp;
 	    /* verification failed */
 	    type(SS, atoi(s), s + 4, "Failed", "Failed");
 	    free((void *) s);
-#ifndef USE_ALLOCA
 	    if (newcp)
 		free((void *) newcp);
-#endif
 	    return;
 	}
 	/* The 's' goes to use below */
@@ -732,12 +873,11 @@ const char *buf, *cp;
     fprintf(SS->mfp, ">\n");
 
     if (SS->policyresult > 0)
-      fprintf(SS->mfp,"comment policytest() reported freeze state\n");
+      fprintf(SS->mfp,"comment policytest() reported freeze state %d\n",
+	      SS->policyresult);
 
-#ifndef USE_ALLOCA
     if (newcp)
-	free(newcp);
-#endif
+	free((void *)newcp);
 
     if (SS->sizeoptval < 0)
 	SS->sizeoptval = 0;
@@ -758,10 +898,11 @@ const char *buf, *cp;
 	if (SS->from_box && SS->rcpt_count > MaxErrorRecipients) {
 	    type(SS, 552, "5.7.1", "SPAM trap -- too many recipients for an empty source address!");
 	} else if (SS->sizeoptval)
-	    type(SS, 250, "2.1.5", "Ok; can accomodate %d byte message",
-		 SS->sizeoptval);
+	    type(SS, 250, "2.1.5", "Ok; can accomodate %d byte message%s",
+		 SS->sizeoptval, srcrtestatus);
 	else
-	    type(SS, 250, "2.1.5", "Recipient address syntax Ok");
+	    type(SS, 250, "2.1.5", "Recipient address syntax Ok%s",
+		 srcrtestatus);
 	SS->rcpt_count += 1;
     }
     if (s)
@@ -799,7 +940,7 @@ const char *buf, *cp;
 {
     char *s;
     int cfi;
-    char *newcp = NULL;
+    const char *newcp = NULL;
     int addrlen;
 
     if (SS->state == Hello) {
@@ -843,10 +984,8 @@ const char *buf, *cp;
     } else
 	type(SS, 252, "2.5.2", (char *) NULL);	/* Syntax ok */
 
-#ifndef USE_ALLOCA
     if (newcp)
-	free(newcp);
-#endif
+	free((void *)newcp);
 }
 
 void smtp_expand(SS, buf, cp)
@@ -893,11 +1032,9 @@ const char *buf, *cp;
 	    /* printf("%s\r\n", s); */
 	    free(s);
 	} else
-	    type(SS, 501, m540, "Unable to verify that address");
-#ifndef USE_ALLOCA
+	    type(SS, 501, m540, "Unable to expand that address");
 	if (newcp)
-	    free(newcp);
-#endif
+	    free((void *)newcp);
     } else
 	type(SS, 502, m540, (char *) NULL);
 }
