@@ -146,6 +146,7 @@ typedef struct {
     struct command *carp;
     struct policystate policystate;
     int  policyresult, reject_net;
+    int  postmasteronly;
     int  rport;
     char ihostaddr[sizeof("[ipv6.ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255]") + 8];
     Usockaddr raddr;
@@ -187,6 +188,10 @@ extern int ListenQueueSize;
 extern int MaxSameIpSource;
 extern int percent_accept;
 extern int smtp_syslog;
+extern int allow_source_route;
+extern int debugcmdok;
+extern int expncmdok;
+extern int vrfycmdok;
 
 extern const char *progname;
 extern int debug, skeptical, checkhelo, ident_flag, verbose;
