@@ -272,8 +272,9 @@ const char *channel;
 const char *logfile;
 FILE *logfp = NULL;
 FILE *verboselog = NULL;
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 int   readalready = 0;		/* does buffer contain valid message data? */
+char  let_buffer[8*BUFSIZ];
 #endif
 uid_t currenteuid;		/* the current euid */
 extern int nobody;		/* safe uid for file/program delivery */
@@ -306,7 +307,7 @@ extern struct passwd *getpwnam __((const char *));
 extern struct passwd *getpwuid __((uid_t));
 #endif
 extern int setupuidgid __((struct rcpt *, int, int));
-extern int createfile __((struct rcpt *, const char *, int));
+extern int createfile __((struct rcpt *, const char *, int, int));
 extern int exstat __((struct rcpt *, const char *, struct stat *, int (*)(const char*, struct stat*) ));
 extern int creatembox __((struct rcpt *, const char *, char **, uid_t*, gid_t*, struct passwd *));
 extern char *exists __((const char *, const char *, struct rcpt *));
@@ -343,6 +344,8 @@ struct conshell *envarlist = NULL;
 extern int stickymem;	/* for strsave() */
 int	D_alloc = 0;
 
+static void decodeXtext __((FILE *, const char *));
+
 #if defined(HAVE_SOCKET) && defined(HAVE_PROTOCOLS_RWHOD_H)
 static int readrwho __((void));
 #endif
@@ -413,17 +416,27 @@ main(argc, argv)
 	  exit(EX_NOPERM);
 	}
 
+	s = getzenv("MAILBOX");
+	if (s != NULL) {
+	  maildirs[0] = s;
+	  maildirs[1] = NULL;
+	}
+
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
 	while (1) {
-	  c = getopt(argc, (char*const*)argv, "abc:Dgh:Hl:rRSMV8");
+	  c = getopt(argc, (char*const*)argv, "abc:d:Dgh:Hl:rRSMV8");
 	  if (c == EOF)
 	    break;
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
 	    break;
+	  case 'd':             /* log file */
+	    maildirs[0] = strdup(optarg);
+	    maildirs[1] = NULL;
+	    break;
 	  case 'D':
 	    ++dirhashes;	/* For user "abcdefg" the mailbox path is:
 				   MAILBOX/a/b/abcdefg -- supported by
@@ -473,18 +486,13 @@ main(argc, argv)
 	  }
 	}
 	if (errflg || optind != argc) {
-	  fprintf(stderr, "Usage: %s [-8abDgHMrSV] [-l logfile] [-c channel] [-h host]\n",
+	  fprintf(stderr, "Usage: %s [-8abDgHMrSV] [-l logfile] [-c channel] [-h host] [-d mailboxdir]\n",
 		  argv[0]);
 	  exit(EX_USAGE);
 	}
 	setreuid(0, 0);		/* make us root all over */
 	currenteuid = 0;
 
-	if ((s = getzenv("MAILBOX")) != NULL) {
-	  maildirs[0] = s;
-	  maildirs[1] = NULL;
-	}
-
 	logfp = NULL;
 	if (logfile != NULL) {
 	  if ((fd = open(logfile, O_CREAT|O_APPEND|O_WRONLY, 0644)) < 0)
@@ -653,11 +661,7 @@ process(dp)
 	 * and strip the quotes from around a name.
 	 */
 
-#ifndef HAVE_MMAP
-	readalready = 0; /* ignore any previous message data cache */
-#endif
-
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	readalready = 0; /* ignore any previous message data cache */
 #endif
 
@@ -1013,14 +1017,14 @@ deliver(dp, rp, usernam, timestring)
 	    }
 	    return;
 	  }
-	  if (!setupuidgid(rp, uid, 1)) {
+	  if (!setupuidgid(rp, uid, -3 /* GID of user with UID=='uid' */)) {
 	    setrootuid(rp);
 	    return;
 	  }
 	  file = usernam;
 	  
 	  if (access(file, F_OK) < 0) {
-	    fdmail = createfile(rp, file, uid);
+	    fdmail = createfile(rp, file, uid, 0);
 	    /* Did the create fail for some other reason, than
 	       that the file exists already ? (Due to a race in
 	       between two processes creating the files.) */
@@ -1200,12 +1204,25 @@ deliver(dp, rp, usernam, timestring)
 	  return;			/* setupuidgid sets status */
 	}
 
-	if ((fdmail = open(file, O_RDWR|O_APPEND)) < 0) {
+	if (verboselog)
+	  fprintf(verboselog,
+		  "To open a file with euid=%d egid=%d file='%s'\n",
+		  geteuid(), getegid(), file);
+
+	fdmail = open(file, O_RDWR|O_APPEND);
+	if (fdmail < 0) {
 	  char fmtbuf[512];
+	  int saverrno = errno;
 
 	  sprintf(fmtbuf, "open(\"%%s\") failed: %s",
-		  strerror(errno));
-	  if (TEMPFAIL(errno))
+		  strerror(saverrno));
+
+	  if (verboselog)
+	    fprintf(verboselog,
+		    " ... failed, errno = %d (%s)\n",
+		    saverrno, strerror(saverrno));
+
+	  if (TEMPFAIL(saverrno))
 	    rp->status = EX_TEMPFAIL;
 	  else if (errno == EACCES)
 	    rp->status = EX_NOPERM;
@@ -1447,6 +1464,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	struct stat st;
 	const char *fromuser;
 	int lastch = 0xFFF;
+	int topipe = (*(file) == TO_PIPE);
+	int failed = 0;
 
 	fstat(fdmail, &st);
 
@@ -1457,7 +1476,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  return NULL;
 	}
 
-	if (*(file) != TO_PIPE && eofindex > 0L) {
+	if (!topipe && eofindex > 0L) {
 
 	  if (keepatime) {
 #ifdef	HAVE_UTIME
@@ -1493,6 +1512,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    strcmp(rp->addr->link->channel, "error") == 0)
 	  fromuser = "";
 
+	if (*fromuser == 0)
+	  fromuser = "MAILER-DAEMON";
+
 	{
 	  char **hdrs = has_header(rp,"Return-Path:");
 	  if (hdrs) delete_header(rp,hdrs);
@@ -1506,16 +1528,39 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    if (hdrs) delete_header(rp,hdrs);
 	    append_header(rp,"Apparently-To: <%s>", rp->addr->link->user);
 	  }
-	}
-	
 
-	if (*fromuser == 0)
-	  fromuser = "MAILER-DAEMON";
+	  hdrs = has_header(rp,"X-Orcpt:");
+	  if (hdrs) delete_header(rp,hdrs);
+
+	  hdrs = has_header(rp,"X-Envid:");
+	  if (hdrs) delete_header(rp,hdrs);
+	}
 
 	/* Add the From_ line and print out the header */
 
 	if (fprintf(fp, "%s%s %s", FROM_, fromuser, timestring) < 0
-	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
+	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0)
+	  failed = 1;
+
+	if (!failed && rp->orcpt) {
+	  fprintf(fp, "X-Orcpt: ");
+	  decodeXtext(fp, rp->orcpt);
+	  fprintf(fp, "\n");
+	}
+	if (!failed && dp->envid) {
+	  fprintf(fp, "X-Envid: ");
+	  decodeXtext(fp, dp->envid);
+	  fprintf(fp, "\n");
+	}
+	fprintf(fp, "\n");
+
+	fflush(fp); /* Headers written, sync possible errors here! */
+	if (!failed)
+	  failed = ferror(fp);
+
+	/* From now on, write errors to PIPE will not cause errors */
+
+	if (failed) {
 	  notaryreport(rp->addr->link->user, "failed",
 		       "5.2.2 (Write to user's mailbox failed)",
 		       "x-local; 500 (Write to user's mailbox failed)");
@@ -1542,47 +1587,34 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	}
 	lastch = appendlet(dp, rp, fp, file, is_mime);
 
-	for (;;) {
-	  errno = 0;
-	  fflush(fp); /* HP-UX 9.x barfs when flushing to the PIPE,
-			 but apparently it does not set ferror() ! */
-	  if (errno == EINTR)
-	    continue;
-	  break;
-	}
+	fflush(fp);
 
 	mw = 1;
-	if (lastch < -128 || ferror(fp)) {
-	  int err;
-      write_failure:
-	  err = errno;
-	  notaryreport(NULL,NULL,NULL,NULL);
-	  DIAGNOSTIC4(rp, file, EX_IOERR,
-		      "message write[%d] to \"%s\" failed: %s",
-		      mw, file, strerror(err));
+	if (!topipe)
+	  if (lastch < -128 || ferror(fp)) {
+	    int err;
+	  write_failure:
+	    err = errno;
+	    notaryreport(NULL,NULL,NULL,NULL);
+	    DIAGNOSTIC4(rp, file, EX_IOERR,
+			"message write[%d] to \"%s\" failed: %s",
+			mw, file, strerror(err));
 #ifdef HAVE_FTRUNCATE
-	  /* XX: should I really do this? */
-	  for (;;) {
-	    errno = 0;
-	    fflush(fp); /* HP-UX 9.x barfs when flushing to the PIPE,
-			   but apparently it does not set ferror() ! */
-	    if (errno == EINTR)
-	      continue;
-	    break;
-	  }
-	  if (eofindex >= 0)
-	    ftruncate(FILENO(fp), (off_t)eofindex);
+	    /* XX: should I really do this? */
+	    fflush(fp);
+	    if (eofindex >= 0)
+	      ftruncate(FILENO(fp), (off_t)eofindex);
 #endif
 #ifdef HAVE_FSYNC
-	  fsync(FILENO(fp));
+	    fsync(FILENO(fp));
 #endif
-	  fclose(fp);
-	  eofindex = -1;
-	  fp = NULL;
-	  goto time_reset;
-	}
+	    fclose(fp);
+	    eofindex = -1;
+	    fp = NULL;
+	    goto time_reset;
+	  }
 
-	if (*file != TO_PIPE) {
+	if (!topipe) {
 	  /*
 	   * Ok, we are NOT writing to a pipe, and thus we can do
 	   * fseek(), and play with things...
@@ -1607,14 +1639,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    --len;
 	    len = (buf[len]!='\n') + (len == 1 ? buf[0]!='\n' : 1);
 	    err = (len > 0 && (fwrite("\n\n", 1, len, fp) != len));
-	    for (;;) {
-	      errno = 0;
-	      fflush(fp); /* HP-UX 9.x barfs when flushing to the PIPE,
-			     but apparently it does not set ferror() ! */
-	      if (errno == EINTR)
-		continue;
-	      break;
-	    }
+	    fflush(fp);
+
 	    if (!err) err = ferror(fp);
 	    if (err) {
 	      notaryreport(NULL,NULL,NULL,NULL);
@@ -1622,14 +1648,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 			 "cleansing of \"%s\" failed", file);
 #ifdef	HAVE_FTRUNCATE
 	      /* XX: should I really do this? */
-	      for (;;) {
-		errno = 0;
-		fflush(fp); /* HP-UX 9.x barfs when flushing to the PIPE,
-			       but apparently it does not set ferror() ! */
-		if (errno == EINTR)
-		  continue;
-		break;
-	      }
+	      fflush(fp);
 	      if (eofindex >= 0)
 		ftruncate(FILENO(fp), (off_t)eofindex);
 #endif /* HAVE_FTRUNCATE */
@@ -1638,6 +1657,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 #endif
 	      fclose(fp);
 	      fp = NULL;
+
 	      goto time_reset;
 	    }
 	  }
@@ -1651,15 +1671,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  }
 	}
 
-	for (;;) {
-	  errno = 0;
-	  fflush(fp); /* HP-UX 9.x barfs when flushing to the PIPE,
-			 but apparently it does not set ferror() ! */
-	  if (errno == EINTR)
-	    continue;
-	  break;
-	}
-	if (ferror(fp)) {
+	fflush(fp);
+	if (!topipe && ferror(fp)) {
 	  mw=3;
 	  goto write_failure;
 	}
@@ -1682,17 +1695,17 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  fprintf(logfp,
 		  "%s: %ld + %ld : %s (pid %d user %s)\n",
 		  dp->logident, eofindex,
-		  ftell(fp) - eofindex, file,
+		  (fp ? ftell(fp): 0) - eofindex, file,
 		  getpid(), rp->addr->user);
 #if 0
 	  fprintf(logfp, "%s: %ld + %ld : %s\n",
 		  dp->logident, eofindex,
-		  ftell(fp) - eofindex, file);
+		  (fp ? ftell(fp): 0) - eofindex, file);
 #endif
 	  fflush(logfp);
 	}
 
-	if (*(rp->addr->user) != TO_PIPE && keepatime) {
+	if (!topipe && keepatime) {
 #ifdef	HAVE_UTIME
 	  struct utimbuf tv;
 	  tv.actime  = st.st_atime;
@@ -1731,6 +1744,8 @@ program(dp, rp, user, timestring)
 
 	time(&starttime);
 
+	notaryreport(rp->addr->user, NULL, NULL, NULL);
+
 	i = 0;
 	env[i++] = "SHELL=/bin/sh";
 	env[i++] = "IFS= \t\n";
@@ -1748,16 +1763,27 @@ program(dp, rp, user, timestring)
 	  if (verboselog) {
 	    fprintf(verboselog,"mailbox: User recipient address privilege code invalid (non-numeric!): '%s'\n",rp->addr->misc);
 	  }
-	  notaryreport(NULL,NULL,NULL,NULL);
+	  sprintf(buf,"x-local; 500 (User recipient address privilege code invalid [non-numeric!]: '%.200s')", rp->addr->misc);
+	  notaryreport(NULL,"failed",
+		       "5.3.0 (User address recipient privilege code invalid)",
+		       buf);
 	  DIAGNOSTIC(rp, user, EX_SOFTWARE,
 		     "Bad privilege for a pipe \"%s\"", rp->addr->misc);
 	  return;
 	}
 	pw = getpwuid(uid);
 	if (pw == NULL) {
-	  gid = 0;
-	  env[i++] = "HOME=/tmp";
-	  env[i++] = "USER=anonymous";
+
+	  if (verboselog) {
+	    fprintf(verboselog,"mailbox: User recipient address privilege code invalid (no user with this uid?): '%s'\n", rp->addr->misc);
+	  }
+	  sprintf(buf,"x-local; 500 (User recipient address privilege code invalid [no user with this uid?]: '%.200s')", rp->addr->misc);
+	  notaryreport(NULL,"failed",
+		       "5.3.0 (User address recipient privilege code invalid)",
+		       buf);
+	  DIAGNOSTIC(rp, user, EX_SOFTWARE,
+		     "Bad privilege for a pipe \"%s\"", rp->addr->misc);
+	  return;
 	} else {
 	  gid = pw->pw_gid;
 	  sprintf(cp, "HOME=%s", pw->pw_dir);
@@ -1814,13 +1840,17 @@ program(dp, rp, user, timestring)
 
 	/* now we can fork off and run the command... */
 	if (pipe(in) < 0) {
-	  notaryreport(NULL,NULL,NULL,NULL);
+	  notaryreport(NULL,"failed",
+		       "5.3.0 (out of pipe resources)",
+		       "x-local; 500 (out of pipe resources)");
 	  DIAGNOSTIC(rp, user, EX_OSERR,
 		     "cannot create pipe from \"%s\"", user);
 	  return;
 	}
 	if (pipe(out) < 0) {
-	  notaryreport(NULL,NULL,NULL,NULL);
+	  notaryreport(NULL,"failed",
+		       "5.3.0 (out of pipe resources)",
+		       "x-local; 500 (out of pipe resources)");
 	  DIAGNOSTIC(rp, user, EX_OSERR,
 		     "cannot create pipe to \"%s\"", user);
 	  close(in[0]);
@@ -1859,7 +1889,9 @@ program(dp, rp, user, timestring)
 	  write(2, "Cannot exec /bin/sh\n", 20);
 	  _exit(128);
 	} else if (pid < 0) {	/* fork failed */
-	  notaryreport(NULL,NULL,NULL,NULL);
+	  notaryreport(NULL,"failed",
+		       "5.3.0 (fork failure)",
+		       "x-local; 500 (fork failure)");
 	  DIAGNOSTIC(rp, user, EX_OSERR, "cannot fork", 0);
 	  return;
 	} /* parent */
@@ -1881,7 +1913,8 @@ program(dp, rp, user, timestring)
 	fclose(fp);
 	/* read any messages from its stdout/err on in[0] */
 	/* ... having forked and set up the pipe, we quickly continue */
-	if (fgets(buf, sizeof buf, errfp) == NULL)
+	buf[sizeof(buf)-100] = 0; /* Chop it just to make sure */
+	if (fgets(buf, (sizeof buf) - 100, errfp) == NULL)
 		buf[0] = '\0';
 	else if ((cp = strchr(buf, '\n')) != NULL)
 		*cp = '\0';
@@ -1906,11 +1939,15 @@ program(dp, rp, user, timestring)
 	    strcat(cp, " (Core dumped)");
 	  strcat(cp, "]");
 	  i = EX_TEMPFAIL;
-	} else if (((status>>8)&0377) > 0) {
+	} else if ((i = (status >> 8) & 0377) > 0) {
 	  if (cp != buf)
 	    *cp++ = ' ';
-	  sprintf(cp, "[exit status %d]", (status>>8)&0377);
-	  i = EX_UNAVAILABLE;
+	  sprintf(cp, "[exit status %d]", i);
+	  /* We report following status codes to the system as is,
+	     all the rest are treated as EX_TEMPFAIL, and retried.. */
+	  if (!(i == EX_NOPERM || i == EX_UNAVAILABLE || i == EX_NOHOST ||
+		i == EX_NOUSER || i == EX_DATAERR))
+	    i = EX_TEMPFAIL;
 	}
 
 	if (verboselog)
@@ -1918,7 +1955,18 @@ program(dp, rp, user, timestring)
 
 	time(&endtime);
 	notary_setxdelay((int)(endtime-starttime));
-	notaryreport(NULL,NULL,NULL,NULL); /* XX: proper values ? */
+	if (i == EX_OK) {
+	  notaryreport(NULL,"delivery",
+		       "2.2.0 (Delivered successfully)",
+		       "x-local; 250 (Delivered successfully)");
+	} else {
+	  char buf2[sizeof(buf)+10];
+	  sprintf(buf2,"x-local; 500 (%s)", buf);
+	  notaryreport(NULL,"failed",
+		       "5.3.0 (subprogram non-zero termination code)",
+		       buf2);
+	}
+	
 	DIAGNOSTIC(rp, user, i, "%s", buf);
 	return;
 }
@@ -1966,7 +2014,7 @@ creatembox(rp, uname, filep, uid, gid, pw)
 	    }
 	  }
 	  strcat(s, uname);
-	  fd = createfile(rp, *filep, *uid);
+	  fd = createfile(rp, *filep, *uid, 1);
 	  if (fd >= 0) {
 #ifdef	HAVE_FCHOWN
 	    fchown(fd, *uid, *gid);
@@ -2013,18 +2061,29 @@ creatembox(rp, uname, filep, uid, gid, pw)
 }
 
 int
-createfile(rp, file, iuid)
+createfile(rp, file, iuid, ismbox)
 	struct rcpt *rp;
 	const char *file;
-	int iuid;
+	int iuid, ismbox;
 {
 	int fd, i = 0, saverrno;
 	struct stat st;
 	char *cp, msg[BUFSIZ];
 	uid_t uid = iuid;
 
-	if ((fd = open(file, O_RDWR|O_CREAT|O_EXCL, MAILMODE)) < 0) {
+	if (verboselog)
+	  fprintf(verboselog,
+		  "To create a file with euid=%d egid=%d file='%s'\n",
+		  geteuid(), getegid(), file);
+
+	fd = open(file, O_RDWR|O_CREAT|O_EXCL, MAILMODE);
+	if (fd < 0) {
 	  saverrno = errno;
+	  if (verboselog)
+	    fprintf(verboselog,
+		    " ... failed, errno = %d (%s)\n",
+		    saverrno, strerror(saverrno));
+	  
 	  if (errno == EEXIST)
 	    return -3;
 	  if ((cp = strrchr(file, '/')) != NULL) {
@@ -2040,7 +2099,7 @@ createfile(rp, file, iuid)
 	      return -2;
 	    }
 	    *cp = '/';
-	    if (st.st_mode & 020) { /* group writable? */
+	    if (ismbox && st.st_mode & 020) { /* group writable? */
 	      if (!setupuidgid(rp, uid, st.st_gid)) {
 		notaryreport(rp->addr->user, "failed",
 			     "5.3.5 (The mailbox configuration is faulty)",
@@ -2052,8 +2111,19 @@ createfile(rp, file, iuid)
 	      }
 	    }
 	  }
-	  if ((fd = open(file, O_RDWR|O_CREAT|O_EXCL, MAILMODE)) < 0) {
+
+	  if (verboselog)
+	    fprintf(verboselog,
+		    "To create a file with euid=%d egid=%d file='%s'\n",
+		    geteuid(), getegid(), file);
+
+	  fd = open(file, O_RDWR|O_CREAT|O_EXCL, MAILMODE);
+	  if (fd < 0) {
 	    saverrno = errno;
+	    if (verboselog)
+	      fprintf(verboselog,
+		      " ... failed, errno = %d (%s)\n",
+		      saverrno, strerror(saverrno));
 	    setrootuid(rp);
 	    errno = saverrno;
 	    if (errno == EEXIST)
@@ -2095,6 +2165,14 @@ setupuidgid(rp, uid, gid)
 	int gid;
 {
 	setrootuid(rp);
+
+	if (gid == -3) {
+	  /* MAGIC! Ask GID of the UID of 'uid' */
+	  struct passwd *pw = getpwuid(uid);
+	  if (pw != NULL)
+	    gid = pw->pw_gid;
+	}
+	
 	if (gid >= 0)
 	  if (setgid(gid) < 0) {
 	    DIAGNOSTIC(rp, "", EX_OSERR, "can't setgid to %d", (int)gid);
@@ -2183,9 +2261,7 @@ appendlet(dp, rp, fp, file, ismime)
 	struct writestate WS;
 	const char *s; /* message body start */
 
-#ifndef	HAVE_MMAP
-	char buffer[BUFSIZ+1];
-	const char *bufend = buffer+BUFSIZ;
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	register int i;
 	register int bufferfull;
 	char iobuf[BUFSIZ]; /* For setvbuf(mfp) */
@@ -2200,14 +2276,14 @@ appendlet(dp, rp, fp, file, ismime)
 	WS.frombuf[0] = 0;
 	WS.fromp = WS.frombuf;
 
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 
 	if (ismime) {
 	  /* can we use cache of message body data ? */
 	  /* Split it to lines.. */
 	  if (readalready > 0) {
 	    int linelen, readidx = 0;
-	    const char *s0 = buffer, *s;
+	    const char *s0 = let_buffer, *s;
 	    while (readidx < readalready) {
 	      s = s0;
 	      linelen = 0;
@@ -2239,13 +2315,14 @@ appendlet(dp, rp, fp, file, ismime)
 	  /* We really can't use the 'let_buffer' cache here */
 	  readalready = 0;
 	  i = 0;
-	  while ((i = cfgets(buffer, BUFSIZ, mfp)) != EOF) {
+	  while ((i = cfgets(let_buffer, sizeof(let_buffer), mfp)) != EOF) {
 	    /* It MAY be malformed -- if it has a BUFSIZ length
 	       line in it, IT CAN'T BE MIME  :-/		*/
-	    if (i == BUFSIZ && buffer[BUFSIZ-1] != '\n')
+	    if (i == sizeof(let_buffer) &&
+		let_buffer[sizeof(let_buffer)-1] != '\n')
 	      ismime = 0;
 	    /* Ok, write the line */
-	    if (writemimeline(&WS, buffer, i) != i) {
+	    if (writemimeline(&WS, let_buffer, i) != i) {
 	      DIAGNOSTIC(rp, file, EX_IOERR, "write to \"%s\" failed", file);
  	      MFPCLOSE;
 	      return -256;
@@ -2263,7 +2340,7 @@ appendlet(dp, rp, fp, file, ismime)
 
 	  /* can we use cache of message body data ? */
 	  if (readalready != 0) {
-	    if (writebuf(&WS, buffer, readalready) != readalready) {
+	    if (writebuf(&WS, let_buffer, readalready) != readalready) {
 	      DIAGNOSTIC(rp, file, EX_IOERR,
 			 "write to \"%s\" failed", file);
 	      return -256;
@@ -2277,7 +2354,7 @@ appendlet(dp, rp, fp, file, ismime)
 	  lseek(mfd, dp->msgbodyoffset, SEEK_SET);
 	  bufferfull = 0;
 	  while (1) {
-	    i = read(mfd, buffer, BUFSIZ);
+	    i = read(mfd, let_buffer, sizeof(let_buffer));
 	    if (i == 0)
 	      break;
 	    if (i < 0) {
@@ -2286,7 +2363,7 @@ appendlet(dp, rp, fp, file, ismime)
 	      readalready = 0;
 	      return -256;
 	    }
-	    if (writebuf(&WS, buffer, i) != i) {
+	    if (writebuf(&WS, let_buffer, i) != i) {
 	      DIAGNOSTIC(rp, file, EX_IOERR, "write to \"%s\" failed", file);
 	      readalready = 0;
 	      return -256;
@@ -2729,7 +2806,6 @@ const char *str;
 	}
 }
 
-static void decodeXtext __((FILE *, const char *));
 static void
 decodeXtext(fp,xtext)
 	FILE *fp;
@@ -2758,7 +2834,7 @@ return_receipt (dp, retrecptaddr, uidstr)
 	FILE *mfp, *efp;
 	struct rcpt *rp;
 	int n;
-	char *boundarystr = buildboundary("return-receipt");
+	char boundarystr[400];
 	struct passwd *pw;
 	int uid;
 	struct stat stb;
@@ -2811,6 +2887,19 @@ return_receipt (dp, retrecptaddr, uidstr)
 
 
 	efp = fopen(mfpath, "r");
+
+	{
+	  char *dom = mydomain(); /* transports/libta/buildbndry.c */
+	  char fname[20];
+	  struct stat stbuf;
+
+	  fstat(FILENO(mfp),&stbuf);
+	  sprintf(fname,"%d",stbuf.st_ino);
+	  taspoolid(boundarystr, sizeof(boundarystr), stbuf.st_ctime, fname);
+	  strcat(boundarystr, "=_/return-receipt/");
+	  strcat(boundarystr, dom);
+	}
+
 	if (efp != NULL) {
 	  int inhdr = 1;
 	  buf[sizeof(buf)-1] = 0;
@@ -2835,7 +2924,7 @@ return_receipt (dp, retrecptaddr, uidstr)
 	} else {
 	  for (cpp = dfltform; *cpp != NULL; ++cpp)
 	    if (*cpp[0] == 0) {
-	      fprintf(mfp, "\tboundary=\"%s\"\n\n\n",boundarystr);
+	      fprintf(mfp, "\tboundary=\"%s\"\n\n", boundarystr);
 	      fprintf(mfp, "--%s\n", boundarystr);
 	      fprintf(mfp, "Content-Type: text/plain\n\n");
 	    } else
@@ -2870,9 +2959,9 @@ return_receipt (dp, retrecptaddr, uidstr)
 	    fprintf(mfp, "\n");
 	  }
 	  fprintf(mfp, "Final-Recipient: X-LOCAL;%s\n", rp->addr->user);
-	  fprintf(mfp, "Action: delivered\n");
-	  fprintf(mfp, "Status: 2.5.0\n");
-	  fprintf(mfp, "Diagnostic-Code: smtp;250 (%s delivered)\n", rp->addr->host );
+	  fprintf(mfp, "Action: delivery\n");
+	  fprintf(mfp, "Status: 2.2.0\n");
+	  fprintf(mfp, "Diagnostic-Code: smtp; 250 ('%s' delivered)\n", rp->addr->user );
 	  fprintf(mfp, "\n");
 	}
 
@@ -2882,6 +2971,7 @@ return_receipt (dp, retrecptaddr, uidstr)
 	rp = dp->recipients;
 	/* write the (new) headers with local "Received:"-line.. */
 	writeheaders(rp,mfp,"\n",0,0);
+	fprintf(mfp, "\n");
 
 	fprintf(mfp, "--%s--\n", boundarystr);
 	if (ferror(mfp)) {
