@@ -1,7 +1,7 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
- *	Copyright 1994 by Matti Aarnio -- MIME processings
+ *	Copyright 1994-1997 by Matti Aarnio -- MIME processings
  */
 
 #define DefCharset "ISO-8859-1"
@@ -22,6 +22,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <errno.h>
 
 #ifdef  HAVE_WAITPID
 # include <sys/wait.h>
@@ -60,7 +61,7 @@ extern int wait();
 char uucpname[MAXHOSTNAMELEN+1];
 
 char	*progname;
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 int	readalready = 0;	/* does buffer contain valid message data? */
 #endif
 int	mimeqpnarrow = 0;	/* Can't send TAB thru without MIME-QP */
@@ -127,7 +128,8 @@ struct exmapinfo {
 	int	newstatus;
 	const char *dsnstatus;
 	const char *dsndiags;
-} exmap[] = {
+};
+struct exmapinfo exmap[] = {
   { EX_USAGE,	"command line usage error",	EX_TEMPFAIL,	"5.3.0", "x-local; 500 (Command line usage error)"	},
   { EX_DATAERR,	"data format error",		EX_DATAERR,	"5.3.0", "x-local; 500 (Data format error)"	},
   { EX_NOINPUT,	"cannot open input",		EX_TEMPFAIL,	"5.3.0", "x-local; 530 (Cannot open input)"	},
@@ -310,7 +312,7 @@ process(dp, mp, verboselog)
 {
 	struct rcpt *rp, *rphead;
 
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	readalready = 0; /* ignore any previous message data cache */
 #endif
 
@@ -459,9 +461,21 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  }
 	  return;
 	}
+	if (verboselog) {
+	  const char **p = av;
+	  fprintf(verboselog,"To run UID=%d GID=%d ARGV[] =",
+		  getuid(), getgid());
+	  for ( ;*p != NULL; ++p) {
+	    fprintf(verboselog," '%s'", *p);
+	  }
+	  fprintf(verboselog,"\n");
+	  fflush(verboselog);
+	}
 	if ((pid = fork()) == 0) { /* child, run the command */
-	  if (!(mp->flags & MO_NORESETUID))
+	  if (!(mp->flags & MO_NORESETUID)) {
+	    /* struct passwd *pw = getpwuid(); */
 	    setuid(getuid());
+	  }
 	  if (in[1] > 0) {
 	    /* its stdout and stderr is the pipe, its stdin is our tafp */
 	    if (out[0] > 0) dup2(out[0], 0);
@@ -643,13 +657,18 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	    uu = "";
 	  append_header(startrp,"Return-Path: <%s>", uu);
 	}
-	if (mp->flags & MO_CRLF)
+	if (mp->flags & MO_CRLF) {
 	  writeheaders(startrp, tafp, "\r\n", convertmode, maxwidth);
-	else
+	  fprintf(tafp, "\r\n");
+	} else {
 	  writeheaders(startrp, tafp, "\n",   convertmode, maxwidth);
+	  fprintf(tafp, "\n");
+	}
 
-	if (verboselog)
+	if (verboselog) {
 	  writeheaders(startrp, verboselog, "\n", convertmode, maxwidth);
+	  fprintf(verboselog, "\n");
+	}
 
 	/* append message body itself */
 	if ((i = appendlet(dp, mp, tafp, verboselog, convertmode)) != EX_OK) {
@@ -706,7 +725,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	} else {
 	  i = WEXITSTATUS(status);
 	  s = NULL;
-	  for (exp = exmap; exp->origstatus != 0; ++exp)
+	  for (exp = & exmap[0]; exp->origstatus != 0; ++exp)
 	    if (exp->origstatus == i) {
 	      s = exp->statusmsg;
 	      i = exp->newstatus;
@@ -714,7 +733,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	      exd = exp->dsndiags;
 	      break;
 	    }
-	  sprintf(cp, "[exit status %d", WEXITSTATUS(status));
+	  sprintf(cp, "[exit status %d/%d", WEXITSTATUS(status), i);
 	  if (s)
 	    sprintf(cp+strlen(cp), " (%s)", s);
 	  /* sprintf(cp+strlen(cp), " of command: %s", mp->command); */
@@ -737,7 +756,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
  * appendlet - append letter to file pointed at by fd
  */
 
-#ifndef	HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 static char let_buffer[BUFSIZ*8];
 #endif
 
@@ -758,14 +777,14 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 
 	register int i;
 	int lastch;
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	register int bufferfull;
 	int mfd = dp->msgfd;
 #endif
 
 	writebuf(mp, fp, (char *)NULL, 0);  /* magic initialization */
 
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	/* can we use cache of message body data */
 	if (convertmode == _CONVERT_NONE && readalready != 0) {
 	  lastch = let_buffer[readalready-1];
@@ -780,7 +799,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 
 	lastch = -1;
 	if (convertmode == _CONVERT_NONE) {
-#ifndef	HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	  bufferfull = 0;
 	  readalready = 0;
 	  lseek(mfd, dp->msgbodyoffset, SEEK_SET);
@@ -794,7 +813,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	    lastch = let_buffer[i-1];
 	    if (writebuf(mp, fp, let_buffer, i) != i)
 	      return EX_IOERR;
-#ifndef	HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	    readalready = i;
 	    bufferfull++;
 	  }
@@ -807,7 +826,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  /* Various esoteric conversion modes..
 	     We are better to feed writemimeline() with LINES
 	     instead of blocks of data.. */
-#ifndef	HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	  char iobuf[BUFSIZ];
 	  FILE *mfp = fdopen(mfd,"r");
 
@@ -837,7 +856,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  i = 0;
 
 	  for (;;) {
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	    if ((i = cfgets(let_buffer, sizeof(let_buffer), mfp)) == EOF)
 	      break;
 #else
@@ -857,7 +876,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	      return EX_IOERR;
 	    }
 	  }
-#ifndef HAVE_MMAP
+#if !(defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	  if (i == EOF && !feof(mfp)) {
 	    MFPCLOSE
 	    return EX_IOERR;
@@ -1237,7 +1256,7 @@ int
 check_7bit_cleanness(dp)
 struct ctldesc *dp;
 {
-#ifdef	HAVE_MMAP
+#if (defined(HAVE_MMAP) && defined(TA_USE_MMAP))
 	/* With MMAP()ed spool file it is sweet and simple.. */
 	register const char *s = dp->let_buffer + dp->msgbodyoffset;
 	while (s < dp->let_end)
